//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "MPlayerX.h"
#include "ChildView.h"
#include "MainFrm.h"
#include <comdef.h>
#include ".\childview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

_COM_SMARTPTR_TYPEDEF(IDDrawExclModeVideo, __uuidof(IDDrawExclModeVideo));

/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView()
{
	m_fViewTimeLine = FALSE;

	m_szText = _T("");
	m_szTime = _T("");
	m_dTime = 0.0;
	m_nOSDElementShown = 0;

	m_dwShowTextTimer = 0;
	m_fEraseBkgnd = TRUE;
	m_BkgndBrush.CreateSolidBrush(RGB(0, 0, 0));
}

CChildView::~CChildView()
{
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_WM_ACTIVATE()
	ON_WM_ACTIVATEAPP()
	ON_WM_DESTROY()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_NCACTIVATE()
	ON_WM_RBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_SYSCOMMAND()
	ON_WM_SYSKEYDOWN()
	//}}AFX_MSG_MAP
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEWHEEL()
	ON_WM_MBUTTONUP()
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(m_BkgndBrush), NULL);

	return TRUE;
}

void CChildView::OnPaint() 
{
	//TRACE("CChildView::OnPaint\n");
	CPaintDC dc(this); // device context for painting

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		if(!pMainFrame->m_fDesktopMode)
		{
			CPoint pointOffset(0, 0);
			if(pMainFrame->m_fOverlayUsed)
			{
				CComPtr<IBaseFilter> pVideoRender = NULL;
				if(pMainFrame->m_pGraphBuilder != NULL &&
					SUCCEEDED(pMainFrame->m_pGraphBuilder->FindFilterByName(L"Overlay Mixer", &pVideoRender)))
				{
					CComPtr<IDDrawExclModeVideo> pDDrawExclModeVideo = NULL;
					if(SUCCEEDED(pVideoRender->QueryInterface(IID_IDDrawExclModeVideo, (LPVOID *)&pDDrawExclModeVideo)))
					{
						BOOL fExt = FALSE;
						CComPtr<IDirectDrawSurface> pDirectDrawSurface;
						if(SUCCEEDED(pDDrawExclModeVideo->GetDDrawSurface(&pDirectDrawSurface, &fExt)) && pDirectDrawSurface != NULL)
						{
							HDC hdc = NULL;
							pDirectDrawSurface->GetDC(&hdc);
							if(hdc)
							{
								if(m_hWnd) ClientToScreen(&pointOffset);
								ClipHDC(hdc);
								PaintOSD(hdc, pointOffset);
								PaintSubtitle(hdc, pointOffset);
								PaintTimeLine(hdc, pointOffset);
								pDirectDrawSurface->ReleaseDC(hdc);
							}
							//pDirectDrawSurface->Release();
							pDirectDrawSurface = NULL;
						}
						pDDrawExclModeVideo = NULL;
					}
					pVideoRender = NULL;
				}
			}
		}
	}
}


BOOL CChildView::OnEraseBkgnd(CDC* pDC) 
{
	if(m_fEraseBkgnd)
		return CWnd::OnEraseBkgnd(pDC);
	else
		return TRUE;
}

void CChildView::OnSize(UINT nType, int cx, int cy) 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
		pMainFrame->UpdateDestinationView(cx, cy);

	CWnd ::OnSize(nType, cx, cy);
}

LRESULT CChildView::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if(message == WM_GRAPHNOTIFY)
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame)
			pMainFrame->MediaEvent();
		return S_OK;
	}
	
	if(message == WM_PAINT_SUBTITLE)
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame && !pMainFrame->m_fDesktopMode)
		{
			if(pMainFrame->m_SettingsCurrent.m_fShowOSD)
				Invalidate();
		}
	}

	return CWnd ::WindowProc(message, wParam, lParam);
}

int CChildView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	return 0;
}

void CChildView::PaintOSD(HDC hDC, CPoint pointOffset)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && hDC && !pMainFrame->m_fDesktopMode && pMainFrame->m_SettingsCurrent.m_fShowOSD)
	{
		CDC dc;
		dc.Attach(hDC);

		CRect rect;
		GetClientRect(&rect);

		if(m_nOSDElementShown & OSD_TEXT)
		{
			CFont * pFontDC = dc.SelectObject(&pMainFrame->m_SettingsCurrent.m_fontOSD);
			dc.SetTextColor(pMainFrame->m_SettingsCurrent.m_colorOSDFont);
			if(pMainFrame->m_SettingsCurrent.m_fOSDTransparent && !pMainFrame->m_fOverlayUsed)
				dc.SetBkMode(TRANSPARENT);
			else if(pMainFrame->m_fOverlayUsed)
				dc.SetBkColor(pMainFrame->GetColorKey());
			else
				dc.SetBkColor(pMainFrame->m_SettingsCurrent.m_colorOSDBackgroundFont);

			CPoint point;
			point.x = (pMainFrame->m_SettingsCurrent.m_nOSDPosX >=0 ? pMainFrame->m_SettingsCurrent.m_nOSDPosX : rect.right + pMainFrame->m_SettingsCurrent.m_nOSDPosX);
			point.y = (pMainFrame->m_SettingsCurrent.m_nOSDPosY >=0 ? pMainFrame->m_SettingsCurrent.m_nOSDPosY : rect.bottom + pMainFrame->m_SettingsCurrent.m_nOSDPosY);

			point += pointOffset;

			dc.TextOut(point.x, point.y, m_szText);

			dc.SelectObject(pFontDC);
		}

		dc.Detach();
	}
}

void CChildView::PaintTimeLine(HDC hDC, CPoint pointOffset)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && hDC && !pMainFrame->m_fDesktopMode && pMainFrame->m_SettingsCurrent.m_fShowOSD)
	{
		CDC dc;
		dc.Attach(hDC);

		CRect rect;
		GetClientRect(&rect);

		if(m_nOSDElementShown & OSD_TIME_LINE)
		{
			CFont * pFontDC = dc.SelectObject(&pMainFrame->m_SettingsCurrent.m_fontTimeLine);
			dc.SetTextColor(pMainFrame->m_SettingsCurrent.m_colorTimeLineFont);
			if(pMainFrame->m_SettingsCurrent.m_fTimeLineTransparent && !pMainFrame->m_fOverlayUsed)
				dc.SetBkMode(TRANSPARENT);
			else if(pMainFrame->m_fOverlayUsed)
				dc.SetBkColor(pMainFrame->GetColorKey());
			else
				dc.SetBkColor(pMainFrame->m_SettingsCurrent.m_colorTimeLineBackgroundFont);

			CRect rect1, rect2, rect3;
			rect1.top = (long)((double)rect.Height() * (double)pMainFrame->m_SettingsCurrent.m_nTimeLinePosY / 100.0);
			rect1.bottom = rect1.top + 20;
			rect1.left = rect.left + 10;
			rect1.right = rect.right - 10;
			rect2 = rect1;
			rect2.right = rect1.left + (int)(rect1.Width() * m_dTime);
			rect3 = rect1;
			rect3.left = rect2.right;

			CBrush BrushForeGround, BrushBackGround;
			BrushForeGround.CreateSolidBrush(pMainFrame->m_SettingsCurrent.m_colorTimeLineFont);
			BrushBackGround.CreateSolidBrush(pMainFrame->m_fOverlayUsed ?
									pMainFrame->m_dwColorKeyRGB :
									pMainFrame->m_SettingsCurrent.m_colorTimeLineBackgroundFont);

			CPoint point;
			point.x = 0;
			point.y = rect1.bottom + 10;
			point += pointOffset;
			dc.TextOut(point.x, point.y, m_szTime);

			rect1 += pointOffset;
			rect2 += pointOffset;
			rect3 += pointOffset;

			dc.FillRect(&rect2, &BrushForeGround);
			dc.FillRect(&rect3, &BrushBackGround);
			dc.FrameRect(&rect1, &BrushForeGround);

			CPoint MousePoint;
			GetCursorPos(&MousePoint);
			if(MousePoint.x < rect1.left)
				MousePoint.x = rect1.left;
			if(MousePoint.x > rect1.right)
				MousePoint.x = rect1.right;

			CPoint TrianglePoints[4];
			TrianglePoints[0].x = MousePoint.x;
			TrianglePoints[0].y = rect1.top;
			TrianglePoints[1].x = TrianglePoints[0].x - 5;
			TrianglePoints[1].y = TrianglePoints[0].y + 5;
			TrianglePoints[2].x = TrianglePoints[0].x + 5;
			TrianglePoints[2].y = TrianglePoints[0].y + 5;

			dc.Polygon(TrianglePoints, 3);

			TrianglePoints[0].x = MousePoint.x;
			TrianglePoints[0].y = rect1.bottom - 1;
			TrianglePoints[1].x = TrianglePoints[0].x - 5;
			TrianglePoints[1].y = TrianglePoints[0].y - 5;
			TrianglePoints[2].x = TrianglePoints[0].x + 5;
			TrianglePoints[2].y = TrianglePoints[0].y - 5;

			dc.Polygon(TrianglePoints, 3);

			rect3 = rect1;
			rect3.left = 0;
			rect3.right = rect1.left;
			dc.FillRect(&rect3, &BrushBackGround);
			rect3.left = rect1.right;
			rect3.right = rect.right + pointOffset.x;
			dc.FillRect(&rect3, &BrushBackGround);

			dc.SelectObject(pFontDC);
		}

		dc.Detach();
	}
}

void CChildView::ShowTimeLine(BOOL fViewTimeLine, LPCTSTR szTime, double dTime)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fDesktopMode)
	{
		if(!(m_nOSDElementShown & OSD_TIME_LINE) && m_fViewTimeLine == fViewTimeLine)
			return;

		if(fViewTimeLine)
			m_nOSDElementShown |= OSD_TIME_LINE;
		else
			m_nOSDElementShown &= ~OSD_TIME_LINE;

		m_szTime = szTime;
		m_dTime = dTime;

		if(!fViewTimeLine)
			Invalidate();
		else
		{
			if(pMainFrame->m_fOverlayUsed)
			{
				IBaseFilter * pVideoRender = NULL;
				if(pMainFrame->m_pGraphBuilder != NULL &&
					SUCCEEDED(pMainFrame->m_pGraphBuilder->FindFilterByName(L"Overlay Mixer", &pVideoRender)) && pVideoRender)
				{
					IDDrawExclModeVideoPtr pDDrawExclModeVideo = pVideoRender;
					if(pDDrawExclModeVideo)
					{
						BOOL fExt = FALSE;
						IDirectDrawSurface * pDirectDrawSurface;
						if(SUCCEEDED(pDDrawExclModeVideo->GetDDrawSurface(&pDirectDrawSurface, &fExt)) && pDirectDrawSurface)
						{
							HDC hdc = NULL;
							pDirectDrawSurface->GetDC(&hdc);
							if(hdc)
							{
								ClipHDC(hdc);
								CRect rect;
								GetWindowRect(&rect);
								CPoint pointOffset(rect.TopLeft());
								PaintOSD(hdc, pointOffset);
								PaintSubtitle(hdc, pointOffset);
								PaintTimeLine(hdc, pointOffset);
								pDirectDrawSurface->ReleaseDC(hdc);
							}
							pDirectDrawSurface->Release();
						}
					}
				}
			}
		}
	}
}

void CChildView::ShowText(LPCTSTR pszText, UINT time)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fDesktopMode)
	{
		if(pszText && pszText[0] != 0)
			m_nOSDElementShown |= OSD_TEXT;
		else
			m_nOSDElementShown &= ~OSD_TEXT;

		m_szText = pszText;
		m_szText += _T("                           ");

		if(pMainFrame->m_fOverlayUsed)
		{
			IBaseFilter * pVideoRender = NULL;
			if(pMainFrame->m_pGraphBuilder != NULL &&
				SUCCEEDED(pMainFrame->m_pGraphBuilder->FindFilterByName(L"Overlay Mixer", &pVideoRender)) && pVideoRender)
			{
				IDDrawExclModeVideoPtr pDDrawExclModeVideo = pVideoRender;
				if(pDDrawExclModeVideo)
				{
					BOOL fExt = FALSE;
					IDirectDrawSurface * pDirectDrawSurface;
					if(SUCCEEDED(pDDrawExclModeVideo->GetDDrawSurface(&pDirectDrawSurface, &fExt)) && pDirectDrawSurface)
					{
						HDC hdc = NULL;
						pDirectDrawSurface->GetDC(&hdc);
						if(hdc)
						{
							ClipHDC(hdc);
							CRect rect;
							GetWindowRect(&rect);
							CPoint pointOffset(rect.TopLeft());
							PaintOSD(hdc, pointOffset);
							PaintSubtitle(hdc, pointOffset);
							PaintTimeLine(hdc, pointOffset);
							pDirectDrawSurface->ReleaseDC(hdc);
						}
						pDirectDrawSurface->Release();
					}
				}
			}
		}

		if(m_dwShowTextTimer)
			KillTimer(m_dwShowTextTimer);

		if(time)
			m_dwShowTextTimer = SetTimer(215, time, NULL);
	}
}

void CChildView::PaintSubtitle(HDC hdc, CPoint pointOffset, LPARAM lParam)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && hdc && !pMainFrame->m_fDesktopMode)
	{
		CDC dc;
		dc.Attach(hdc);
		
		CFont * pFontDC = dc.SelectObject(&pMainFrame->m_SettingsCurrent.m_fontSubtitle);
		dc.SetTextColor(pMainFrame->m_SettingsCurrent.m_colorSubtitleFont);
		if(pMainFrame->m_SettingsCurrent.m_fSubtitleTransparent && !pMainFrame->m_fOverlayUsed)
			dc.SetBkMode(TRANSPARENT);
		else if(pMainFrame->m_fOverlayUsed)
			dc.SetBkColor(pMainFrame->GetColorKey());
		else
			dc.SetBkColor(pMainFrame->m_SettingsCurrent.m_colorSubtitleBackgroundFont);

		int nCount = pMainFrame->m_SubtitleManager.GetSubtitleCount();
		for(int i = 0; i < nCount; i++)
		{
			CSubtitle * pSubtitle = pMainFrame->m_SubtitleManager.GetSubtitle(i);
			if(pSubtitle)
			{
				if((lParam && (CSubtitle *)lParam == pSubtitle) || !lParam)
				{
					CRect rect, rect2;
					GetClientRect(&rect);
					rect2 = rect;
					rect2.left = rect2.left + (long)(rect.Width() * 0.02);
					rect2.right = rect2.right - (long)(rect.Width() * 0.02);
					rect2.top = rect2.top;
					rect2.bottom = rect2.bottom + (long)(rect.Height() * 0.30);
					rect2 += pointOffset;
					rect2 += CPoint(0,
						(long)((double)rect.Height() * (double)pMainFrame->m_SettingsCurrent.m_nSubtitlePosY / 100.0));

					CBrush BrushBackGround;
					BrushBackGround.CreateSolidBrush(dc.GetBkColor());
					FillRect(hdc, &rect2, (HBRUSH)BrushBackGround);

					CString szSubtitle;
					if(pSubtitle->GetCurrentSutitle(szSubtitle))
						dc.DrawText(szSubtitle, &rect2, DT_CENTER | DT_LEFT | DT_TOP);

					if(lParam && (CSubtitle *)lParam == pSubtitle)
						break;
				}
			}
		}
		dc.SelectObject(pFontDC);
		dc.Detach();
	}
}

void CChildView::ClipHDC(HDC hdc)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fDesktopMode)
	{
		CRect rectClip;
		GetWindowRect(&rectClip);
		CRgn region;
		region.CreateRectRgn(rectClip.left, rectClip.top, rectClip.right, rectClip.bottom);
		
		HWND hPrevTopWnd = pMainFrame->m_hWnd;
		while(hPrevTopWnd = ::GetNextWindow(hPrevTopWnd, GW_HWNDPREV))
		{
			if(::IsWindowVisible(hPrevTopWnd))
			{
				CRgn regionTop;
				CRect rectTop;
				::GetWindowRect(hPrevTopWnd, &rectTop);
				regionTop.CreateRectRgn(rectTop.left, rectTop.top, rectTop.right, rectTop.bottom);
				region.CombineRgn(&region, &regionTop, RGN_DIFF);
			}
		}
		::SelectClipRgn(hdc, (HRGN)region);
	}
}

void CChildView::OnTimer(UINT nIDEvent) 
{
	KillTimer(m_dwShowTextTimer);
	m_dwShowTextTimer = 0;

	m_szText += _T("");
	m_nOSDElementShown &= ~OSD_TEXT;

	Invalidate();
	
	CWnd::OnTimer(nIDEvent);
}

void CChildView::DestroyMainWnd() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_fScreenSaverMode)
		pMainFrame->DestroyWindow();
}
void CChildView::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	TRACE(_T("CChildView::OnActivate WM_ACTIVATE WM_ACTIVATE WM_ACTIVATE WM_ACTIVATE\n"));
//	CWnd::OnActivate(nState, pWndOther, bMinimized);

	if(nState == WA_INACTIVE)
		DestroyMainWnd();
}

void CChildView::OnActivateApp(BOOL bActive, DWORD dwThreadID) 
{
	TRACE(_T("CChildView::OnActivateApp WM_ACTIVATEAPP WM_ACTIVATEAPP WM_ACTIVATEAPP WM_ACTIVATEAPP\n"));
//	CWnd::OnActivateApp(bActive, hTask);
	
	if(!bActive)
		DestroyMainWnd();
}

void CChildView::OnDestroy() 
{
	CWnd::OnDestroy();
	
	// TODO: Add your message handler code here
	
}

void CChildView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);

	DestroyMainWnd();
}

void CChildView::AdjustTimeLine(UINT nFlags, CPoint point) 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fDesktopMode && pMainFrame->m_pMediaSeeking != NULL)
	{
		CRect rect;
		GetClientRect(&rect);

		TRACE(_T("point.x = %d, rect.Width() = %d (%.2f%%)\n"), point.x - 10, rect.Width() - 20, 100.0 * (double)(point.x - 10) / (double)(rect.Width() - 20));

		LONGLONG llPosition = pMainFrame->m_MediaTime.m_llDuration * (point.x - 10) / (rect.Width() - 20);
		if(llPosition < 0) llPosition = 0;
		if(llPosition > pMainFrame->m_MediaTime.m_llDuration) llPosition = pMainFrame->m_MediaTime.m_llDuration;
		pMainFrame->m_pMediaSeeking->SetPositions(&llPosition, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
//		Invalidate();
		OnPaint();
	}
}

void CChildView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CWnd::OnLButtonDblClk(nFlags, point);

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fScreenSaverMode)
	{
		if(m_nOSDElementShown & OSD_TIME_LINE)
			AdjustTimeLine(nFlags, point);
		else
			pMainFrame->SendMessage(WM_COMMAND, IDM_VIEW_FULLSCREEN);
	}
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point)
{
	CWnd::OnLButtonUp(nFlags, point);

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fScreenSaverMode)
	{
		if(m_nOSDElementShown & OSD_TIME_LINE)
			AdjustTimeLine(nFlags, point);
		else
			pMainFrame->SendMessage(WM_COMMAND, IDM_CONTROL_PLAY_PAUSE);
	}
}

void CChildView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CWnd::OnLButtonDown(nFlags, point);

	DestroyMainWnd();
}

void CChildView::OnMButtonDown(UINT nFlags, CPoint point) 
{
	CWnd::OnMButtonDown(nFlags, point);

	m_fWheelRotated = FALSE;

	DestroyMainWnd();
}

void CChildView::OnMButtonUp(UINT nFlags, CPoint point)
{
	if(!m_fWheelRotated)
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame && !pMainFrame->m_fScreenSaverMode)
			pMainFrame->SendMessage(WM_COMMAND, IDM_VIEW_TIMELINE);
	}

	CWnd::OnMButtonUp(nFlags, point);
}

void CChildView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CWnd::OnMouseMove(nFlags, point);

	DestroyMainWnd();

	if((m_nOSDElementShown & OSD_TIME_LINE) && (nFlags & MK_LBUTTON))
		AdjustTimeLine(nFlags, point);
}

BOOL CChildView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	m_fWheelRotated = TRUE;

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fScreenSaverMode)
	{
		if(nFlags & MK_MBUTTON)
		{
			if(pMainFrame->m_fPlayListMode && pMainFrame->m_wndPlayListDlgBar.GetCount() < 2)
				pMainFrame->SendMessage(WM_COMMAND, ID_FILEBROWSER_SYNCHRONISE_CURRENT_FILE);
			
			if(pMainFrame->m_fPlayListMode)
				return pMainFrame->SendMessage(WM_COMMAND, zDelta > 0 ? IDM_CONTROL_NEXT_FILE : IDM_CONTROL_PREVIOUS_FILE);
			else
				return pMainFrame->SendMessage(WM_COMMAND, zDelta > 0 ? IDM_CONTROL_NEXT_FOLDER_FILE : IDM_CONTROL_PREVIOUS_FOLDER_FILE);
		}
		else
			return pMainFrame->SendMessage(WM_COMMAND, zDelta > 0 ? IDM_VIEW_VOLUME_PLUS : IDM_VIEW_VOLUME_MOINS);
	}

	DestroyMainWnd();

	return CWnd::OnMouseWheel(nFlags, zDelta, pt);
}

BOOL CChildView::OnNcActivate(BOOL bActive) 
{
	return CWnd::OnNcActivate(bActive);

	if(!bActive)
		DestroyMainWnd();
}

void CChildView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CWnd::OnRButtonDown(nFlags, point);

	DestroyMainWnd();

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && !pMainFrame->m_fScreenSaverMode)
	{
		CPoint pt = point;
		ClientToScreen(&pt);
		pMainFrame->ShowMenu(pt);
	}
}

BOOL CChildView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_fScreenSaverMode)
	{
		SetCursor(NULL);
		return TRUE;
	}

	return CWnd ::OnSetCursor(pWnd, nHitTest, message);
}

void CChildView::OnSysCommand(UINT nID, LPARAM lParam) 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_fScreenSaverMode &&
		((nID == SC_SCREENSAVE) || (nID == SC_CLOSE)))
		return;

	CWnd::OnSysCommand(nID, lParam);
}

void CChildView::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);

	DestroyMainWnd();
}
