//    NetExplorer - Windows LAN explorer application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// LeftView.cpp : implementation of the CLeftView class
//

#include "stdafx.h"
#include "MPlayerX.h"

#include "MltiTree.h"	//actual tree ctrl
#include "NetExplorerDoc.h"
#include "LeftView.h"
#include "MainFrm.h"
#include "leftview.h"
#include "NetFind.h"
#include "RecentFileListEx.h"
#include "DragDropImpl.h"
#include ".\leftview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CMap<CString, LPCTSTR, CTreeNode *, CTreeNode *> CLeftView::m_mapTreeNodeEnum;

/////////////////////////////////////////////////////////////////////////////
// CLeftView

IMPLEMENT_DYNCREATE(CLeftView, CTreeView)

BEGIN_MESSAGE_MAP(CLeftView, CTreeView)
	//{{AFX_MSG_MAP(CLeftView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_ITEMEXPANDING, OnItemexpanding)
	ON_NOTIFY_REFLECT(TVN_DELETEITEM, OnDeleteitem)
	ON_NOTIFY_REFLECT(TVN_SELCHANGING, OnSelchanging)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	//ON_UPDATE_COMMAND_UI(IDM_TREE_EXPLORE, OnUpdateTreeExplore)
	//ON_UPDATE_COMMAND_UI(IDM_TREE_EXPAND, OnUpdateTreeExpand)
	//ON_COMMAND(IDM_TREE_EXPAND, OnTreeExpand)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnNMDblclk)
	ON_COMMAND_RANGE(IDM_MRU_DIR1, IDM_MRU_DIR16, OnMRUDirectoryHandler) 
	ON_UPDATE_COMMAND_UI_RANGE(IDM_MRU_DIR1, IDM_MRU_DIR16, OnUpdateMRUDirectoryHandler)
	ON_COMMAND_RANGE(IDM_MRU_FILE1, IDM_MRU_FILE16, OnMRUFileHandler) 
	ON_UPDATE_COMMAND_UI_RANGE(IDM_MRU_FILE1, IDM_MRU_FILE16, OnUpdateMRUFileHandler)
	ON_COMMAND(ID_FILEBROWSER_PLAY, OnFilebrowserPlay)
	ON_UPDATE_COMMAND_UI(ID_FILEBROWSER_PLAY, OnUpdateFilebrowserPlay)
	ON_WM_INITMENUPOPUP()
	ON_COMMAND(ID_FILEBROWSER_ENQUEUE, OnFilebrowserEnqueue)
	ON_UPDATE_COMMAND_UI(ID_FILEBROWSER_ENQUEUE, OnUpdateFilebrowserEnqueue)
	ON_COMMAND(ID_SORT_NOSORT, OnSortByNoSort)
	ON_UPDATE_COMMAND_UI(ID_SORT_NOSORT, OnUpdtateByNoSort)
	ON_COMMAND(ID_SORT_BYNAME, OnSortByName)
	ON_UPDATE_COMMAND_UI(ID_SORT_BYNAME, OnUpdtateSortByName)
	ON_COMMAND(ID_SORT_BYEXTENSION, OnSortByExtension)
	ON_UPDATE_COMMAND_UI(ID_SORT_BYEXTENSION, OnUpdtateSortByExtension)
	ON_COMMAND(ID_SORT_BYDATE, OnSortByDate)
	ON_UPDATE_COMMAND_UI(ID_SORT_BYDATE, OnUpdtateSortByDate)
	ON_COMMAND(ID_SORT_ASCENDING, OnSortAscending)
	ON_UPDATE_COMMAND_UI(ID_SORT_ASCENDING, OnUpdtateSortAscending)
	ON_COMMAND(ID_SORT_DESCENDING, OnSortDescending)
	ON_UPDATE_COMMAND_UI(ID_SORT_DESCENDING, OnUpdtateSortDescending)
	//}}AFX_MSG_MAP
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG, OnTvnBegindrag)
	ON_COMMAND(ID_FILEBROWSER_SYNCHRONISE_CURRENT_FILE, OnFilebrowserSynchroniseCurrentFile)
	ON_UPDATE_COMMAND_UI(ID_FILEBROWSER_SYNCHRONISE_CURRENT_FILE, OnUpdateFilebrowserSynchroniseCurrentFile)
	ON_COMMAND(ID_FILEBROWSER_REFRESH, OnFilebrowserRefresh)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLeftView construction/destruction

CLeftView::CLeftView()
{
	m_hLastMenuDropTreeItem = NULL;
	m_pTree = NULL;
	m_pDoc = new CNetExplorerDoc();
	m_nSortAttribute = CNetExplorerDoc::SortAscending;
}

CLeftView::~CLeftView()
{
	if (m_pTree)
	{
		m_pTree->m_hWnd = NULL;
		delete m_pTree;
	}
	if(m_pDoc != NULL)
	{
		delete m_pDoc;
		m_pDoc = NULL;
	}
}

void CLeftView::PostNcDestroy()
{
	// default for views is to allocate them on the heap
	//  the default post-cleanup is to 'delete this'.
	//  never explicitly call 'delete' on a view
//	delete this;
}

BOOL CLeftView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= TVS_HASBUTTONS|TVS_HASLINES|TVS_SHOWSELALWAYS|TVS_LINESATROOT;// | TVS_INFOTIP;
	return CTreeView::PreCreateWindow(cs);
}

void CLeftView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	m_SmallImageList.Attach(::ImageList_LoadImage(AfxFindResourceHandle((LPCTSTR)IDB_BITMAP_SMALL_ICON, RT_BITMAP),
			(LPCTSTR)IDB_BITMAP_SMALL_ICON, 16, 0, RGB(255,255,255), IMAGE_BITMAP, 0));
	m_pTree->SetImageList(&m_SmallImageList, TVSIL_NORMAL);
}

CNetExplorerDoc* CLeftView::GetDoc() //GetDocument()
{ 
	return (CNetExplorerDoc*)m_pDoc;
}

/////////////////////////////////////////////////////////////////////////////
// CLeftView message handlers

void CLeftView::AllocEmbeddedTree()
{
	//over-ride this virtual to allocate your own
	// CTreeCtrl class
	ASSERT(!m_pTree);		//don't call base if over-ridden
	m_pTree = new CMultiTree;
}

int CLeftView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CTreeView::OnCreate(lpCreateStruct) == -1)
		return -1;

	AllocEmbeddedTree();
	//force tree to use this
	m_pTree->m_hWnd = m_hWnd;
	m_pTree->m_pfnSuper = m_pfnSuper;
	m_pTree->PreSubclassWindow();

	GetTreeCtrl().ModifyStyle(0, TVS_HASLINES | TVS_HASBUTTONS);

	CNetExplorerDoc* pDoc = GetDoc(); //();
	ASSERT_VALID(pDoc);

	CTreeNode * pTreeNode = new CTreeNode;
	TV_INSERTSTRUCT sTreeCtrlItem;
	sTreeCtrlItem.hParent = TVI_ROOT;
	sTreeCtrlItem.hInsertAfter = TVI_LAST;
	sTreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	sTreeCtrlItem.item.pszText = "Desktop";
	sTreeCtrlItem.item.lParam = (LPARAM)pTreeNode;
	sTreeCtrlItem.item.iImage = CTreeNode::Type2ImageNumber(CTreeNode::Desktop);
	sTreeCtrlItem.item.iSelectedImage = CTreeNode::Type2ImageNumber(CTreeNode::Desktop);
	pTreeNode->hItem = GetTreeCtrl().InsertItem(&sTreeCtrlItem);
	pTreeNode->hParent = NULL;
	pTreeNode->nType = CTreeNode::Desktop;
	pTreeNode->szPath = "Desktop";
	pTreeNode->pLeftView = this;
	pTreeNode->hDummyItem = GetTreeCtrl().InsertItem("Dummy", pTreeNode->hItem);
	pTreeNode->pParent = NULL;

	GetTreeCtrl().Expand(pTreeNode->hItem, TVE_EXPAND);

	return 0;
}

void CLeftView::OnItemexpanding(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	CNetExplorerDoc* pDoc = GetDoc(); //();
	ASSERT_VALID(pDoc);

	CTreeNode * pTreeNode = (CTreeNode *)pNMTreeView->itemNew.lParam;
	if(pTreeNode && pTreeNode->hDummyItem)
	{
		CWaitCursor wc;
		GetTreeCtrl().DeleteItem(pTreeNode->hDummyItem);
		pTreeNode->hDummyItem = NULL;
		switch(pTreeNode->nType)
		{
		case CTreeNode::Desktop:
			pDoc->EnumerateDesktop(pTreeNode, CLeftView::EnumerateDesktop);
			break;
		case CTreeNode::WorkArea:
			pDoc->EnumerateWorkArea(pTreeNode, CLeftView::EnumerateDesktop);
			break;
		case CTreeNode::NetNeighborhood:
			pDoc->EnumerateNetNeighborhood(pTreeNode, CLeftView::EnumerateDesktop);
			break;
		case CTreeNode::Network:
		case CTreeNode::NetProtocol:
		case CTreeNode::NetDirectory:
		case CTreeNode::Domain:
			pDoc->EnumerateNetwork(pTreeNode, CLeftView::EnumerateDesktop);
			break;
		case CTreeNode::Server:
			pDoc->EnumerateShare(pTreeNode, CLeftView::EnumerateServer);
			break;
		case CTreeNode::Share:
		case CTreeNode::NetShare:
		case CTreeNode::Drive:
		case CTreeNode::Directory:
			pDoc->EnumerateFiles(ENUM_DIRECTORIES, (CNetExplorerDoc::SortType)(CNetExplorerDoc::SortAscending & CNetExplorerDoc::SortByName), pTreeNode, CLeftView::EnumerateFiles);
			pDoc->EnumerateFiles(ENUM_FILES, m_nSortAttribute, pTreeNode, CLeftView::EnumerateFiles);
			break;
		}
	}
	
	*pResult = 0;
}

BOOL CLeftView::EnumerateDesktop(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode)
{
	if(pTreeNode && pParentTreeNode && pParentTreeNode->pLeftView)
	{
		TV_INSERTSTRUCT sTreeCtrlItem;
		sTreeCtrlItem.hParent = pParentTreeNode->hItem;
		sTreeCtrlItem.hInsertAfter = TVI_LAST;
		sTreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		sTreeCtrlItem.item.pszText = (LPSTR)(LPCTSTR)pTreeNode->szName;
		sTreeCtrlItem.item.lParam = (LPARAM)pTreeNode;
		sTreeCtrlItem.item.iImage = CTreeNode::Type2ImageNumber(pTreeNode->nType);
		sTreeCtrlItem.item.iSelectedImage = CTreeNode::Type2ImageNumber(pTreeNode->nType);
		pTreeNode->hItem = pParentTreeNode->pLeftView->GetTreeCtrl().InsertItem(&sTreeCtrlItem);
		pTreeNode->hParent = NULL;
		pTreeNode->pLeftView = pParentTreeNode->pLeftView;
		pTreeNode->hDummyItem = pTreeNode->pLeftView->GetTreeCtrl().InsertItem("Dummy", pTreeNode->hItem);
		return TRUE;
	}
	return FALSE;
}

BOOL CLeftView::EnumerateServer(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode)
{
	if(pTreeNode && pParentTreeNode && pParentTreeNode->pLeftView)
	{
		TV_INSERTSTRUCT sTreeCtrlItem;
		sTreeCtrlItem.hParent = pParentTreeNode->hItem;
		sTreeCtrlItem.hInsertAfter = TVI_LAST;
		sTreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		sTreeCtrlItem.item.pszText = (LPSTR)(LPCTSTR)pTreeNode->szName;
		sTreeCtrlItem.item.lParam = (LPARAM)pTreeNode;
		sTreeCtrlItem.item.iImage = CTreeNode::Type2ImageNumber(pTreeNode->nType);
		sTreeCtrlItem.item.iSelectedImage = CTreeNode::Type2ImageNumber(pTreeNode->nType);
		pTreeNode->hItem = pParentTreeNode->pLeftView->GetTreeCtrl().InsertItem(&sTreeCtrlItem);
		pTreeNode->hParent = NULL;
		pTreeNode->nType = CTreeNode::Directory;
		pTreeNode->pLeftView = pParentTreeNode->pLeftView;
		pTreeNode->hDummyItem = pTreeNode->pLeftView->GetTreeCtrl().InsertItem("Dummy", pTreeNode->hItem);
		return TRUE;
	}
	return FALSE;
}

BOOL CLeftView::EnumerateFiles(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode)
{
	if(pTreeNode && pParentTreeNode && pParentTreeNode->pLeftView)
	{
		TV_INSERTSTRUCT sTreeCtrlItem;
		sTreeCtrlItem.hParent = pParentTreeNode->hItem;
		sTreeCtrlItem.hInsertAfter = TVI_LAST;
		sTreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		sTreeCtrlItem.item.pszText = (LPSTR)(LPCTSTR)pTreeNode->szName;
		sTreeCtrlItem.item.lParam = (LPARAM)pTreeNode;
		sTreeCtrlItem.item.iImage = CTreeNode::Type2ImageNumber(pTreeNode->nType); //CTreeNode::Directory);
		sTreeCtrlItem.item.iSelectedImage = CTreeNode::Type2ImageNumber(pTreeNode->nType == CTreeNode::Directory ? CTreeNode::SelectedDirectory : pTreeNode->nType);
		pTreeNode->hItem = pParentTreeNode->pLeftView->GetTreeCtrl().InsertItem(&sTreeCtrlItem);
		pTreeNode->hParent = NULL;
		pTreeNode->pLeftView = pParentTreeNode->pLeftView;
		pTreeNode->hDummyItem = (pTreeNode->nType == CTreeNode::File ? NULL : pTreeNode->pLeftView->GetTreeCtrl().InsertItem("Dummy", pTreeNode->hItem));
		return TRUE;
	}
	return FALSE;
}

void CLeftView::OnDeleteitem(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	if(pNMTreeView->itemOld.lParam)
	  delete ((CTreeNode*)pNMTreeView->itemOld.lParam);

	*pResult = 0;
}

void CLeftView::OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

/*	short nn = GetAsyncKeyState(VK_RBUTTON);

	if(!(GetAsyncKeyState(VK_SHIFT) & 0x8000) &&
		!(GetAsyncKeyState(VK_CONTROL) & 0x8000) &&
		!((pNMTreeView->action & TVC_BYMOUSE) && (nn & 0x8000)) &&
		(pNMTreeView->itemNew.state & TVIS_FOCUSED)) {
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame && pNMTreeView->itemNew.hItem)
			pMainFrame->TreeSelectionChanging((CTreeNode *)pNMTreeView->itemNew.lParam);
	}
*/
	*pResult = 0;
}

HTREEITEM CLeftView::FindItemByName(LPCTSTR pszName, HTREEITEM hParentItem)
{
	if(hParentItem && pszName)
	{
		CTreeCtrl & TreeCtrl = GetTreeCtrl();

		HTREEITEM hItem = TreeCtrl.GetNextItem(hParentItem, TVGN_CHILD);
		while(hItem)
		{
			if(TreeCtrl.GetItemText(hItem).CompareNoCase(pszName) == 0)
				return hItem;
			hItem = TreeCtrl.GetNextItem(hItem, TVGN_NEXT);
		}
	}

	return NULL;
}

void CLeftView::ListCtrlSelectionChanging(CTreeNode * pTreeNode)
{
	if(GetTreeCtrl().GetItemState(pTreeNode->hParent, TVIS_EXPANDED) != TVIS_EXPANDED)
		GetTreeCtrl().Expand(pTreeNode->hParent, TVE_EXPAND);

	HTREEITEM hItem;
	if(hItem = FindItemByName(pTreeNode->szName, pTreeNode->hParent))
		GetTreeCtrl().SelectItem(hItem);
}

void CLeftView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	GetCursorPos(&point);
	CMenu menu;
	menu.LoadMenu(IDR_MENU_FILE_BROWSER);
	CMenu * pMenu = NULL;
	if(pMenu = menu.GetSubMenu(0))
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame)
			pMainFrame->m_fRecentDirectoryListForFileBrowserDlgBar = TRUE;
		pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this); //AfxGetMainWnd());
		if(pMainFrame)
			pMainFrame->m_fRecentDirectoryListForFileBrowserDlgBar = FALSE;
	}
	*pResult = 0;
}

void CLeftView::OnMRUFileHandler(UINT i)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_pRecentFileList)
	{
		int nIndex = i - IDM_MRU_FILE1;
		if((*pMainFrame->m_pRecentFileList)[nIndex].GetLength() != 0)
		{
			if(pMainFrame->PlayFile((*pMainFrame->m_pRecentFileList)[nIndex], 0))
				pMainFrame->AddFileToRecentFileList((*pMainFrame->m_pRecentFileList)[nIndex]);
			else
				pMainFrame->m_pRecentFileList->Remove(nIndex);
			Find((*pMainFrame->m_pRecentFileList)[nIndex]);
			//if(OpenFile((*pMainFrame->m_pRecentFileList)[nIndex], FALSE, TRUE, TRUE))
			//	pMainFrame->PlayCurrent();
			//else
			//	pMainFrame->m_pRecentFileList->Remove(nIndex);
		}
	}
}

void CLeftView::OnMRUDirectoryHandler(UINT i)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_pRecentDirectoryList)
	{
		CString szName, strCurDir, strMessage;
		int nIndex = i - IDM_MRU_DIR1;
		if((*(pMainFrame->m_pRecentDirectoryList))[nIndex].GetLength() != 0)
			Find((*pMainFrame->m_pRecentDirectoryList)[nIndex] + _T('\\'));
	}
} 

void CLeftView::OnUpdateMRUFileHandler(CCmdUI* pCmdUI)
{
	//TRACE("CMainFrame::OnUpdateMRUFileHandler ID %d, Index %d, MenuCount %d, SubMenuCount %d\n",
	//	pCmdUI->m_nID, pCmdUI->m_nIndex,
	//	(pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetMenuItemCount() : -1),
	//	(pCmdUI->m_pSubMenu ? pCmdUI->m_pSubMenu->GetMenuItemCount() : -1));

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_pRecentFileList && pCmdUI->m_pSubMenu == NULL)
		pMainFrame->m_pRecentFileList->UpdateMenuEx(pCmdUI);
}

void CLeftView::OnUpdateMRUDirectoryHandler(CCmdUI* pCmdUI)
{
	//TRACE("CMainFrame::OnUpdateMRUDirectoryHandler ID %d, Index %d, MenuCount %d, SubMenuCount %d\n",
	//	pCmdUI->m_nID, pCmdUI->m_nIndex,
	//	(pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetMenuItemCount() : -1),
	//	(pCmdUI->m_pSubMenu ? pCmdUI->m_pSubMenu->GetMenuItemCount() : -1));
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && pMainFrame->m_pRecentDirectoryList && pCmdUI->m_pSubMenu == NULL)
		pMainFrame->m_pRecentDirectoryList->UpdateMenuEx(pCmdUI);
}

void CLeftView::OnTreeExpand() 
{
	if(m_hLastMenuDropTreeItem)
		GetTreeCtrl().Expand(m_hLastMenuDropTreeItem, TVE_TOGGLE);
}

void CLeftView::OnUpdateTreeExplore(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

void CLeftView::OnUpdateTreeExpand(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

LRESULT CLeftView::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	LRESULT lResult = 0;
	if(!OnWndMsg(message, wParam, lParam, &lResult))
		if(!m_pTree || !m_pTree->OnWndMsg(message, wParam, lParam, &lResult))
			lResult = DefWindowProc(message, wParam, lParam);
	return lResult;
}

BOOL CLeftView::OnChildNotify(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pLResult) 
{
	if(CTreeView::OnChildNotify(message, wParam, lParam, pLResult))
		return TRUE;
	return m_pTree && m_pTree->OnChildNotify(message, wParam, lParam, pLResult);
}

BOOL CLeftView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	if(CTreeView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;
	return m_pTree && m_pTree->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CLeftView::OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex,BOOL bSysMenu)
{
    ASSERT(pPopupMenu != NULL);
    // Check the enabled state of various menu items.

    CCmdUI state;
    state.m_pMenu = pPopupMenu;
    ASSERT(state.m_pOther == NULL);
    ASSERT(state.m_pParentMenu == NULL);

    // Determine if menu is popup in top-level menu and set m_pOther to
    // it if so (m_pParentMenu == NULL indicates that it is secondary popup).
    HMENU hParentMenu;
    if (AfxGetThreadState()->m_hTrackingMenu == pPopupMenu->m_hMenu)
        state.m_pParentMenu = pPopupMenu;    // Parent == child for tracking popup.
    else if ((hParentMenu = ::GetMenu(m_hWnd)) != NULL)
    {
        CWnd* pParent = this;
           // Child windows don't have menus--need to go to the top!
        if (pParent != NULL &&
           (hParentMenu = ::GetMenu(pParent->m_hWnd)) != NULL)
        {
           int nIndexMax = ::GetMenuItemCount(hParentMenu);
           for (int nIndex = 0; nIndex < nIndexMax; nIndex++)
           {
            if (::GetSubMenu(hParentMenu, nIndex) == pPopupMenu->m_hMenu)
            {
                // When popup is found, m_pParentMenu is containing menu.
                state.m_pParentMenu = CMenu::FromHandle(hParentMenu);
                break;
            }
           }
        }
    }

    state.m_nIndexMax = pPopupMenu->GetMenuItemCount();
    for (state.m_nIndex = 0; state.m_nIndex < state.m_nIndexMax;
      state.m_nIndex++)
    {
        state.m_nID = pPopupMenu->GetMenuItemID(state.m_nIndex);
        if (state.m_nID == 0)
           continue; // Menu separator or invalid cmd - ignore it.

        ASSERT(state.m_pOther == NULL);
        ASSERT(state.m_pMenu != NULL);
        if (state.m_nID == (UINT)-1)
        {
           // Possibly a popup menu, route to first item of that popup.
           state.m_pSubMenu = pPopupMenu->GetSubMenu(state.m_nIndex);
           if (state.m_pSubMenu == NULL ||
            (state.m_nID = state.m_pSubMenu->GetMenuItemID(0)) == 0 ||
            state.m_nID == (UINT)-1)
           {
            continue;       // First item of popup can't be routed to.
           }
           state.DoUpdate(this, TRUE);   // Popups are never auto disabled.
        }
        else
        {
           // Normal menu item.
           // Auto enable/disable if frame window has m_bAutoMenuEnable
           // set and command is _not_ a system command.
           state.m_pSubMenu = NULL;
           state.DoUpdate(this, FALSE);
        }

        // Adjust for menu deletions and additions.
        UINT nCount = pPopupMenu->GetMenuItemCount();
        if (nCount < state.m_nIndexMax)
        {
           state.m_nIndex -= (state.m_nIndexMax - nCount);
           while (state.m_nIndex < nCount &&
            pPopupMenu->GetMenuItemID(state.m_nIndex) == state.m_nID)
           {
            state.m_nIndex++;
           }
        }
        state.m_nIndexMax = nCount;
    }
} 

void CLeftView::OnNMDblclk(NMHDR *pNMHDR, LRESULT *pResult)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		//BOOL fSomethingToPlay = FALSE;
		HTREEITEM hTreeItem = m_pTree->GetFirstSelectedItem();
		//do
		//{
			CTreeNode * pTreeNode = (CTreeNode *)m_pTree->GetItemData(hTreeItem);
			if(pTreeNode != NULL && pTreeNode->nType == CTreeNode::File)
			{
				//pMainFrame->OpenFile(pTreeNode->szPath, FALSE, TRUE, TRUE);
				if(pMainFrame->PlayFile(pTreeNode->szPath, FALSE))
					pMainFrame->AddFileToRecentFileList(pTreeNode->szPath);
				//else
				//	pMainFrame->m_pRecentFileList->FRemove(nIndex);
				//fSomethingToPlay = TRUE;
			}
		//}
		//while((hTreeItem = m_pTree->GetNextSelectedItem(hTreeItem)) != NULL);

		//if(fSomethingToPlay)
		//	pMainFrame->PlayCurrent();
	}

	*pResult = 0;
}

void CLeftView::Find(LPCTSTR pszFindPath)
{
	//CString szFindPath(pszFindPath);
	//if(szFindPath.GetLength() != 0 && szFindPath[szFindPath.GetLength() - 1] == '\\')
	//	szFindPath += '.'; //szFindPath.Left(szFindPath.GetLength() - 1);
	CNetFind NetFind;
	NetFind.Init(m_pDoc, OnSearchOne, (LPARAM)this);
	NetFind.Find(pszFindPath);
	CString szFindPath(pszFindPath);
	int nPos = szFindPath.ReverseFind('\\');
	if(nPos != -1)
	{
		CString szFile = szFindPath.Mid(nPos + 1);
		CTreeCtrl & treeCtrl = GetTreeCtrl();
		CMultiTree & multiTreeCtrl = GetMultiTreeCtrl();

		HTREEITEM hTreeItem = treeCtrl.GetNextItem(treeCtrl.GetSelectedItem(), TVGN_CHILD);
		while(hTreeItem)
		{
			CTreeNode * pTreeNode = (CTreeNode *)treeCtrl.GetItemData(hTreeItem);
			if(pTreeNode && _strnicmp(szFindPath, pTreeNode->szPath, pTreeNode->szPath.GetLength()) == 0)
			{
				treeCtrl.Expand(hTreeItem, TVE_EXPAND);
				treeCtrl.SelectItem(hTreeItem);
				treeCtrl.EnsureVisible(hTreeItem);

				if(szFile != _T(""))
				{
					hTreeItem = treeCtrl.GetNextItem(hTreeItem, TVGN_CHILD);
					while(hTreeItem)
					{

						pTreeNode = (CTreeNode *)(treeCtrl.GetItemData(hTreeItem));

						if(pTreeNode && _strnicmp(szFindPath, pTreeNode->szPath, pTreeNode->szPath.GetLength()) == 0)
						{
							treeCtrl.Expand(hTreeItem, TVE_EXPAND);
							treeCtrl.SelectItem(hTreeItem);
							treeCtrl.EnsureVisible(hTreeItem);
							break;
						}
						hTreeItem = treeCtrl.GetNextItem(hTreeItem, TVGN_NEXT);
					}
				}
				break;
			}
			hTreeItem = treeCtrl.GetNextItem(hTreeItem, TVGN_NEXT);
		}

		//HTREEITEM hParentTreeItem = FindItemByName(szFolder, GetTreeCtrl().GetSelectedItem());
		//if(hParentTreeItem != NULL)
		//{
		//	GetTreeCtrl().Expand(hParentTreeItem, TVE_EXPAND);
		//	GetTreeCtrl().SelectItem(hParentTreeItem);
		//}
	}
}

void CLeftView::OnSearchOne(CTreeNodeList * pTreeNodeList, LPARAM lParam)
{
	CWaitCursor wc;

	CLeftView * pLeftView = (CLeftView *)lParam;
	CTreeCtrl & TreeCtrl = pLeftView->GetTreeCtrl();

	CTreeNode * pTreeNode;
	POSITION pos = pTreeNodeList->GetTailPosition(),
			 posHead = pTreeNodeList->GetHeadPosition();

	HTREEITEM hParentTreeItem = TreeCtrl.GetRootItem();
	pTreeNodeList->GetPrev(pos); // Pass the "Desktop"
	while(pos && pos != posHead)
	{
		if(pTreeNode = pTreeNodeList->GetPrev(pos))
		{
			hParentTreeItem = pLeftView->FindItemByName(pTreeNode->szName, hParentTreeItem);
			if(!hParentTreeItem)
				break;
			TreeCtrl.Expand(hParentTreeItem, TVE_EXPAND);
		}
	}

	if(hParentTreeItem)
	{
		TreeCtrl.SelectItem(hParentTreeItem);
		TreeCtrl.Expand(hParentTreeItem, TVE_EXPAND);
	}

	while(!pTreeNodeList->IsEmpty())
	{
		CTreeNode * pTreeNode = pTreeNodeList->RemoveHead();
		if(pTreeNode)
			delete pTreeNode;
	}
	delete pTreeNodeList;
}

void CLeftView::OnFilebrowserPlay()
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		BOOL fFirst = TRUE;
		HTREEITEM hTreeItem = GetMultiTreeCtrl().GetFirstSelectedItem();
		do
		{
			CTreeNode * pTreeNode = (CTreeNode *)GetMultiTreeCtrl().GetItemData(hTreeItem);
			if(pTreeNode != NULL && pTreeNode->nType == CTreeNode::File)
			{
				pMainFrame->OpenFile(pTreeNode->szPath, FALSE, TRUE, fFirst);
				fFirst = FALSE;
			}
		}
		while((hTreeItem = GetMultiTreeCtrl().GetNextSelectedItem(hTreeItem)) != NULL);

		if(!fFirst)
			pMainFrame->PlayCurrent();
	}
}

void CLeftView::OnUpdateFilebrowserPlay(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(GetMultiTreeCtrl().GetSelectedCount() > 0);
}

void CLeftView::OnFilebrowserEnqueue()
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		HTREEITEM hTreeItem = GetMultiTreeCtrl().GetFirstSelectedItem();
		do
		{
			CTreeNode * pTreeNode = (CTreeNode *)GetMultiTreeCtrl().GetItemData(hTreeItem);
			if(pTreeNode != NULL && pTreeNode->nType == CTreeNode::File)
			{
				pMainFrame->OpenFile(pTreeNode->szPath, FALSE, TRUE, FALSE);
			}
		}
		while((hTreeItem = GetMultiTreeCtrl().GetNextSelectedItem(hTreeItem)) != NULL);
	}
}

void CLeftView::OnUpdateFilebrowserEnqueue(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(GetMultiTreeCtrl().GetSelectedCount() > 0);
}

void CLeftView::OnTvnBegindrag(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

	DragNDrop::CIDropSource* pdsrc = new DragNDrop::CIDropSource;
	pdsrc->AddRef();
	DragNDrop::CIDataObject* pdobj = new DragNDrop::CIDataObject(pdsrc);
	pdobj->AddRef();

	// Init the supported format
	FORMATETC fmtetc; // = {0}; 
	memset(&fmtetc, 0, sizeof(FORMATETC));
	fmtetc.cfFormat = CF_HDROP; 
	fmtetc.dwAspect = DVASPECT_CONTENT; 
	fmtetc.lindex = -1; 
	fmtetc.tymed = TYMED_HGLOBAL;
	// Init the medium used
	STGMEDIUM medium;// = {0};
	memset(&medium, 0, sizeof(STGMEDIUM));
	medium.tymed = TYMED_HGLOBAL;

//	CString szFiles;
	CStringArray aszFiles;
	int nLength = 0;
	HTREEITEM hTreeItem = GetMultiTreeCtrl().GetFirstSelectedItem();
	do
	{
		CTreeNode * pTreeNode = (CTreeNode *)GetMultiTreeCtrl().GetItemData(hTreeItem);
		if(pTreeNode != NULL && pTreeNode->nType == CTreeNode::File)
		{
			aszFiles.Add(pTreeNode->szPath);
//			szFiles += pTreeNode->szPath + '\0';
			nLength += pTreeNode->szPath.GetLength() + 1;
		}
	}
	while((hTreeItem = GetMultiTreeCtrl().GetNextSelectedItem(hTreeItem)) != NULL);
	//szFiles += '\0';
	//int nLength = szFiles.GetLength();
	nLength++;

	TCHAR * pszFiles = new TCHAR[nLength];
	TCHAR * pszFiles2 = pszFiles;
	memset(pszFiles, 0, nLength * sizeof(TCHAR));
	for(int i = 0; i < aszFiles.GetCount(); i++)
	{
		int nLengthFile = aszFiles.GetAt(i).GetLength();
		memcpy(pszFiles2, (LPCTSTR)(aszFiles[i]), nLengthFile * sizeof(TCHAR));
		pszFiles2 += nLengthFile;
		pszFiles2++;
	}
//	memcpy(pszFiles, (LPCTSTR)szFiles, nLength);

	// Make the HANDLE for DROPFILES
	int nSize = sizeof(DROPFILES) + nLength; // + 1;
	medium.hGlobal = ::GlobalAlloc(GMEM_ZEROINIT /*| GMEM_MOVEABLE | GMEM_DDESHARE*/, nSize);
	LPDROPFILES pDropFiles = (LPDROPFILES)::GlobalLock(medium.hGlobal);
	pDropFiles->fWide = false;
	GetCursorPos(&(pDropFiles->pt));
	// Above position is screen coord.
	pDropFiles->fNC = TRUE;

	pDropFiles->pFiles = sizeof(DROPFILES);
	LPBYTE pData = (LPBYTE)pDropFiles + sizeof(DROPFILES);
	::CopyMemory(pData, pszFiles, nLength); // + 1);
	::GlobalUnlock(medium.hGlobal);
	delete [] pszFiles;

	// Add it to DataObject
	pdobj->SetData(&fmtetc, &medium, TRUE); // Release the medium for me
	// add more formats and medium if needed
	// Initiate the Drag & Drop
	DWORD dwEffect = 0;
	::DoDragDrop(pdobj, pdsrc, DROPEFFECT_COPY, &dwEffect);
	pdsrc->Release();
	pdobj->Release();

	*pResult = 0;
}

BOOL CLeftView::PreTranslateMessage(MSG* pMsg)
{
    if (pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST)
    {
//		if(pMsg->wParam == VK_NEXT || pMsg->wParam == VK_PRIOR)
			return FALSE;

        // dipatch keyboard messages directly (skip main accelerators)
        ::TranslateMessage(pMsg);
        ::DispatchMessage(pMsg);
        return TRUE;
    }

	return CTreeView::PreTranslateMessage(pMsg);
}

void CLeftView::OnFilebrowserSynchroniseCurrentFile()
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		Find(pMainFrame->m_szFileName);
		pMainFrame->m_fPlayListMode = FALSE;
	}
}

void CLeftView::OnUpdateFilebrowserSynchroniseCurrentFile(CCmdUI *pCmdUI)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
		pCmdUI->Enable(pMainFrame->m_szFileName != _T(""));
}

int CALLBACK CLeftView::CompareTreeProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CLeftView * pLeftView = (CLeftView *)lParamSort;
	CMultiTree & treeCtrl = pLeftView->GetMultiTreeCtrl();

	CTreeNode * pTreeNode1 = (CTreeNode *)lParam1;
	CTreeNode * pTreeNode2 = (CTreeNode *)lParam2;

	int nResult = 0;
	if(pTreeNode1->nType == CTreeNode::File && pTreeNode2->nType == CTreeNode::File)
	{
		if(pLeftView->m_nSortAttribute & CNetExplorerDoc::SortByName)
			nResult = CNetExplorerDoc::CTreeNodeCompareByName(pTreeNode1, pTreeNode2);
		else if(pLeftView->m_nSortAttribute & CNetExplorerDoc::SortByExtension)
			nResult = CNetExplorerDoc::CTreeNodeCompareByExtension(pTreeNode1, pTreeNode2);
		else if(pLeftView->m_nSortAttribute & CNetExplorerDoc::SortByDate)
			nResult = CNetExplorerDoc::CTreeNodeCompareByDate(pTreeNode1, pTreeNode2);

		if((pLeftView->m_nSortAttribute & CNetExplorerDoc::SortAscending) == 0)
			nResult *= -1;
	}
	return nResult;
}

BOOL CLeftView::RefreshEnumerateElements(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode)
{
	m_mapTreeNodeEnum[(LPCTSTR)pTreeNode->szName] = pTreeNode;
	return TRUE;
}

void CLeftView::RefreshEnumerate(HTREEITEM hParentItem)
{
	CNetExplorerDoc * pDoc = GetDoc();
	CTreeCtrl & treeCtrl = GetTreeCtrl();

	CTreeNode * pParentTreeNode = (CTreeNode *)treeCtrl.GetItemData(hParentItem);
	if(pParentTreeNode->hDummyItem != NULL)
		return;
	switch(pParentTreeNode->nType)
	{
	case CTreeNode::Desktop:
		pDoc->EnumerateDesktop(pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	case CTreeNode::WorkArea:
		pDoc->EnumerateWorkArea(pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	case CTreeNode::NetNeighborhood:
		pDoc->EnumerateNetNeighborhood(pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	case CTreeNode::Network:
	case CTreeNode::NetProtocol:
	case CTreeNode::NetDirectory:
	case CTreeNode::Domain:
		pDoc->EnumerateNetwork(pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	case CTreeNode::Server:
		pDoc->EnumerateShare(pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	case CTreeNode::Share:
	case CTreeNode::NetShare:
	case CTreeNode::Drive:
	case CTreeNode::Directory:
		pDoc->EnumerateFiles(ENUM_DIRECTORIES | ENUM_FILES, CNetExplorerDoc::NoSort, pParentTreeNode, CLeftView::RefreshEnumerateElements);
		break;
	}
	CMap<CString, LPCTSTR, CTreeNode *, CTreeNode *> mapTreeNodeEnum;
	CMap<CString, LPCTSTR, CTreeNode *, CTreeNode *>::CPair* pCurVal = m_mapTreeNodeEnum.PGetFirstAssoc();
	while(pCurVal != NULL)
	{
		mapTreeNodeEnum[pCurVal->key] = pCurVal->value;
		pCurVal = m_mapTreeNodeEnum.PGetNextAssoc(pCurVal);
	}
	m_mapTreeNodeEnum.RemoveAll();

//	CArray<HTREEITEM> apItemToRemove;
	HTREEITEM hItem = treeCtrl.GetNextItem(hParentItem, TVGN_CHILD);
	while(hItem)
	{
		CTreeNode * pTreeNode = (CTreeNode *)treeCtrl.GetItemData(hItem);
		if(pTreeNode)
		{
			CTreeNode * pEnumTreeNode = mapTreeNodeEnum[pTreeNode->szName];
			if(pEnumTreeNode)
			{
				mapTreeNodeEnum.RemoveKey(pTreeNode->szName);
				delete pEnumTreeNode;
				RefreshEnumerate(hItem);
			}
			else
			{
				//apItemToRemove.Add(hItem);
				HTREEITEM hItemToRemove = hItem;
				hItem = treeCtrl.GetNextItem(hItem, TVGN_NEXT);
				treeCtrl.DeleteItem(hItemToRemove);
			}
		}
		hItem = treeCtrl.GetNextItem(hItem, TVGN_NEXT);
	}

	//for(int i = 0; i < apItemToRemove.GetCount(); i++)
	//	treeCtrl.DeleteItem(apItemToRemove[i]);

	{
		CMap<CString, LPCTSTR, CTreeNode *, CTreeNode *>::CPair* pCurVal2 = mapTreeNodeEnum.PGetFirstAssoc();
		while(pCurVal2 != NULL)
		{
			CTreeNode * pTreeNode = pCurVal2->value;
			if(pTreeNode != NULL)
			{
				TV_INSERTSTRUCT sTreeCtrlItem;
				sTreeCtrlItem.hParent = pParentTreeNode->hItem;
				sTreeCtrlItem.hInsertAfter = TVI_LAST;
				sTreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
				sTreeCtrlItem.item.pszText = (LPSTR)(LPCTSTR)pTreeNode->szName;
				sTreeCtrlItem.item.lParam = (LPARAM)pTreeNode;
				sTreeCtrlItem.item.iImage = CTreeNode::Type2ImageNumber(pTreeNode->nType); //CTreeNode::Directory);
				sTreeCtrlItem.item.iSelectedImage = CTreeNode::Type2ImageNumber(pTreeNode->nType == CTreeNode::Directory ? CTreeNode::SelectedDirectory : pTreeNode->nType);
				pTreeNode->hItem = pParentTreeNode->pLeftView->GetTreeCtrl().InsertItem(&sTreeCtrlItem);
				pTreeNode->hParent = NULL;
				pTreeNode->pLeftView = pParentTreeNode->pLeftView;
				pTreeNode->hDummyItem = (pTreeNode->nType == CTreeNode::File ? NULL : pTreeNode->pLeftView->GetTreeCtrl().InsertItem("Dummy", pTreeNode->hItem));
			}
			pCurVal2 = mapTreeNodeEnum.PGetNextAssoc(pCurVal2);
		}
		mapTreeNodeEnum.RemoveAll();
	}

	if(m_nSortAttribute & 0x07)
	{
		TVSORTCB tvs;
		tvs.hParent = hParentItem;
		tvs.lpfnCompare = CLeftView::CompareTreeProc;
		tvs.lParam = (LPARAM)this;
		treeCtrl.SortChildrenCB(&tvs);
	}
}

void CLeftView::OnFilebrowserRefresh()
{
	CTreeCtrl & treeCtrl = GetTreeCtrl();

	RefreshEnumerate(treeCtrl.GetRootItem());
}

void CLeftView::OnSortByNoSort()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)(CNetExplorerDoc::SortAscending & m_nSortAttribute);
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateByNoSort(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & 0x07) == 0);
}

void CLeftView::OnSortByName()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)((CNetExplorerDoc::SortAscending & m_nSortAttribute) | (0x07 & CNetExplorerDoc::SortByName));
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateSortByName(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & CNetExplorerDoc::SortByName) != 0);
}

void CLeftView::OnSortByExtension()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)((CNetExplorerDoc::SortAscending & m_nSortAttribute) | (0x07 & CNetExplorerDoc::SortByExtension));
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateSortByExtension(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & CNetExplorerDoc::SortByExtension) != 0);
}

void CLeftView::OnSortByDate()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)((CNetExplorerDoc::SortAscending & m_nSortAttribute) | (0x07 & CNetExplorerDoc::SortByDate));
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateSortByDate(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & CNetExplorerDoc::SortByDate) != 0);
}

void CLeftView::OnSortAscending()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)(CNetExplorerDoc::SortAscending | (0x07 & m_nSortAttribute));
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateSortAscending(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & CNetExplorerDoc::SortAscending) != 0);
}

void CLeftView::OnSortDescending()
{
	m_nSortAttribute = (CNetExplorerDoc::SortType)((0x07 & m_nSortAttribute));
	OnFilebrowserRefresh();
}

void CLeftView::OnUpdtateSortDescending(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck((m_nSortAttribute & CNetExplorerDoc::SortAscending) == 0);
}
