//    NetExplorer - Windows LAN explorer application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// LeftView.h : interface of the CLeftView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_LEFTVIEW_H__INCLUDED_)
#define AFX_LEFTVIEW_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxcview.h>
#include "NetExplorerDoc.h"

class CMultiTree;

class CLeftView : public CTreeView
{
	friend class CFileBrowserDialogBar;
public: // create from serialization only
	CLeftView();
	virtual ~CLeftView();

	DECLARE_DYNCREATE(CLeftView)

protected:
	CNetExplorerDoc* GetDoc(); //GetDocument();
	CNetExplorerDoc* m_pDoc; //GetDocument();
	CMultiTree * m_pTree;
	CImageList m_SmallImageList;

	DWORD m_dwCurrentFile;
	CImageList m_ImageList;

	CImageList* m_pDragImage;	//For creating and managing the drag-image
	BOOL		m_bDragging;	//T during a drag operation
	int			m_nDragIndex;	//Index of selected item in the List we are dragging FROM
	int			m_nDropIndex;	//Index at which to drop item in the List we are dropping ON

// Operations
public:
	//over-scope base version
	CMultiTree & GetMultiTreeCtrl() const {return *m_pTree;};
	CTreeCtrl & GetTreeCtrl() const {return *((CTreeCtrl*)m_pTree);};

	void Find(LPCTSTR pszFindPath);
	static void OnSearchOne(CTreeNodeList * pTreeNodeList, LPARAM lParam);

	void ListCtrlSelectionChanging(CTreeNode * pTreeNode);
	HTREEITEM FindItemByName(LPCTSTR pszName, HTREEITEM hParentItem);
	HTREEITEM m_hLastMenuDropTreeItem;

	static BOOL EnumerateDesktop(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode);
	static BOOL EnumerateServer(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode);
	static BOOL EnumerateFiles(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode);

	void RefreshEnumerate(HTREEITEM hParentItem);
	static BOOL RefreshEnumerateElements(CTreeNode * pTreeNode, CTreeNode * pParentTreeNode);
	static CMap<CString, LPCTSTR, CTreeNode *, CTreeNode *> m_mapTreeNodeEnum;

	static int CALLBACK CompareTreeProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
	enum CNetExplorerDoc::SortType m_nSortAttribute;

protected:
	virtual void OnInitialUpdate(); // called first time after construct
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	virtual void AllocEmbeddedTree();
	virtual void PostNcDestroy();
public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL OnChildNotify(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pLResult);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
	//virtual BOOL PreTranslateMessage(MSG* pMsg);
	void OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex,BOOL bSysMenu);

	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnItemexpanding(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeleteitem(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnNMDblclk(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnRclick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnUpdateTreeExplore(CCmdUI* pCmdUI);
	afx_msg void OnUpdateTreeExpand(CCmdUI* pCmdUI);
	afx_msg void OnTreeExpand();
	afx_msg void OnClick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnMRUDirectoryHandler(UINT i);
	afx_msg void OnUpdateMRUDirectoryHandler(CCmdUI* pCmdUI);
	afx_msg void OnMRUFileHandler(UINT i);
	afx_msg void OnUpdateMRUFileHandler(CCmdUI* pCmdUI);
	afx_msg void OnFilebrowserPlay();
	afx_msg void OnUpdateFilebrowserPlay(CCmdUI *pCmdUI);
	afx_msg void OnFilebrowserEnqueue();
	afx_msg void OnUpdateFilebrowserEnqueue(CCmdUI *pCmdUI);
	afx_msg void OnTvnBegindrag(NMHDR *pNMHDR, LRESULT *pResult);
//	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
//	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	afx_msg void OnFilebrowserSynchroniseCurrentFile();
	afx_msg void OnUpdateFilebrowserSynchroniseCurrentFile(CCmdUI *pCmdUI);
	afx_msg void OnFilebrowserRefresh();
	afx_msg void OnSortByNoSort();
	afx_msg void OnUpdtateByNoSort(CCmdUI *pCmdUI);
	afx_msg void OnSortByName();
	afx_msg void OnUpdtateSortByName(CCmdUI *pCmdUI);
	afx_msg void OnSortByExtension();
	afx_msg void OnUpdtateSortByExtension(CCmdUI *pCmdUI);
	afx_msg void OnSortByDate();
	afx_msg void OnUpdtateSortByDate(CCmdUI *pCmdUI);
	afx_msg void OnSortAscending();
	afx_msg void OnUpdtateSortAscending(CCmdUI *pCmdUI);
	afx_msg void OnSortDescending();
	afx_msg void OnUpdtateSortDescending(CCmdUI *pCmdUI);
	DECLARE_MESSAGE_MAP()
};

#endif // !defined(AFX_LEFTVIEW_H__INCLUDED_)
