//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// MPlayerX.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "MPlayerX.h"

#include "MainFrm.h"
#include "PreviewScreenSaverWnd.h"
//{{AFX_INCLUDES()
#include "webbrowser2.h"
#include "afxwin.h"
//}}AFX_INCLUDES

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

LPCTSTR AfxGetExecutableName()
{
	static char pszFilename[1024];
	HINSTANCE hInstance = GetModuleHandle(NULL);
	if(GetModuleFileName(hInstance, pszFilename, sizeof(pszFilename)))
		return pszFilename;

	return NULL;
}

BOOL AfxGetProfileFont(LPCTSTR lpszKey, LPCTSTR lpszVal, CFont& font, CDC* pDC)
{
	CWinApp *pApp = AfxGetApp();
	ASSERT_VALID(pApp);
	CString s = pApp->GetProfileString(lpszKey, lpszVal);
	if (s.IsEmpty())
		return FALSE;
	
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfCharSet = DEFAULT_CHARSET;
	int bItalic;
	int iPtSize;
	
	// scanf is overkill, but I'm lazy
	if (sscanf((LPCTSTR)s, "%[a-zA-Z ],%d,%d,%d",
		lf.lfFaceName, &iPtSize, &lf.lfWeight, &bItalic) != 4)
		return FALSE;
	lf.lfHeight = MulDiv(-iPtSize,   // convert ptsize to logical units
		::GetDeviceCaps(pDC ? pDC->m_hDC : ::GetDC(NULL), LOGPIXELSY), 72);
	lf.lfItalic = bItalic;  // because lf.lfItalic is a BYTE
	font.DeleteObject();    // bye 
	lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
	return font.CreateFontIndirect(&lf);
}

BOOL AfxWriteProfileFont(LPCTSTR lpszKey, LPCTSTR lpszVal, CFont& font, CDC* pDC)
{
	CWinApp *pApp = AfxGetApp();
	ASSERT_VALID(pApp);
	LOGFONT lf;
	font.GetLogFont(&lf);
	int iPtSize = MulDiv(-lf.lfHeight, 72,
		::GetDeviceCaps(pDC ? pDC->m_hDC : ::GetDC(NULL), LOGPIXELSY));
	CString s;
	s.Format("%s,%d,%d,%d", lf.lfFaceName, iPtSize, lf.lfWeight, lf.lfItalic);
	return pApp->WriteProfileString(lpszKey, lpszVal, s);
}

void AfxMessageBoxEx(LPCTSTR lpszText)
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
		pMainFrame->SetStatusTest(lpszText);
}

BOOL AfxDoEvents()
{
	MSG msg;
	while(::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
	{
		if(!AfxGetApp()->PumpMessage())
		{
			::PostQuitMessage(0);
			return FALSE;
		}
	}

	LONG lIdle = 0;
	while(AfxGetApp()->OnIdle(lIdle++));
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMPlayerXApp

BEGIN_MESSAGE_MAP(CMPlayerXApp, CWinApp)
	//{{AFX_MSG_MAP(CMPlayerXApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMPlayerXApp construction

CMPlayerXApp::CMPlayerXApp()
{
	m_fScreensaverStopped = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMPlayerXApp object

CMPlayerXApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMPlayerXApp initialization

BOOL CMPlayerXApp::InitInstance()
{
#define OPTION_FULLSCREEN "fullscreen"
#define OPTION_HELP1 "h"
#define OPTION_HELP2 "?"
#define OPTION_HELP3 "help"
#define OPTION_SCREENSAVER_SETTINGS "c"
#define OPTION_SCREENSAVER_PREVIEW "p"
#define OPTION_SCREENSAVER_SAVER "s"

	BOOL fFullscreen = FALSE;
	BOOL fScreenSaver = FALSE;
	enum { ScreenSaverNo, ScreenSaverSettings, ScreenSaverPreview, ScreenSaverSaver } nScreenSaverType = ScreenSaverNo;

	CStringArray aszMediaFile;

	for(int i = 1; i < __argc; i++)
	{
		if(__argv[i][0] == '/' || __argv[i][0] == '-')
		{
			if(strnicmp(__argv[i] + 1, OPTION_FULLSCREEN, strlen(OPTION_FULLSCREEN)) == 0)
			{
				fFullscreen = TRUE;
			}
			else if(strnicmp(__argv[i] + 1, OPTION_SCREENSAVER_SETTINGS, strlen(OPTION_SCREENSAVER_SETTINGS)) == 0)
			{
				fScreenSaver = TRUE;
				nScreenSaverType = ScreenSaverSettings;
			}
			else if(strnicmp(__argv[i] + 1, OPTION_SCREENSAVER_PREVIEW, strlen(OPTION_SCREENSAVER_PREVIEW)) == 0)
			{
				fScreenSaver = TRUE;
				nScreenSaverType = ScreenSaverPreview;
			}
			else if(strnicmp(__argv[i] + 1, OPTION_SCREENSAVER_SAVER, strlen(OPTION_SCREENSAVER_SAVER)) == 0)
			{
				fScreenSaver = TRUE;
				nScreenSaverType = ScreenSaverSaver;
			}
			else if(strnicmp(__argv[i] + 1, OPTION_HELP1, strlen(OPTION_HELP1)) == 0 ||
					strnicmp(__argv[i] + 1, OPTION_HELP2, strlen(OPTION_HELP2)) == 0 ||
					strnicmp(__argv[i] + 1, OPTION_HELP3, strlen(OPTION_HELP3)) == 0)
			{
				CString szTitle, szText;
				szTitle.LoadString(IDR_MAINFRAME);
				szText.LoadString(IDS_USAGE);
				MessageBox(NULL, szText, szTitle, MB_OK);
				return FALSE;
			}
		}
		else
			aszMediaFile.Add(__argv[i]);
	}


	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}
	AfxEnableControlContainer();
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	// of your final executable, you should remove from the following
	// the specific initialization routines you do not need
	// Change the registry key under which our settings are stored
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization

/*
#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif
*/
	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(IDS_STRING_REGISTRY_KEY);

/*	CString szExecutableName = AfxGetExecutableName();
	int nPos = szExecutableName.ReverseFind(_T('\\'));
	CString szPathName = szExecutableName.Left(nPos);
	SetCurrentDirectory(szPathName);
*/
	m_pMainWnd = NULL;

	if(fScreenSaver)
	{
		switch(nScreenSaverType)
		{
		case ScreenSaverSettings:
			DoSaverSetting();
			return FALSE;
		case ScreenSaverPreview:
			DoSaverPreview();
			return TRUE;
		case ScreenSaverSaver:
			DoScreenSaver();
			return TRUE;
		}
	}
	else
	{

		// Test if another instance already exist
		HANDLE hMutex = CreateMutex(NULL, FALSE, _T("LePlayerDuBonheur"));
		BOOL fAnotherInstanceExist = (GetLastError() == ERROR_ALREADY_EXISTS);
		BOOL fOnlyOneInstance = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("OnlyOneInstance"), TRUE);
		CString szClassName;
		szClassName.LoadString(IDS_CLASSNAME);

		int nCount = aszMediaFile.GetCount();

		if(nCount > 0 && fOnlyOneInstance && fAnotherInstanceExist)
		{
	        HWND hPreviousWnd = FindWindow(szClassName, NULL);
			if(hPreviousWnd)
			{
				TRACE("Found Previous Windows!!!!!!!!!!!!!!!!!!\n");

				CString szParameters = "";
				for(int i = 0; i < nCount; i++)
					szParameters += aszMediaFile[i] + "\n";

				COPYDATASTRUCT MyCDS;
				MyCDS.dwData = 217;          // function identifier
				MyCDS.cbData = szParameters.GetLength() + 1;  // size of data
				MyCDS.lpData = (PVOID)(LPCTSTR)szParameters;           // data structure
				
				// Call function, passing data in &MyCDS
				SendMessage(hPreviousWnd, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&MyCDS);

				return TRUE;
			}
		}

		// common initialization
		WNDCLASS wndcls;
		memset(&wndcls, 0, sizeof(WNDCLASS));   // start with NULL defaults
		wndcls.lpfnWndProc = ::DefWindowProc;
		wndcls.hInstance = AfxGetInstanceHandle();
		wndcls.hCursor = ::LoadCursor(NULL, IDC_ARROW);
		wndcls.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
		wndcls.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
		wndcls.lpszClassName = szClassName;
		wndcls.hIcon = ::LoadIcon(NULL, IDI_APPLICATION);
		AfxRegisterClass(&wndcls);

		// To create the main window, this code creates a new frame window
		// object and then sets it as the application's main window object.
		CMainFrame * pFrame = new CMainFrame;
		m_pMainWnd = pFrame;
		pFrame->m_hMutex = hMutex;

		// create and load the frame with its resources
		pFrame->LoadFrame(IDR_MAINFRAME,
			WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, NULL, NULL);

		// The one and only window has been initialized, so show and update it.
		pFrame->ShowWindow(SW_SHOW);
		pFrame->UpdateWindow();

		for(int i = 0; i < nCount; i++)
			pFrame->OpenFile(aszMediaFile[i], FALSE, i != 0, i == 0);
		if(nCount > 0)
			pFrame->PlayCurrent();

		if(fFullscreen)
			pFrame->OnViewFullscreen();

		SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0, &m_fScreensaverStopped, 0);
		if(m_fScreensaverStopped)
			SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, FALSE, 0, 0);
	}

	return TRUE;
}

int CMPlayerXApp::ExitInstance()
{
	if(m_fScreensaverStopped)
		SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, TRUE, 0, 0);

	return CWinApp::ExitInstance();
}

void CMPlayerXApp::DoSaverSetting()
{
	CMainFrame * pFrame = new CMainFrame;
	m_pMainWnd = pFrame;

	pFrame->m_fScreenSaverMode = TRUE;

	// create and load the frame with its resources
	pFrame->LoadFrame(IDR_MAINFRAME,
		WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, NULL, NULL);

	// The one and only window has been initialized, so show and update it.
	pFrame->OnPreferences();
	pFrame->DestroyWindow();
}

void CMPlayerXApp::DoSaverPreview()
{
	CWnd* pParent;
	if(__argc > 2)
		pParent = CWnd::FromHandle((HWND)atol(__argv[2]));
	else
		pParent = CWnd::FromHandle((HWND)NULL);

//	ASSERT(pParent != NULL);
	CPreviewScreenSaverWnd * pWnd = new CPreviewScreenSaverWnd();
	CRect rect;
	pParent->GetClientRect(&rect);
	pWnd->Create(NULL, WS_VISIBLE | WS_CHILD, rect, pParent, NULL);
	m_pMainWnd = pWnd;
}

void CMPlayerXApp::DoScreenSaver()
{
	CMainFrame * pFrame = new CMainFrame;
	m_pMainWnd = pFrame;

	pFrame->m_fScreenSaverMode = TRUE;

	// create and load the frame with its resources
	pFrame->LoadFrame(IDR_MAINFRAME,
		WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, NULL, NULL);

	// The one and only window has been initialized, so show and update it.
	pFrame->GoScreenSaver();
}

/////////////////////////////////////////////////////////////////////////////
// CMPlayerXApp message handlers

void CMPlayerXApp::OnUpdateRecentFileMenu(CCmdUI* pCmdUI)
{
	if (pCmdUI->m_pSubMenu!=NULL) // updating a submenu?
	{
		// update your submenu here, if you need to
		
		return;
	}
	
	CWinApp::OnUpdateRecentFileMenu(pCmdUI);
}



/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CWebBrowser2	m_ctrlWebBrowser;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	BOOL m_fAlreadyLoad;
	CString m_szEditHelp;
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnDocumentCompleteExplorer(LPDISPATCH pDisp, VARIANT FAR* URL);
	DECLARE_EVENTSINK_MAP()
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnSize(UINT nType, int cx, int cy);
	CButton m_wndButtonOk;
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_EXPLORER, m_ctrlWebBrowser);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDOK, m_wndButtonOk);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
	ON_WM_SIZE()
END_MESSAGE_MAP()

// App command to run the dialog
void CMPlayerXApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CMPlayerXApp message handlers

CString g_szHTMLTextHeader("<html><head><meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'> \
<title>Readme.txt</title> \
<style> \
body {background-color:#000000;color:#FFFFFF;} \
body {font-family:Arial;font-size:8pt;font-weight:600;border:thin;} \
tr {line-height:0.8;} \
td {padding-right:10;} \
</style> \
</head> \
<body id=body oncontextmenu='return false;'>");

CString g_szHTMLTextFooter("</body></html>");

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CString szResource;
	szResource.Format(_T("#%d"), IDR_HTML_HELP);
	LPCTSTR lpszName = szResource;
	HGLOBAL hGlobal;
	HRSRC hResource;
	LPCTSTR lpResLock;
	HINSTANCE hInst = AfxGetResourceHandle();
	if ((hResource = ::FindResource(hInst, lpszName, RT_HTML)) != NULL &&
		(hGlobal = ::LoadResource(hInst, hResource)) != 0 &&
		(lpResLock = (LPCTSTR)LockResource(hGlobal)) != NULL)
	{
		// found a non-zero string in app
		m_szEditHelp = (LPCTSTR)lpResLock;
	}

	m_szEditHelp.Replace(_T("\n"), _T("<br>"));
	m_szEditHelp = g_szHTMLTextHeader + m_szEditHelp + g_szHTMLTextFooter;

	m_fAlreadyLoad = FALSE;
	m_ctrlWebBrowser.Navigate(_T("about:blank"), NULL, NULL, NULL, NULL );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


BEGIN_EVENTSINK_MAP(CAboutDlg, CDialog)
    //{{AFX_EVENTSINK_MAP(CAboutDlg)
	ON_EVENT(CAboutDlg, IDC_EXPLORER, 259 /* DocumentComplete */, OnDocumentCompleteExplorer, VTS_DISPATCH VTS_PVARIANT)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CAboutDlg::OnDocumentCompleteExplorer(LPDISPATCH pDisp, VARIANT FAR* URL) 
{
	HRESULT hr;
	IUnknown* pUnkBrowser = NULL;
	IUnknown* pUnkDisp = NULL;
	IStream* pStream = NULL;
	HGLOBAL hHTMLText;
	
	LPDISPATCH pBrowser = m_ctrlWebBrowser.GetApplication();
	if(pBrowser != NULL)
	{
		// Is this the DocumentComplete event for the top frame window?
		// Check COM identity: compare IUnknown interface pointers.
		hr = pBrowser->QueryInterface( IID_IUnknown,  (void**)&pUnkBrowser );
		if ( SUCCEEDED(hr) )
		{
			hr = pDisp->QueryInterface( IID_IUnknown,  (void**)&pUnkDisp );
			if ( SUCCEEDED(hr) )
			{
				if ( pUnkBrowser == pUnkDisp && !m_fAlreadyLoad)
				{   // This is the DocumentComplete event for the top frame - page is loaded!
					m_fAlreadyLoad = TRUE;
					// Create a stream containing the HTML.
					// Alternatively, this stream may have been passed to us.
					hHTMLText = GlobalAlloc( GPTR, (lstrlen(m_szEditHelp)+1) * sizeof(TCHAR) );
					if ( hHTMLText )
					{
						lstrcpy( (TCHAR*)hHTMLText, m_szEditHelp );
						hr = CreateStreamOnHGlobal( hHTMLText, TRUE, &pStream );
						if ( SUCCEEDED(hr) )
						{
							IDispatch* pHtmlDoc = NULL;
							IPersistStreamInit* pPersistStreamInit = NULL;
							
							// Retrieve the document object.
							pHtmlDoc = m_ctrlWebBrowser.GetDocument();
							if (pHtmlDoc)
							{
								// Query for IPersistStreamInit.
								hr = pHtmlDoc->QueryInterface( IID_IPersistStreamInit,  (void**)&pPersistStreamInit );
								if ( SUCCEEDED(hr) )
								{
									// Initialize the document.
									hr = pPersistStreamInit->InitNew();
									if ( SUCCEEDED(hr) )
									{
										// Load the contents of the stream.
										hr = pPersistStreamInit->Load( pStream );
									}
									pPersistStreamInit->Release();
								}
							}
							
							pStream->Release();
						}
					}
				}
				pUnkDisp->Release();
			}
			pUnkBrowser->Release();
		}
		pBrowser->Release();
	}
}

void CAboutDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	CRect rect;
	m_wndButtonOk.GetClientRect(&rect);
	m_wndButtonOk.MoveWindow(cx - rect.Width() - 5, 5, rect.Width(), rect.Height());
	m_ctrlWebBrowser.GetWindowRect(&rect);
	ScreenToClient(&rect);
	rect.right = cx - rect.left;
	rect.bottom = cy - rect.left;
	m_ctrlWebBrowser.MoveWindow(&rect);
}
