//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "MPlayerX.h"

#include "MainFrm.h"
#include "SettingsDlg.h"
#include "SettingsGeneralPropertyPage.h"
#include "SettingsScreensaverPropertyPage.h"
#include "SettingsOSDPropertyPage.h"
#include "SettingsSubtitlePropertyPage.h"
#include "SettingsTimeLinePropertyPage.h"
#include "SettingsLircPropertyPage.h"
#include "RecentFileListEx.h"
#include "delaytransformfilter.h"
#include "MltiTree.h"
#include "NetExplorerDoc.h"

#ifdef _DEBUG
#define COMPILE_MULTIMON_STUBS
#endif
#include "multimon.h"    
#include "Mpconfig.h"
#include <winable.h>

#ifndef WS_EX_NOACTIVATE
#define WS_EX_NOACTIVATE        0x08000000L
#endif

#define TIMER_FRAME_SEEKING 217
#define TIMER_FAST_FORWARD_SEEKING 218
#define TIMER_SLIDESHOW 219

// An application can advertise the existence of its filter graph
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_STATUS,
	ID_INDICATOR_TIME
};

// HKEY_USERS\.DEFAULT\Control Panel\Desktop
// HKEY_CURRENT_USER\Control Panel\Desktop\SCRNSAVE.EXE
// HKEY_CURRENT_USER\Control Panel\Desktop\ScreenSaveTimeOut second
// http://codeguru.earthweb.com/misc/mfcsaver.shtml

typedef struct RemoteCommandParameter2
{
	TCHAR * pszRemoteCommandName;
	BOOL fAcceptRepeat;
	UINT nWndCommand;
} RemoteCommandParameter2;

RemoteCommandParameter2 g_arcp[] =
{
	{ _T("VCR_p-"), FALSE, IDM_CONTROL_PREVIOUS_FILE },
	{ _T("VCR_p+"), FALSE, IDM_CONTROL_NEXT_FILE },
	{ _T("VCR_fast_rewind"), FALSE, IDM_CONTROL_FAST_REWIND },
	{ _T("VCR_fast_forward"), FALSE, IDM_CONTROL_FAST_FORWARD },
	{ _T("VCR_play"), FALSE, IDM_CONTROL_PLAY_PAUSE },
	{ _T("VCR_stop"), FALSE, IDM_CONTROL_STOP },
	{ _T("VCR_record"), FALSE, IDM_CONTROL_STEP_BY_STEP },
	{ _T("VCR_AB"), FALSE, IDM_CONTROL_REPEAT_AB },
	{ _T("VCR_3"), FALSE, IDM_CONTROL_SPEED_PLUS },
	{ _T("VCR_6"), FALSE, IDM_CONTROL_SPEED_NORMAL },
	{ _T("VCR_9"), FALSE, IDM_CONTROL_SPEED_MOINS },
	{ _T("mute"), FALSE, IDM_VIEW_MUTE },
	{ _T("vol-"), TRUE, IDM_VIEW_VOLUME_MOINS },
	{ _T("vol+"), TRUE, IDM_VIEW_VOLUME_PLUS },
	{ _T("bal_left"), TRUE, IDM_VIEW_BALANCE_LEFT },
	{ _T("bal_right"), TRUE, IDM_VIEW_BALANCE_RIGHT },
	{ _T("VCR_av"), FALSE, IDM_VIEW_SIZE_WINDOW_FIT },
	{ _T("sys_05_command_0c"), FALSE, IDM_VIEW_FULLSCREEN },
	{ _T("VCR_2"), TRUE, IDM_VIEW_ZOOM_PLUS },
	{ _T("VCR_5"), FALSE, IDM_VIEW_ZOOM_RESET },
	{ _T("VCR_8"), TRUE, IDM_VIEW_ZOOM_MOINS },
	{ _T("zoom_half"), FALSE, IDM_VIEW_SIZE_HALF },
	{ _T("zoom_34"), FALSE, IDM_VIEW_SIZE_THREEQUARTER },
	{ _T("zoom_normal"), FALSE, IDM_VIEW_SIZE_NORMAL },
	{ _T("zoom_double"), FALSE, IDM_VIEW_SIZE_DOUBLE },
	{ _T("maintain_ar"), FALSE, IDM_VIEW_MAINTAIN_ASPECT_RATIO },
	{ _T("VCR_menu"), FALSE, IDM_VIEW_169 },
	{ _T("VCR_1"), TRUE, IDM_VIEW_ASPECT_RATIO_PLUS },
	{ _T("VCR_4"), FALSE, IDM_VIEW_ASPECT_RATIO_RESET },
	{ _T("VCR_7"), TRUE, IDM_VIEW_ASPECT_RATIO_MOINS },
	{ _T("VCR_0"), FALSE, IDM_VIEW_CENTER },
	{ _T("VCR_down"), TRUE, IDM_VIEW_SCROLL_DOWN },
	{ _T("VCR_up"), TRUE, IDM_VIEW_SCROLL_UP },
	{ _T("VCR_left"), TRUE, IDM_VIEW_SCROLL_LEFT },
	{ _T("VCR_right"), TRUE, IDM_VIEW_SCROLL_RIGHT },
	{ _T("VCR_reset"), FALSE, IDM_VIEW_SCROLL_RESET },
	{ _T("time_line"), FALSE, IDM_VIEW_TIMELINE },
	{ _T("language"), FALSE, IDM_CONTROL_LANGUAGE_TOGGLE },
	{ _T("delay_no"), FALSE, IDM_CONTROL_DELAY_NO },
	{ _T("delay_plus"), TRUE, IDM_CONTROL_DELAY_PLUS },
	{ _T("delay_moins"), TRUE, IDM_CONTROL_DELAY_MOINS },
	{ _T("osd"), FALSE, IDM_VIEW_OSD },
/*	{ _T("openclip"), FALSE, ID_FILE_OPENCLIP },
	{ _T("addclip"), FALSE, ID_FILE_ADDCLIP },
	{ _T("closeclip"), FALSE, ID_FILE_CLOSE_CLIP },
	{ _T("closeselclip"), FALSE, ID_FILE_CLOSE_SELECTED_CLIP },
	{ _T("closeallclip"), FALSE, ID_FILE_CLOSE_ALL_CLIP },
*/
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_ACTIVATE()
	ON_WM_CLOSE()
	ON_WM_CREATE()
	ON_WM_DROPFILES()
	ON_WM_ERASEBKGND()
	ON_WM_HSCROLL()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_NCACTIVATE()
	ON_WM_RBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_WM_SYSCOMMAND()
	ON_WM_SYSKEYDOWN()
	ON_WM_TIMER()
	ON_COMMAND(ID_FILE_ADD_SUBTITLE, OnFileAddSubtitle)
	ON_UPDATE_COMMAND_UI(ID_FILE_ADD_SUBTITLE, OnUpdateFileAddSubtitle)
	ON_COMMAND(ID_FILE_ADDCLIP, OnFileAddclip)
	ON_UPDATE_COMMAND_UI(ID_FILE_ADDCLIP, OnUpdateFileAddclip)
	ON_COMMAND(ID_FILE_CLOSE_ALL_CLIP, OnFileCloseAllClip)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE_ALL_CLIP, OnUpdateFileCloseAllClip)
	ON_COMMAND(ID_FILE_CLOSE_CLIP, OnFileCloseClip)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE_CLIP, OnUpdateFileCloseClip)
	ON_COMMAND(ID_FILE_CLOSE_SELECTED_CLIP, OnFileCloseSelectedClip)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE_SELECTED_CLIP, OnUpdateFileCloseSelectedClip)
	ON_COMMAND(ID_FILE_OPENCLIP, OnFileOpenclip)
	ON_COMMAND(ID_FILE_PLAYLIST_ADD, OnFilePlaylistAdd)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAYLIST_ADD, OnUpdateFilePlaylistAdd)
	ON_COMMAND(ID_FILE_PLAYLIST_LOAD, OnFilePlaylistLoad)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAYLIST_LOAD, OnUpdateFilePlaylistLoad)
	ON_COMMAND(ID_FILE_PLAYLIST_SAVE, OnFilePlaylistSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAYLIST_SAVE, OnUpdateFilePlaylistSave)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CONTROL_BAR, OnUpdateViewControlBar)
	ON_COMMAND(ID_VIEW_CONTROL_BAR, OnViewControlBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FILE_BROWSER_BAR, OnUpdateViewFileBrowserBar)
	ON_COMMAND(ID_VIEW_FILE_BROWSER_BAR, OnViewFileBrowserBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLAY_LIST_BAR, OnUpdateViewPlayListBar)
	ON_COMMAND(ID_VIEW_PLAY_LIST_BAR, OnViewPlayListBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SEEKING_BAR, OnUpdateViewSeekingBar)
	ON_COMMAND(ID_VIEW_SEEKING_BAR, OnViewSeekingBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SUBTITLE_BAR, OnUpdateViewSubtitleBar)
	ON_COMMAND(ID_VIEW_SUBTITLE_BAR, OnViewSubtitleBar)
	ON_COMMAND(IDM_CONTROL_DELAY_MOINS, OnControlDelayMoins)
	ON_COMMAND(IDM_CONTROL_DELAY_NO, OnControlDelayNo)
	ON_COMMAND(IDM_CONTROL_DELAY_PLUS, OnControlDelayPlus)
	ON_COMMAND(IDM_CONTROL_FAST_FORWARD, OnControlFastForward)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_FAST_FORWARD, OnUpdateControlFastForward)
	ON_COMMAND(IDM_CONTROL_FAST_REWIND, OnControlFastRewind)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_FAST_REWIND, OnUpdateControlFastRewind)
	ON_COMMAND_RANGE(IDM_CONTROL_LANGUAGE1, IDM_CONTROL_LANGUAGE20, OnControlLanguage)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_CONTROL_LANGUAGE1, IDM_CONTROL_LANGUAGE20, OnUpdateControlLanguage)
	ON_COMMAND(IDM_CONTROL_LANGUAGE_TOGGLE, OnControlLanguageToggle)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_LANGUAGE_TOGGLE, OnUpdateControlLanguageToggle)
	ON_COMMAND(IDM_CONTROL_OSD_TEST, OnControlOsdTest)
	ON_COMMAND(IDM_CONTROL_PLAY_PAUSE, OnControlPlayPause)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_PLAY_PAUSE, OnUpdateControlPlayPause)
	ON_COMMAND(IDM_CONTROL_PLAY_RANDOM, OnControlPlayRandom)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_PLAY_RANDOM, OnUpdateControlPlayRandom)
	ON_COMMAND(IDM_CONTROL_PREVIOUS_FILE, OnControlPreviousFile)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_PREVIOUS_FILE, OnUpdateControlPreviousFile)
	ON_COMMAND(IDM_CONTROL_NEXT_FILE, OnControlNextFile)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_NEXT_FILE, OnUpdateControlNextFile)
	ON_COMMAND(IDM_CONTROL_PREVIOUS_FOLDER_FILE, OnControlPreviousFolderFile)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_PREVIOUS_FOLDER_FILE, OnUpdateControlPreviousFolderFile)
	ON_COMMAND(IDM_CONTROL_NEXT_FOLDER_FILE, OnControlNextFolderFile)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_NEXT_FOLDER_FILE, OnUpdateControlNextFolderFile)
	ON_COMMAND_RANGE(IDM_CONTROL_PROPERTY1, IDM_CONTROL_PROPERTY20, OnControlProperty)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_CONTROL_PROPERTY1, IDM_CONTROL_PROPERTY20, OnUpdateControlProperty)
	ON_COMMAND(IDM_CONTROL_REPEAT_AB, OnControlRepeatAb)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_REPEAT_AB, OnUpdateControlRepeatAb)
	ON_COMMAND(IDM_CONTROL_SPEED_MOINS, OnControlSpeedMoins)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_SPEED_MOINS, OnUpdateControlSpeedMoins)
	ON_COMMAND(IDM_CONTROL_SPEED_NORMAL, OnControlSpeedNormal)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_SPEED_NORMAL, OnUpdateControlSpeedNormal)
	ON_COMMAND(IDM_CONTROL_SPEED_PLUS, OnControlSpeedPlus)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_SPEED_PLUS, OnUpdateControlSpeedPlus)
	ON_COMMAND(IDM_CONTROL_STEP_BY_STEP, OnControlStepByStep)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_STEP_BY_STEP, OnUpdateControlStepByStep)
	ON_COMMAND(IDM_CONTROL_STOP, OnControlStop)
	ON_UPDATE_COMMAND_UI(IDM_CONTROL_STOP, OnUpdateControlStop)
	ON_COMMAND_RANGE(IDM_MRU_DIR1, IDM_MRU_DIR16, OnMRUDirectoryHandler) 
	ON_UPDATE_COMMAND_UI_RANGE(IDM_MRU_DIR1, IDM_MRU_DIR16, OnUpdateMRUDirectoryHandler)
	ON_COMMAND_RANGE(IDM_MRU_FILE1, IDM_MRU_FILE16, OnMRUFileHandler) 
	ON_UPDATE_COMMAND_UI_RANGE(IDM_MRU_FILE1, IDM_MRU_FILE16, OnUpdateMRUFileHandler)
	ON_COMMAND(IDM_MRU_FILE_CLEAN, OnMruFileClean)
	ON_UPDATE_COMMAND_UI(IDM_MRU_FILE_CLEAN, OnUpdateMruFileClean)
	ON_COMMAND(IDM_PREFERENCES, OnPreferences)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_169, OnUpdateView169)
	ON_COMMAND(IDM_VIEW_169, OnView169)
	ON_COMMAND(IDM_VIEW_ASPECT_RATIO_MOINS, OnViewAspectRatioMoins)
	ON_COMMAND(IDM_VIEW_ASPECT_RATIO_PLUS, OnViewAspectRatioPlus)
	ON_COMMAND(IDM_VIEW_ASPECT_RATIO_RESET, OnViewAspectRatioReset)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_BALANCE_LEFT, OnUpdateViewBalanceLeft)
	ON_COMMAND(IDM_VIEW_BALANCE_LEFT, OnViewBalanceLeft)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_BALANCE_RIGHT, OnUpdateViewBalanceRight)
	ON_COMMAND(IDM_VIEW_BALANCE_RIGHT, OnViewBalanceRight)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_CENTER, OnUpdateViewCenter)
	ON_COMMAND(IDM_VIEW_CENTER, OnViewCenter)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_DESKTOP, OnUpdateViewDesktop)
	ON_COMMAND(IDM_VIEW_DESKTOP, OnViewDesktop)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_FULLSCREEN, OnUpdateViewFullscreen)
	ON_COMMAND(IDM_VIEW_FULLSCREEN, OnViewFullscreen)
	ON_COMMAND(IDM_VIEW_FULLSCREEN_OFF, OnViewFullscreenOff)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_MAINTAIN_ASPECT_RATIO, OnUpdateViewMaintainAspectRatio)
	ON_COMMAND(IDM_VIEW_MAINTAIN_ASPECT_RATIO, OnViewMaintainAspectRatio)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_MUTE, OnUpdateViewMute)
	ON_COMMAND(IDM_VIEW_MUTE, OnViewMute)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_OSD, OnUpdateViewOsd)
	ON_COMMAND(IDM_VIEW_OSD, OnViewOsd)
	ON_COMMAND(IDM_VIEW_SCROLL_DOWN, OnViewScrollDown)
	ON_COMMAND(IDM_VIEW_SCROLL_LEFT, OnViewScrollLeft)
	ON_COMMAND(IDM_VIEW_SCROLL_RESET, OnViewScrollReset)
	ON_COMMAND(IDM_VIEW_SCROLL_RIGHT, OnViewScrollRight)
	ON_COMMAND(IDM_VIEW_SCROLL_UP, OnViewScrollUp)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_SIZE_DOUBLE, OnUpdateViewSizeDouble)
	ON_COMMAND(IDM_VIEW_SIZE_DOUBLE, OnViewSizeDouble)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_SIZE_HALF, OnUpdateViewSizeHalf)
	ON_COMMAND(IDM_VIEW_SIZE_HALF, OnViewSizeHalf)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_SIZE_NORMAL, OnUpdateViewSizeNormal)
	ON_COMMAND(IDM_VIEW_SIZE_NORMAL, OnViewSizeNormal)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_SIZE_THREEQUARTER, OnUpdateViewSizeThreequarter)
	ON_COMMAND(IDM_VIEW_SIZE_THREEQUARTER, OnViewSizeThreequarter)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_SIZE_WINDOW_FIT, OnUpdateViewSizeWindowFit)
	ON_COMMAND(IDM_VIEW_SIZE_WINDOW_FIT, OnViewSizeWindowFit)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_TIMELINE, OnUpdateViewTimeline)
	ON_COMMAND(IDM_VIEW_TIMELINE, OnViewTimeline)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_TOPMOST, OnUpdateViewTopmost)
	ON_COMMAND(IDM_VIEW_TOPMOST, OnViewTopmost)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_USE_OVERLAY, OnUpdateViewUseOverlay)
	ON_COMMAND(IDM_VIEW_USE_OVERLAY, OnViewUseOverlay)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_VOLUME_MOINS, OnUpdateViewVolumeMoins)
	ON_COMMAND(IDM_VIEW_VOLUME_MOINS, OnViewVolumeMoins)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_VOLUME_PLUS, OnUpdateViewVolumePlus)
	ON_COMMAND(IDM_VIEW_VOLUME_PLUS, OnViewVolumePlus)
	ON_COMMAND(IDM_VIEW_ZOOM_MOINS, OnViewZoomMoins)
	ON_COMMAND(IDM_VIEW_ZOOM_PLUS, OnViewZoomPlus)
	ON_COMMAND(IDM_VIEW_ZOOM_RESET, OnViewZoomReset)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXTA, 0, 0xFFFF, OnToolTipText)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXTW, 0, 0xFFFF, OnToolTipText)
	ON_MESSAGE(WM_SETTINGSDIALOG_CLOSE, OnCloseSettingDialog)
	ON_MESSAGE(WM_TRAY_ICON_NOTIFY_MESSAGE, OnTrayNotify)
	//}}AFX_MSG_MAP
	ON_WM_MOVE()
	ON_COMMAND(ID_FILEBROWSER_SYNCHRONISE_CURRENT_FILE, OnFilebrowserSynchroniseCurrentFile)
	ON_COMMAND(ID_FILEBROWSER_REFRESH, OnFilebrowserRefresh)
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MOUSEWHEEL()
	ON_WM_INITMENU()
	END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame() :
	m_SubtitleManager(m_MediaTime)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_szFileName = _T("");
	m_bAudioOnly = FALSE;
	m_dwGraphRegister = 0;
	m_enCurrentPlayState = PSUnload;

	// DirectShow interfaces
	m_pGraphBuilder = NULL;
	m_pMediaControl = NULL;
	m_pMediaEventEx = NULL;
	m_pVMRWindowlessControl = NULL;
	m_pVideoWindow = NULL;
	m_pBasicAudio = NULL;
	m_pBasicVideo = NULL;
	m_pMediaSeeking = NULL;

	m_pAMStreamSelect = NULL;
	m_pDelayTransformFilter = NULL;

	m_nFastForwardSeekingTime = 1000;
	m_hFastForwardSeekingTimer = NULL;
	m_hFrameSeekingTimer = NULL;
	m_hSlideShowTimer = NULL;

	m_pSettingsDlg = NULL;
	m_hMutex = NULL;

	m_nidIconData.cbSize			= sizeof(NOTIFYICONDATA);
	m_nidIconData.hWnd				= 0;
	m_nidIconData.uID				= 1;
	m_nidIconData.uCallbackMessage	= WM_TRAY_ICON_NOTIFY_MESSAGE;
	m_nidIconData.hIcon				= 0;
	m_nidIconData.szTip[0]			= 0;	
	m_nidIconData.uFlags			= NIF_MESSAGE;

	m_fScreenSaverMode = FALSE;
	m_fScreenSaverNoDisturb = FALSE;
	m_fLircInitialize = FALSE;
	m_fLircSupport = TRUE;

	m_nDelay = 0;
	m_nRepeatABMode = 0;
	m_fFullscreen = FALSE;
	m_fDesktopMode = FALSE;
	m_colorOldValues = 0;
	m_fColorKeyDirty = TRUE;
	m_fMaintainAspectRatio = TRUE;
	m_fWindowFit = TRUE;
	m_fWindowWidthFit = FALSE;
	m_fRatio169 = FALSE;
	m_rCurrentAspectRatio = 0.0;
	m_fViewCenter = TRUE;
	m_fViewTimeLine = FALSE;
	m_fShowDefaultOSD = TRUE;
	m_fOverlayUsed = FALSE;
	m_fUseVMR = FALSE;
	m_fPlayListMode = FALSE;

	m_rZoom = 1.0;
	m_rZoomStep = 1.0250;
	m_rAspectRatioStep = 1.0125;
	m_nOffsetCenterX = 0;
	m_nOffsetCenterY = 0;
	m_nOffsetStepSlow = 1;
	m_nOffsetStepNormal = 5;

#ifdef COMPILE_MULTIMON_STUBS
	m_fMultipleMonitorSupported = InitMultipleMonitorStubs();
#else
	m_fMultipleMonitorSupported = TRUE;
#endif
	m_pRecentFileList = new CRecentFileListEx(IDM_MRU_FILE1, _T("MRUFile"), _T("File%d"), 16);
	m_pRecentDirectoryList = new  CRecentFileListEx(IDM_MRU_DIR1, _T("MRUDir"), _T("Dir%d"), 16);
	m_fRecentDirectoryListForFileBrowserDlgBar = FALSE;
}

CMainFrame::~CMainFrame()
{
//	if(m_fLircSupport)
//		m_RemoteControl.Stop();

	if(m_pSettingsDlg)
		delete m_pSettingsDlg;

	if(m_pRecentFileList)
		delete m_pRecentFileList;
	if(m_pRecentDirectoryList)
		delete m_pRecentDirectoryList;
}

BOOL CMainFrame::GoScreenSaver()
{
	m_fScreenSaverMode = TRUE;
	m_fPlayListMode = TRUE;
	OnViewFullscreen();

	int nFileCount = 0;
	CStdioFile file;
	if(file.Open(m_SettingsCurrent.m_szScreensaverPlaylistFile, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
	{
		BOOL fFirst = TRUE;
		CString szLine;
		while(file.ReadString(szLine))
		{
			szLine.TrimLeft();
			szLine.TrimRight();
			if(szLine != _T(""))
			{
				m_wndPlayListDlgBar.AddFileToTail(szLine, fFirst);
				nFileCount++;
				fFirst = FALSE;
			}
		}
		file.Close();
	}

	LPCTSTR pszCurrentFile = m_wndPlayListDlgBar.GetFile(AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverCurrentItem"), 0));
	if(pszCurrentFile)
		m_wndPlayListDlgBar.SetCurrentFile(pszCurrentFile);
	else
		m_wndPlayListDlgBar.SetCurrentFile(m_wndPlayListDlgBar.GetCurrentFile());
	if(FAILED(PlayMovieInWindow(m_wndPlayListDlgBar.GetCurrentFile())))
		while(!PlayNextFile() && --nFileCount);

	return TRUE;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	Preferences(TRUE);
	m_wndFileBrowserDlgBar.m_LeftView.m_nSortAttribute = (CNetExplorerDoc::SortType)m_SettingsCurrent.m_nSortAttribute;

	WINDOWPLACEMENT * pWindowPlacement = NULL;
	unsigned int nCount;
	AfxGetApp()->GetProfileBinary(INI_SETTINGS_MISC, _T("WindowPlacement"), (LPBYTE *)&pWindowPlacement, &nCount);
	if(nCount == sizeof(WINDOWPLACEMENT))
		SetWindowPlacement(pWindowPlacement);
	if(pWindowPlacement != NULL)
		delete [] pWindowPlacement;

	if(m_SettingsCurrent.m_fTopMost)
		SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);


	DragAcceptFiles(TRUE);

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, FALSE);		// Set small icon
	SetIcon(m_hIcon, TRUE);			// Set big icon

	m_nidIconData.hWnd = m_hWnd;
	m_nidIconData.uID = 1;
	m_nidIconData.uFlags |= NIF_TIP | NIF_ICON;
	m_nidIconData.hIcon = m_hIcon;
	_tcscpy(m_nidIconData.szTip, AfxGetAppName());


	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}

	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT,
			WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_GRIPPER |
			CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	CString szString;
	szString.LoadString(IDS_STRING_SEEKING_BAR);
	if(!m_wndSeekingDlgBar.Create(szString, this, IDR_SEEKING_DIALOGBAR,
		WS_CHILD | WS_VISIBLE | CBRS_ALIGN_TOP | CBRS_GRIPPER |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC))
	{
		TRACE(_T("Failed to create ControlDlgBar\n"));
		return -1;
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndStatusBar.SetPaneInfo(m_wndStatusBar.CommandToIndex(ID_INDICATOR_TIME), ID_INDICATOR_TIME,
		SBPS_NOBORDERS, 150);
	m_wndStatusBar.SetPaneInfo(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), ID_INDICATOR_STATUS,
		SBPS_NOBORDERS, 300);

	szString.LoadString(IDS_STRING_CONTROL_BAR);
	if(!m_wndControlDlgBar.Create(szString, this, IDC_DIALOG_BAR_CONTROL))
	{
		TRACE(_T("Failed to create ControlDlgBar\n"));
		return -1;
	}
	szString.LoadString(IDS_STRING_PLAYLIST_BAR);
	if(!m_wndPlayListDlgBar.Create(szString, this, IDC_DIALOG_BAR_PLAY_LIST))
	{
		TRACE(_T("Failed to create PlayListDlgBar\n"));
		return -1;
	}
	szString.LoadString(IDS_STRING_FILEBROWSER_BAR);
	if(!m_wndFileBrowserDlgBar.Create(szString, this, IDC_DIALOG_BAR_FILE_BROWSER))
	{
		TRACE(_T("Failed to create FileBrowserDlgBar\n"));
		return -1;
	}
	szString.LoadString(IDS_STRING_SUBTITLE_BAR);
	if(!m_wndSubtitleDlgBar.Create(szString, this, IDC_DIALOG_BAR_SUBTITLE))
	{
		TRACE(_T("Failed to create SubtitleDlgBar\n"));
		return -1;
	}

	m_wndControlDlgBar.SetBarStyle(m_wndControlDlgBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndPlayListDlgBar.SetBarStyle(m_wndPlayListDlgBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndFileBrowserDlgBar.SetBarStyle(m_wndFileBrowserDlgBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndSubtitleDlgBar.SetBarStyle(m_wndSubtitleDlgBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndSeekingDlgBar.EnableDocking(CBRS_ALIGN_TOP | CBRS_ALIGN_BOTTOM);
	m_wndControlDlgBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndPlayListDlgBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndFileBrowserDlgBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndSubtitleDlgBar.EnableDocking(CBRS_ALIGN_ANY);

	EnableDocking(CBRS_ALIGN_ANY);

	DockControlBar(&m_wndToolBar, AFX_IDW_DOCKBAR_TOP);
	DockControlBar(&m_wndSeekingDlgBar, AFX_IDW_DOCKBAR_BOTTOM);
	DockControlBar(&m_wndControlDlgBar, AFX_IDW_DOCKBAR_RIGHT);
	DockControlBar(&m_wndPlayListDlgBar, AFX_IDW_DOCKBAR_LEFT);
	DockControlBar(&m_wndFileBrowserDlgBar, AFX_IDW_DOCKBAR_LEFT);
	DockControlBar(&m_wndSubtitleDlgBar, AFX_IDW_DOCKBAR_RIGHT);

	m_wndSeekingDlgBar.m_wndSliderBar.EnableWindow(FALSE);

	CString sProfile(_T("BarState"));
    if(VerifyBarState(sProfile))
    {
        LoadBarState(sProfile);
		m_wndControlDlgBar.LoadState(sProfile);
		m_wndPlayListDlgBar.LoadState(sProfile);
		m_wndFileBrowserDlgBar.LoadState(sProfile);
		m_wndSubtitleDlgBar.LoadState(sProfile);
    }

	CMenu * pMenu = GetMenu();
	if(pMenu)
		UpdateMenu(pMenu);

	if(m_pRecentFileList)
		m_pRecentFileList->ReadList();
	if(m_pRecentDirectoryList)
		m_pRecentDirectoryList->ReadList();

	if(m_fLircSupport = (m_SettingsCurrent.m_szGirderPluginFilename != _T("")))
	{
		if(m_RemoteControl.Init(m_hWnd, m_SettingsCurrent.m_szGirderPluginFilename))
		{
			m_fLircInitialize = TRUE;
			if(!m_RemoteControl.Start())
			{
				CString szString;
				szString.LoadString(IDS_STRING_GIRDER_CANNOT_START);
				AfxMessageBoxEx(szString + m_SettingsCurrent.m_szGirderPluginFilename);
				m_fLircSupport = FALSE;
			}
		}
		else
		{
			CString szString;
			szString.LoadString(IDS_STRING_GIRDER_CANNOT_INITIALIZE);
			AfxMessageBoxEx(szString + m_SettingsCurrent.m_szGirderPluginFilename);
			m_fLircSupport = FALSE;
		}
	}

	m_wndControlDlgBar.m_wndSliderCtrlVolume.SetPos(m_SettingsCurrent.m_nSoundLevel);
	m_wndControlDlgBar.m_wndSliderCtrlBalance.SetPos(m_SettingsCurrent.m_nSoundBalance);

	if(m_wndControlDlgBar.m_wndSliderCtrlDelay.m_hWnd)
	{
		m_wndControlDlgBar.m_wndSliderCtrlDelay.EnableWindow(m_SettingsCurrent.m_fUseDelayFilter);
		m_wndControlDlgBar.m_wndStaticDelay.EnableWindow(m_SettingsCurrent.m_fUseDelayFilter);
	}

	m_SubtitleManager.SetHWNDCallback(m_wndView.m_hWnd);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;

	if(!m_fScreenSaverMode)
	{
		static CString szClassName;
		szClassName.LoadString(IDS_CLASSNAME);
		cs.lpszClass = szClassName;
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

BOOL CMainFrame::OnToolTipText(UINT, NMHDR* pNMHDR, LRESULT* pResult)
{
	ASSERT(pNMHDR->code == TTN_NEEDTEXTA || pNMHDR->code == TTN_NEEDTEXTW);

    #define _countof(array) (sizeof(array)/sizeof(array[0]))

	// need to handle both ANSI and UNICODE versions of the message
	TOOLTIPTEXTA* pTTTA = (TOOLTIPTEXTA*)pNMHDR;
	TOOLTIPTEXTW* pTTTW = (TOOLTIPTEXTW*)pNMHDR;
	CString strTipText;
	UINT nID = pNMHDR->idFrom;

	if(pNMHDR->code == TTN_NEEDTEXTA && (pTTTA->uFlags & TTF_IDISHWND) ||
		pNMHDR->code == TTN_NEEDTEXTW && (pTTTW->uFlags & TTF_IDISHWND)) {

		// idFrom is actually the HWND of the tool
		nID = ::GetDlgCtrlID((HWND)nID);
	}

	if(nID != 0)
        strTipText.LoadString(nID);
	else
		return FALSE;

#ifndef _UNICODE
	if (pNMHDR->code == TTN_NEEDTEXTA)
		lstrcpyn(pTTTA->szText, strTipText, _countof(pTTTA->szText));
	else
		_mbstowcsz(pTTTW->szText, strTipText, _countof(pTTTW->szText));
#else
	if (pNMHDR->code == TTN_NEEDTEXTA)
		_wcstombsz(pTTTA->szText, strTipText, _countof(pTTTA->szText));
	else
		lstrcpyn(pTTTW->szText, strTipText, _countof(pTTTW->szText));
#endif
	*pResult = 0;

	// bring the tooltip window above other popup windows
	::SetWindowPos(pNMHDR->hwndFrom, HWND_TOP, 0, 0, 0, 0,
		SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOMOVE|SWP_NOOWNERZORDER);

	return TRUE;    // message was handled
}
void CMainFrame::OnMRUFileHandler(UINT i)
{
	if(m_pRecentFileList)
	{
		int nIndex = i - IDM_MRU_FILE1;
		if((*m_pRecentFileList)[nIndex].GetLength() != 0)
		{
			if(OpenFile((*m_pRecentFileList)[nIndex], FALSE, TRUE, TRUE))
				PlayCurrent();
			else
				m_pRecentFileList->Remove(nIndex);
		}
	}
} 

void CMainFrame::OnMRUDirectoryHandler(UINT i)
{
	if(m_pRecentDirectoryList)
	{
		CString szName, strCurDir, strMessage;
		int nIndex = i - IDM_MRU_DIR1;
		if((*m_pRecentDirectoryList)[nIndex].GetLength() != 0)
		{
			if(OpenFile((*m_pRecentDirectoryList)[nIndex], TRUE, TRUE, TRUE))
				PlayCurrent();
		}
	}
} 

void CMainFrame::OnUpdateMRUFileHandler(CCmdUI* pCmdUI)
{
	TRACE("CMainFrame::OnUpdateMRUFileHandler ID %d, Index %d, MenuCount %d, SubMenuCount %d\n",
		pCmdUI->m_nID, pCmdUI->m_nIndex,
		(pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetMenuItemCount() : -1),
		(pCmdUI->m_pSubMenu ? pCmdUI->m_pSubMenu->GetMenuItemCount() : -1));

	if(m_pRecentFileList && pCmdUI->m_pSubMenu == NULL)
		m_pRecentFileList->UpdateMenuEx(pCmdUI);
}

void CMainFrame::OnUpdateMRUDirectoryHandler(CCmdUI* pCmdUI)
{
	TRACE("CMainFrame::OnUpdateMRUDirectoryHandler ID %d, Index %d, MenuCount %d, SubMenuCount %d\n",
		pCmdUI->m_nID, pCmdUI->m_nIndex,
		(pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetMenuItemCount() : -1),
		(pCmdUI->m_pSubMenu ? pCmdUI->m_pSubMenu->GetMenuItemCount() : -1));

	if(m_pRecentDirectoryList && pCmdUI->m_pSubMenu == NULL)
		m_pRecentDirectoryList->UpdateMenuEx(pCmdUI);
}

void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
	// forward focus to the view window
//	m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	CPushRoutingFrame push(this);

	// let the view have first crack at the command
	if(m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	//// otherwise, do default handling
	//return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);

	//// pump through current view FIRST
	//CView* pView = GetActiveView();
	//if (pView != NULL && pView->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
	//	return TRUE;

	// then pump through frame
	if (CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// last but not least, pump through app
	CWinApp* pApp = AfxGetApp();
	if (pApp != NULL && pApp->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	return FALSE;
}

void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
	if(m_pVideoWindow && !m_fDesktopMode)
	{
		CRect rectClient(0, 0, 0, 0);
		CRect rectStatusBar(0, 0, 0, 0);
		CRect rectReBar(0, 0, 0, 0);
		if(m_wndStatusBar.IsWindowVisible())
			m_wndStatusBar.GetWindowRect(&rectStatusBar);
		
		GetClientRect(&rectClient);
		rectClient.bottom -= rectStatusBar.Height() + rectReBar.Height();
		
		// Track the movement of the container window and resize as needed
		m_pVideoWindow->SetWindowPosition(rectClient.left, rectClient.top, 
												rectClient.right, rectClient.bottom);
	}
}

BOOL CMainFrame::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CMainFrame::OnClose() 
{
	if(m_fDesktopMode)
		OnViewDesktop();

	CloseCurrentClip();

	CFrameWnd::OnClose();
}

void CMainFrame::ShowError(LPCTSTR pszMessage)
{
	m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), pszMessage);
	m_wndView.ShowText(pszMessage);
}

void CMainFrame::UpdateTimeString(BOOL fSliderBarPosUpdate)
{
	CString szTime(_T(""));
	LONGLONG llPosition;
	if((llPosition = m_MediaTime.GetTimeString(szTime)) != (LONGLONG)-1)
	{
		m_wndView.ShowTimeLine(m_fViewTimeLine, szTime,
			m_MediaTime.m_llDuration == 0 ? 0.0 : (double)llPosition / (double)m_MediaTime.m_llDuration);

		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_TIME), szTime);

		if(fSliderBarPosUpdate && m_MediaTime.m_enTimeFormat != TimeFormatNone)
			m_wndSeekingDlgBar.m_wndSliderBar.SetPos((int)(llPosition / m_MediaTime.m_dwFrameStep));
	}
}

void CMainFrame::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CSliderCtrl * pSliderCtrl = (CSliderCtrl *)pScrollBar;
	if(pScrollBar && pScrollBar->m_hWnd == m_wndSeekingDlgBar.m_wndSliderBar.m_hWnd)
	{
		TRACE(_T("CMainFrame::OnHScroll %d, 0x%08x\n"), nSBCode);

		BOOL fIFrame = (GetAsyncKeyState (VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1) != 0);
		LONGLONG llPosition = 0, llStopPosition = 0;
		llPosition = (LONGLONG)m_wndSeekingDlgBar.m_wndSliderBar.GetPos() * (LONGLONG)m_MediaTime.m_dwFrameStep;
		m_pMediaSeeking->SetPositions(
			&llPosition, AM_SEEKING_AbsolutePositioning | (fIFrame ? AM_SEEKING_SeekToKeyFrame : 0 ),
			NULL, AM_SEEKING_NoPositioning);

		UpdateTimeString(FALSE);

		m_SubtitleManager.Start();
	}
	else
		CFrameWnd::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CMainFrame::OnTimer(UINT nIDEvent) 
{
	if(m_pMediaSeeking != NULL)
	{
		if(nIDEvent == TIMER_FAST_FORWARD_SEEKING)
		{
			if(m_enCurrentPlayState == PSFastRewind)
			{
				LONGLONG llPosition = 0, llStopPosition = 0;
				m_pMediaSeeking->GetPositions(&llPosition, &llStopPosition);
				llPosition -= (long)((double)m_nFastForwardSpeed * (double)m_nFastForwardSeekingTime * 10000.0 / m_MediaTime.m_timeAvgTimePerFrame);
				if(llPosition < 0)
				{
					m_enCurrentPlayState = PSStopped;
					llPosition = 0;

					if(m_hFastForwardSeekingTimer != 0)
					{
						KillTimer(m_hFastForwardSeekingTimer);
						m_hFastForwardSeekingTimer = 0;
					}
				}
				m_pMediaSeeking->SetPositions(&llPosition, AM_SEEKING_AbsolutePositioning,
					NULL, AM_SEEKING_NoPositioning);
			}

			if(m_enCurrentPlayState == PSFastForward)
			{
				LONGLONG llPosition = 0, llStopPosition = 0;
				m_pMediaSeeking->GetPositions(&llPosition, &llStopPosition);
				m_pMediaSeeking->GetDuration(&llStopPosition);
				llPosition += (long)((double)m_nFastForwardSpeed * (double)m_nFastForwardSeekingTime * 10000.0 / m_MediaTime.m_timeAvgTimePerFrame);
				if(llPosition > llStopPosition)
				{
					m_enCurrentPlayState = PSStopped;
					llPosition = llStopPosition;

					if(m_hFastForwardSeekingTimer != 0)
					{
						KillTimer(m_hFastForwardSeekingTimer);
						m_hFastForwardSeekingTimer = 0;
					}
				}
				m_pMediaSeeking->SetPositions(&llPosition, AM_SEEKING_AbsolutePositioning,
					NULL, AM_SEEKING_NoPositioning);
			}
			UpdateTimeString();
		}

		if(nIDEvent == TIMER_FRAME_SEEKING)
			UpdateTimeString();

		if(nIDEvent == TIMER_SLIDESHOW)
		{
			if(m_hSlideShowTimer)
				KillTimer(m_hSlideShowTimer);
			if(!(m_fIsAPicture && m_wndPlayListDlgBar.GetCount() == 1))
			{
				OnControlNextFile();
				if(m_fDesktopMode)
				{
					while(!m_fOverlayUsed)
						OnControlNextFile();
				}
			}
		}

	}

	CFrameWnd::OnTimer(nIDEvent);
}

typedef enum EnumScreenPixelAlignment { ScreenUnknown = 0, ScreenBGR555, ScreenBGR565, ScreenBGR888, ScreenRGB888 } EnumScreenPixelAlignment;
EnumScreenPixelAlignment GetScreen()
{
	EnumScreenPixelAlignment nResult = ScreenUnknown;
    int dib_size;
    LPBITMAPINFOHEADER dib_hdr;
    HDC hdc;
    HBITMAP hbm;

	// Allocate enough space for a DIB header plus palette (for
	// 8-bit modes) or bitfields (for 16- and 32-bit modes)
 
    dib_size = sizeof(BITMAPINFOHEADER) + 256 * sizeof (RGBQUAD);
    dib_hdr = (LPBITMAPINFOHEADER) malloc(dib_size);
    memset(dib_hdr, 0, dib_size);
    dib_hdr->biSize = sizeof(BITMAPINFOHEADER);
    
	// Get a device-dependent bitmap that's compatible with the screen.

    hdc = GetDC(NULL);
    hbm = CreateCompatibleBitmap( hdc, 1, 1 );

    // Convert the DDB to a DIB.  We need to call GetDIBits twice:  
    // the first call just fills in the BITMAPINFOHEADER; the 
    // second fills in the bitfields or palette.

    GetDIBits(hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) dib_hdr, DIB_RGB_COLORS);
    GetDIBits(hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) dib_hdr, DIB_RGB_COLORS);
    DeleteObject(hbm);
    ReleaseDC(NULL, hdc);

    TRACE(_T("Current video mode is %lu-bit.\n"), dib_hdr->biBitCount);

    if(dib_hdr->biCompression == BI_BITFIELDS)
    {
        DWORD * fields = (DWORD*)((char*)dib_hdr + dib_hdr->biSize);
        switch(fields[0])
        {
        case 0xf800:
			TRACE(_T("    (565 BGR pixel alignment)\n"));
			nResult = ScreenBGR565;
            break;
        case 0x7c00:
			TRACE(_T("    (555 BGR pixel alignment)\n"));
			nResult = ScreenBGR555;
            break;
        case 0xff0000:
			TRACE(_T("    (888 BGR pixel alignment)\n"));
			nResult = ScreenBGR888;
            break;
        case 0x0000ff:
			TRACE(_T("    (888 RGB pixel alignment)\n"));
			nResult = ScreenRGB888;
            break;
        default:
			TRACE(_T("    (Unknown pixel alignment %x:%x:%x)\n"), fields[0], fields[1], fields[2] );
			nResult = ScreenUnknown;
        }
    }

    free(dib_hdr);

	return nResult;
}

BOOL GetCurrentPaletteEntry(UINT nIndex, COLORREF & color)
{
	BOOL bResult = FALSE;
    int dib_size;
	LPBITMAPINFOHEADER dib_hdr;
	HDC hdc;
	HBITMAP hbm;
	
	/* Allocate enough space for a DIB header plus palette (for
	* 8-bit modes) or bitfields (for 16- and 32-bit modes)
	*/
	dib_size = sizeof(BITMAPINFOHEADER) + 256 * sizeof (RGBQUAD);
	dib_hdr = (LPBITMAPINFOHEADER) malloc(dib_size);
	memset(dib_hdr, 0, dib_size);
	dib_hdr->biSize = sizeof(BITMAPINFOHEADER);
	
	/* Get a device-dependent bitmap that's compatible with the
	screen.
	*/
	hdc = ::GetDC(NULL);
	hbm = CreateCompatibleBitmap( hdc, 1, 1 );
	
	/* Convert the DDB to a DIB.  We need to call GetDIBits twice:  
	* the first call just fills in the BITMAPINFOHEADER; the 
	* second fills in the bitfields or palette.
	*/
	GetDIBits(hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) dib_hdr, DIB_RGB_COLORS);
	GetDIBits(hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) dib_hdr, DIB_RGB_COLORS);
	DeleteObject(hbm);
	ReleaseDC(NULL, hdc);
	
	TRACE(_T("Current video mode is %lu-bit.\n"), dib_hdr->biBitCount);
	
	if(dib_hdr->biBitCount <= 8 && nIndex <= (UINT)(1 << dib_hdr->biBitCount))
	{
		color = RGB(((LPBITMAPINFO)dib_hdr)->bmiColors[nIndex].rgbRed,
			((LPBITMAPINFO)dib_hdr)->bmiColors[nIndex].rgbGreen,
			((LPBITMAPINFO)dib_hdr)->bmiColors[nIndex].rgbBlue);
		bResult = TRUE;
	}
	
	free(dib_hdr);

	return bResult;
}

BOOL CMainFrame::InsertFilter(IBaseFilter * pFilterToInsert, BOOL fBefore, IBaseFilter * pBaseFilter)
{
	BOOL fResult = FALSE;

	CComPtr<IEnumPins> pBaseEnumPins = NULL;
	if(SUCCEEDED(pBaseFilter->EnumPins(&pBaseEnumPins)))
	{
		ULONG ul = 0;
		CComPtr<IPin> pBasePin = NULL;
		while(SUCCEEDED(pBaseEnumPins->Next(1, &pBasePin, &ul)) && ul == 1)
		{
			PIN_DIRECTION nBasePinDirection;
			if(SUCCEEDED(pBasePin->QueryDirection(&nBasePinDirection)))
			{
				if(	(fBefore && nBasePinDirection == PINDIR_INPUT) ||
					(!fBefore && nBasePinDirection == PINDIR_OUTPUT))
				{
					// Disconnect BaseFilter from the other Filter
					CComPtr<IPin> pPinConnectedTo = NULL;
					if(SUCCEEDED(pBasePin->ConnectedTo(&pPinConnectedTo)))
					{
						HRESULT hr2 = S_OK;
						hr2 = m_pGraphBuilder->Disconnect(pBasePin);
						hr2 = m_pGraphBuilder->Disconnect(pPinConnectedTo);
					}

					BOOL fInDone = FALSE;
					BOOL fOutDone = FALSE;

					CComPtr<IPin> pToInsertPinIn = NULL;
					CComPtr<IPin> pToInsertPinOut = NULL;

					CComPtr<IEnumPins> pToInsertEnumPins = NULL;
					if(pFilterToInsert && SUCCEEDED(pFilterToInsert->EnumPins(&pToInsertEnumPins)))
					{
						CComPtr<IPin> pToInsertPin = NULL;

						// Get the input and output pins
						while(SUCCEEDED(pToInsertEnumPins->Next(1, &pToInsertPin.p, &ul)) && ul == 1)
						{
							PIN_DIRECTION nToInsertPinDirection;
							if(SUCCEEDED(pToInsertPin->QueryDirection(&nToInsertPinDirection)))
							{
								if(nToInsertPinDirection == PINDIR_INPUT)
								{
									pToInsertPinIn = NULL;
									pToInsertPinIn = pToInsertPin;
									continue;
								}
								if(nToInsertPinDirection == PINDIR_OUTPUT)
								{
									pToInsertPinOut = NULL;
									pToInsertPinOut = pToInsertPin;
									continue;
								}
							}
							//pToInsertPin->Release();
							pToInsertPin = NULL;
						}
						pToInsertEnumPins = NULL;

						// Connect the filter to insert
						HRESULT hr = 0;
						if(fBefore)
						{
							if(pToInsertPinIn != NULL && pPinConnectedTo != NULL)
								hr = m_pGraphBuilder->Connect(pPinConnectedTo, pToInsertPinIn);
							if(pToInsertPinOut != NULL)
								hr = m_pGraphBuilder->Connect(pToInsertPinOut, pBasePin);
						}
						else
						{
							if(pToInsertPinIn != NULL)
								hr = m_pGraphBuilder->Connect(pBasePin, pToInsertPinIn);
							if(pToInsertPinOut != NULL && pPinConnectedTo != NULL)
								hr = m_pGraphBuilder->Connect(pToInsertPinOut, pPinConnectedTo);
						}

						// Test the connections
						CComPtr<IPin> pTestPin = NULL;
						if(fBefore)
						{
							pTestPin = NULL;
							if(SUCCEEDED(pBasePin->ConnectedTo(&pTestPin)))
							{
								//pTestPin->Release();
								pTestPin = NULL;
								fOutDone = TRUE;
							}
							else
							{
								fOutDone = FALSE;
								if(pToInsertPinOut != NULL) // Try to connect again 
								{
									hr = m_pGraphBuilder->Connect(pToInsertPinOut, pBasePin);
									pTestPin = NULL;
									if(SUCCEEDED(pBasePin->ConnectedTo(&pTestPin)))
									{
										//pTestPin->Release();
										pTestPin = NULL;
										fOutDone = TRUE;
									}
								}
							}

							if(pPinConnectedTo != NULL)
							{
								pTestPin = NULL;
								if(SUCCEEDED(pPinConnectedTo->ConnectedTo(&pTestPin)))
								{
									//pTestPin->Release();
									pTestPin = NULL;
									fInDone = TRUE;
								}
								else
								{
									fInDone = FALSE;
									if(pToInsertPinIn != NULL) // Try to connect again 
									{
										hr = m_pGraphBuilder->Connect(pPinConnectedTo, pToInsertPinIn);
										pTestPin = NULL;
										if(SUCCEEDED(pPinConnectedTo->ConnectedTo(&pTestPin)))
										{
											//pTestPin->Release();
											pTestPin = NULL;
											fInDone = TRUE;
										}
									}
								}
							}
						}
						else
						{
							pTestPin = NULL;
							if(SUCCEEDED(pBasePin->ConnectedTo(&pTestPin)))
							{
								//pTestPin->Release();
								pTestPin = NULL;
								fInDone = TRUE;
							}
							else
							{
								fInDone = FALSE;
								if(pToInsertPinIn != NULL) // Try to connect again 
								{
									hr = m_pGraphBuilder->Connect(pBasePin, pToInsertPinIn);
									pTestPin = NULL;
									if(SUCCEEDED(pBasePin->ConnectedTo(&pTestPin)))
									{
										//pTestPin->Release();
										pTestPin = NULL;
										fInDone = TRUE;
									}
								}
							}

							if(pPinConnectedTo != NULL)
							{
								if(SUCCEEDED(pPinConnectedTo->ConnectedTo(&pTestPin)))
								{
									//pTestPin->Release();
									pTestPin = NULL;
									fOutDone = TRUE;
								}
								else
								{
									fOutDone = FALSE;
									if(pToInsertPinOut != NULL) // Try to connect again 
									{
										hr = m_pGraphBuilder->Connect(pToInsertPinOut, pPinConnectedTo);
										pTestPin = NULL;
										if(SUCCEEDED(pPinConnectedTo->ConnectedTo(&pTestPin)))
										{
											//pTestPin->Release();
											pTestPin = NULL;
											fOutDone = TRUE;
										}
									}
								}
							}
						}
					}

					fResult = (pPinConnectedTo != NULL && (fInDone && fOutDone)) || (pPinConnectedTo == NULL && ((!fBefore && fInDone) || (fBefore && fOutDone)));

					HRESULT hr = 0;
					// If error, try to reconnect the originals filters
					if(pPinConnectedTo != NULL && !(fInDone && fOutDone))
					{
						if(fBefore)
							hr = m_pGraphBuilder->Connect(pPinConnectedTo, pBasePin);
						else
							hr = m_pGraphBuilder->Connect(pBasePin, pPinConnectedTo);
					}

					pPinConnectedTo = NULL;
					pToInsertPinIn = NULL;;
					pToInsertPinOut = NULL;;
					pBasePin = NULL;
					break;
				}
			}
			pBasePin = NULL;
		}
		pBaseEnumPins = NULL;
	}

	return fResult;
}

HRESULT CMainFrame::PlayMovieInWindow(LPCTSTR szFile)
{
	TRACE("CMainFrame::PlayMovieInWindow (%s)\n", szFile);

	m_wndView.EnableWindow(FALSE);

	m_fScreenSaverNoDisturb = TRUE;

    WCHAR wFile[MAX_PATH];
    HRESULT hr = S_FALSE;

//	if(m_pMediaControl != NULL)
	if(m_pGraphBuilder != NULL)
		CloseCurrentClip();

    // Clear open dialog remnants before calling RenderFile()
    m_wndView.UpdateWindow();

#ifndef UNICODE
    MultiByteToWideChar(CP_ACP, 0, szFile, -1, wFile, MAX_PATH);
#else
    lstrcpy(wFile, szFile);
#endif
	
	m_fIsAPicture = FALSE;
	
	CString szExtension(szFile);
	int nPos = szExtension.ReverseFind(_T('.'));
	if(nPos != -1)
		szExtension = szExtension.Mid(nPos + 1);
	szExtension.MakeLower();
	m_fIsAPicture = (szExtension == _T("jpg") || szExtension == _T("jpeg") || szExtension == _T("gif") || szExtension == _T("bmp") || szExtension == _T("tga"));

    // Get the interface for DirectShow's GraphBuilder

//	JIF(CoCreateInstance(CLSID_FilterGraphNoThread, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGraphBuilder));
	JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGraphBuilder));
	if(SUCCEEDED(hr))
	{
		try
		{
			 m_fUseVMR = FALSE;

			CComPtr<IBaseFilter> pOverlayMixer = NULL;
			CComPtr<IBaseFilter> pVideoRenderer = NULL;

			if(!m_fScreenSaverMode && m_SettingsCurrent.m_fUseOverlayWhenPossible)
			{
				CoCreateInstance(CLSID_OverlayMixer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&(pOverlayMixer));
				if(pOverlayMixer)
					hr = m_pGraphBuilder->AddFilter(pOverlayMixer, L"Overlay Mixer"); 
			}

			//CoCreateInstance(CLSID_VideoMixingRenderer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pVideoRenderer);
			//if(pVideoRenderer)
			//{
			//	hr = m_pGraphBuilder->AddFilter(pVideoRenderer, L"Video Renderer"); 

			//	CComPtr<IVMRFilterConfig> pVMRFilterConfig = NULL;
			//	if(SUCCEEDED(hr = pVideoRenderer->QueryInterface(IID_IVMRFilterConfig, (LPVOID *)&pVMRFilterConfig)))
			//	{
			//		hr = pVMRFilterConfig->SetRenderingMode(VMRMode_Windowless); 

			//		// Set the window. 
			//		if(SUCCEEDED(hr = pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl, (void**)&m_pVMRWindowlessControl)))
			//		{ 
			//			if(SUCCEEDED(hr = m_pVMRWindowlessControl->SetVideoClippingWindow(m_wndView.m_hWnd)))
			//			{
			//				m_fUseVMR = TRUE;
			//			}
			//		}

			//		pVMRFilterConfig = NULL;
			//	}

			//	pVideoRenderer = NULL;
			//}
			//else
			//{
			//}
			BOOL fNeedRebuild = FALSE;

			hr = m_pGraphBuilder->RenderFile(wFile, NULL);
			if(FAILED(hr))
			{
				if(hr == VFW_E_UNSUPPORTED_STREAM)
				{
					TCHAR pszErrorMessage[512];
					AMGetErrorText(hr, pszErrorMessage, sizeof(pszErrorMessage));
					ShowError(pszErrorMessage);
					goto Error;
				}

				fNeedRebuild = TRUE;
				if(pOverlayMixer)
					pOverlayMixer = NULL;
			}

			m_fOverlayUsed = FALSE;

			if(pOverlayMixer)
			{
				BOOL fOverlayFilterConnected = FALSE;
				{
					CComPtr<IEnumPins> pEnumPins = NULL;
					if(SUCCEEDED(pOverlayMixer->EnumPins(&pEnumPins)))
					{
						CComPtr<IPin> pPin = NULL;
						ULONG ul = 0;
						while(SUCCEEDED(pEnumPins->Next(1, &pPin, &ul)) && ul == 1)
						{
							CComPtr<IPin> pPin2 = NULL;
							if(SUCCEEDED(pPin->ConnectedTo(&pPin2)))
							{
								m_fOverlayUsed = TRUE;
								pPin2 = NULL;
							}
							pPin = NULL;
						}
						pEnumPins = NULL;
					}
				}

				if(!m_fOverlayUsed)
				{
					m_pGraphBuilder->RemoveFilter(pOverlayMixer);
					pOverlayMixer = NULL;
				}
			}

			if(fNeedRebuild)
			{
				m_pGraphBuilder = NULL;

				// Get the interface for DirectShow's GraphBuilder
				JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGraphBuilder));
				if(FAILED(hr = m_pGraphBuilder->RenderFile(wFile, NULL)))
				{
					TCHAR pszErrorMessage[512];
					AMGetErrorText(hr, pszErrorMessage, sizeof(pszErrorMessage));
					ShowError(pszErrorMessage);

					goto Error;
				}
			}


			m_fOverlayUsed = (pOverlayMixer != NULL);
			if(pOverlayMixer != NULL)
			{
				CComPtr<IEnumPins> pEnumPins = NULL;
				CComPtr<IPin> pPin = NULL;
				ULONG ul = NULL;
				PIN_DIRECTION pd = PINDIR_OUTPUT;
				CComPtr<IMixerPinConfig> pMPC = NULL;
				if(SUCCEEDED(pOverlayMixer->EnumPins(&pEnumPins)))
				{
					while(SUCCEEDED(pEnumPins->Next(1, &pPin, &ul)) && ul == 1)  // try all pins
					{
						pPin->QueryDirection(&pd);
						if(pd == PINDIR_INPUT)  // only the 1st in pin
						{
							if(SUCCEEDED(pPin->QueryInterface(IID_IMixerPinConfig, (LPVOID *)&pMPC)))
							{
								pMPC->GetColorKey(NULL, &m_dwColorKey);  // just get the physical color

								//  Set mode to stretch - that way we don't fight the overlay
								//  mixer about the exact way to fix the aspect ratio
								pMPC->SetAspectRatioMode(AM_ARMODE_STRETCHED);
								pMPC = NULL;
							}
							pPin = NULL;
							break;   // we are done
						}
						pPin = NULL;
					}
					pEnumPins = NULL;
				}
				pOverlayMixer = NULL;
			}

			m_fColorKeyDirty = TRUE;

			if(m_SettingsCurrent.m_fUseDelayFilter)
			{
				CComPtr<IBaseFilter> pVideoRender = NULL;
				if(m_fOverlayUsed)
					m_pGraphBuilder->FindFilterByName(L"Overlay Mixer", &pVideoRender);
				else
					m_pGraphBuilder->FindFilterByName(L"Video Renderer", &pVideoRender);

				if(pVideoRender != NULL)
				{
					m_pDelayTransformFilter = new CDelayTransformFilter();
					m_pDelayTransformFilter->m_nDelay = m_nDelay;
					m_pDelayTransformFilter->AddRef();

					CComPtr<IBaseFilter> pDelayTransformFilter = NULL;
					m_pDelayTransformFilter->QueryInterface(IID_IBaseFilter, (LPVOID *)&pDelayTransformFilter);
					hr = m_pGraphBuilder->AddFilter(pDelayTransformFilter, L"Delay Filter");

					if(!InsertFilter(pDelayTransformFilter, TRUE, pVideoRender))
					{
						hr = m_pGraphBuilder->RemoveFilter(pDelayTransformFilter);
					}
					pDelayTransformFilter = NULL;
					pVideoRender = NULL;
				}
			}

			m_pAMStreamSelect = NULL;
			CComPtr<IBaseFilter> pStreamSelectFilter = NULL;
			m_pGraphBuilder->FindFilterByName(L"Morgan Stream Switcher", &pStreamSelectFilter);
			if(pStreamSelectFilter != NULL)
			{
				pStreamSelectFilter->QueryInterface(IID_IAMStreamSelect, (LPVOID *)&m_pAMStreamSelect);
				//pStreamSelectFilter->Release();
				pStreamSelectFilter = NULL;
			}


			// QueryInterface for DirectShow interfaces
			JIF(m_pGraphBuilder->QueryInterface(IID_IMediaControl, (void **)&m_pMediaControl));
			JIF(m_pGraphBuilder->QueryInterface(IID_IMediaEventEx, (void **)&m_pMediaEventEx));
			JIF(m_pGraphBuilder->QueryInterface(IID_IMediaSeeking, (void **)&m_pMediaSeeking));

			// Query for video interfaces, which may not be relevant for audio files
			JIF(m_pGraphBuilder->QueryInterface(IID_IVideoWindow, (void **)&m_pVideoWindow));
			JIF(m_pGraphBuilder->QueryInterface(IID_IBasicVideo, (void **)&m_pBasicVideo));

			// Query for audio interfaces, which may not be relevant for video-only files
			JIF(m_pGraphBuilder->QueryInterface(IID_IBasicAudio, (void **)&m_pBasicAudio));

			if(m_pMediaControl == NULL || m_pMediaEventEx == NULL || m_pMediaSeeking == NULL ||
				m_pVideoWindow == NULL || m_pBasicVideo == NULL || m_pBasicAudio == NULL)
				goto Error;

			// Time
			m_wndSeekingDlgBar.m_wndSliderBar.SetRange(0, 1);
			m_wndSeekingDlgBar.m_wndSliderBar.SetPos(0);
			m_wndSeekingDlgBar.m_wndSliderBar.EnableWindow(FALSE);

			m_MediaTime.Init(m_pMediaSeeking);
			if(m_MediaTime.m_enTimeFormat != TimeFormatNone)
			{
				m_wndSeekingDlgBar.m_wndSliderBar.EnableWindow();
				m_wndSeekingDlgBar.m_wndSliderBar.SetRange(0, (int)(m_MediaTime.m_llDuration / m_MediaTime.m_dwFrameStep));
			}

			// Sound level
			m_pBasicAudio->put_Volume(m_SettingsCurrent.m_fSoundMute || (m_fScreenSaverMode && m_SettingsCurrent.m_fScreensaverMute) ? -10000L : m_SettingsCurrent.m_nSoundLevel);
			m_pBasicAudio->put_Balance(m_SettingsCurrent.m_nSoundBalance);

			// Is this an audio-only file (no video component)?
			m_bAudioOnly = FALSE;
			m_fViewCenter = TRUE;

			m_szFileName = szFile;
			UpdateTimeString();
			UpdateMainTitle();

			if(m_pVideoWindow == NULL || m_pBasicVideo == NULL)
				m_bAudioOnly = TRUE;
			else
			{
				long lVisible = 0;
				m_bAudioOnly = !m_fUseVMR && (m_pVideoWindow->get_Visible(&lVisible) == E_NOINTERFACE);
			}

			m_wndView.m_fEraseBkgnd = m_bAudioOnly;

			if (!m_bAudioOnly)
			{
				//if(m_fUseVMR)
				//{
				//	// Find the native video size.
				//	//long lWidth, lHeight; 
				//	//if(SUCCEEDED(hr = m_pVMRWindowlessControl->GetNativeVideoSize(&lWidth, &lHeight, NULL, NULL)))
				//	//{
				//	//	RECT rcSrc, rcDest; 
				//	//	// Set the source rectangle.
				//	//	SetRect(&rcSrc, 0, 0, lWidth/2, lHeight/2); 

				//	//	// Get the window client area.
				//	//	m_wndView.GetClientRect(&rcDest); 
				//	//	// Set the destination rectangle.
				//	//	SetRect(&rcDest, 0, 0, rcDest.right/2, rcDest.bottom/2); 

				//	//	// Set the video position.
				//	//	hr = m_pVMRWindowlessControl->SetVideoPosition(&rcSrc, &rcDest); 
				//	//}

				//	long nSourceWidth = 0, nSourceHeight = 0;
				//	m_pVMRWindowlessControl->GetNativeVideoSize(&nSourceWidth, &nSourceHeight, NULL, NULL);

				//	if(nSourceWidth && nSourceHeight)
				//		m_rCurrentAspectRatio = (double)nSourceWidth / (double)nSourceHeight;
				//	else
				//		m_rCurrentAspectRatio = 0.0;

				//	UpdateDestinationView();
				//}
				//else
				//{
  					hr = m_pVideoWindow->put_Owner((OAHWND)m_wndView.m_hWnd);
					hr = m_pVideoWindow->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
					hr = m_pVideoWindow->put_MessageDrain((OAHWND)m_wndView.m_hWnd);

					hr = m_pVideoWindow->put_AutoShow(OAFALSE);

					// Aspect ratio
					long nSourceWidth = 0, nSourceHeight = 0;
					m_pBasicVideo->get_SourceWidth(&nSourceWidth);
					m_pBasicVideo->get_SourceHeight(&nSourceHeight);

					if(nSourceWidth && nSourceHeight)
						m_rCurrentAspectRatio = (double)nSourceWidth / (double)nSourceHeight;
					else
						m_rCurrentAspectRatio = 0.0;

					CRect rect;
					GetClientRect(&rect);
					hr = m_pVideoWindow->SetWindowPosition(0, 0, rect.Width(), rect.Height());
					UpdateDestinationView();

					if(m_fDesktopMode)
					{
						INT nElements = COLOR_DESKTOP;
						COLORREF colorValues = GetColorKey();
						COLORREF colorOldValues = GetSysColor(nElements);
						if(colorOldValues != colorValues)
							SetSysColors(1, &nElements, &colorValues);
					}
				//}
			}
			else
			{
				m_pBasicVideo = NULL;
				m_pVideoWindow = NULL;
			}

			OAEVENT m_hGraphNotifyEvent = NULL;
			hr = m_pMediaEventEx->GetEventHandle((OAEVENT*) &m_hGraphNotifyEvent);
			
			// Have the graph signal event via window callbacks for performance
			JIF(m_pMediaEventEx->SetNotifyWindow((OAHWND)m_wndView.m_hWnd, WM_GRAPHNOTIFY, 0));


			//if(m_hFrameSeekingTimer != 0)
			//{
			//	KillTimer(m_hFrameSeekingTimer);
			//	m_hFrameSeekingTimer = 0;
			//}
			//m_hFrameSeekingTimer = SetTimer(TIMER_FRAME_SEEKING, 100, NULL);

			//if(m_hSlideShowTimer != 0)
			//{
			//	KillTimer(m_hSlideShowTimer);
			//	m_hSlideShowTimer = 0;
			//}
			//if(m_fIsAPicture && m_SettingsCurrent.m_fSlideshow && m_SettingsCurrent.m_nSlideshowWaitingTime)
			//	m_hSlideShowTimer = SetTimer(TIMER_SLIDESHOW, m_SettingsCurrent.m_nSlideshowWaitingTime, NULL);


			hr = AddGraphToRot(m_pGraphBuilder, &m_dwGraphRegister);
			if(FAILED(hr))
			{
				TRACE(_T("Failed to register filter graph with ROT!  hr=0x%x"), hr);
				m_dwGraphRegister = 0;
			}

			if(m_pVideoWindow && !m_fUseVMR)
				hr = m_pVideoWindow->put_Visible(OATRUE);

			m_wndView.EnableWindow(TRUE);

			m_pMediaControl->Run();

			OAFilterState pfs;
			//m_pMediaControl->GetState(INFINITE, &pfs);
			//hr = m_pMediaControl->GetState(3000, &pfs);

			LONGLONG pos = 0;
			hr = m_pMediaSeeking->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
			hr = m_pMediaControl->Stop();

			hr = m_pMediaControl->GetState(1000, &pfs);

			// Run the graph to play the media file
			if(FAILED(m_pMediaControl->Run()))
			{
				//m_pMediaControl->GetState(INFINITE, &pfs);
				hr = m_pMediaControl->GetState(1000, &pfs);
				if((FILTER_STATE)pfs != State_Running)
				{

				}
			}
			CString szPlayingStatus;
			szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PLAYING);
			m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);
		}
		catch(const TCHAR * pszTextMessage)
		{
			hr = S_FALSE;
			ShowError(pszTextMessage);
			goto Error;
		}
		catch(...)
		{
			hr = S_FALSE;
			CString szPlayingStatus;
			szPlayingStatus.LoadString(IDS_STRING_PLAY_ERROR);
			ShowError(szPlayingStatus);
			goto Error;
		}

		//m_wndView.m_wndOSDView.ShowWindow(SW_SHOW);

		hr = S_OK;
    	m_enCurrentPlayState = PSRunning;

		m_SubtitleManager.Start();

		//CMenu * pMenu = GetMenu();
		//if(pMenu) UpdateMenu(pMenu);

		m_wndView.SetFocus();

		TRACE("PPPPPPPPPP Playing %s\n", m_szFileName);
	}
Error:
	m_fScreenSaverNoDisturb = FALSE;

    return hr;
}

void CMainFrame::SetTimers()
{
	// Set Timers
	if(m_hFrameSeekingTimer != 0)
	{
		KillTimer(m_hFrameSeekingTimer);
		m_hFrameSeekingTimer = 0;
	}
	m_hFrameSeekingTimer = SetTimer(TIMER_FRAME_SEEKING, 100, NULL);

	if(m_hSlideShowTimer != 0)
	{
		KillTimer(m_hSlideShowTimer);
		m_hSlideShowTimer = 0;
	}
	if(m_fIsAPicture && m_SettingsCurrent.m_fSlideshow && m_SettingsCurrent.m_nSlideshowWaitingTime)
		m_hSlideShowTimer = SetTimer(TIMER_SLIDESHOW, m_SettingsCurrent.m_nSlideshowWaitingTime, NULL);
}

COLORREF CMainFrame::GetColorKey()
{
	if(m_fColorKeyDirty)
	{
		COLORREF colorValues = m_dwColorKey;
		DEVMODE DevMode;
		DevMode.dmSize = sizeof(DEVMODE);
		DevMode.dmDriverExtra = 0;
		if(EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &DevMode))
		{
			if(DevMode.dmBitsPerPel <= 8)
			{
				GetCurrentPaletteEntry(m_dwColorKey, colorValues);
			}
			else if(DevMode.dmBitsPerPel == 16)
			{
				if(GetScreen() == ScreenBGR565) // 565
					colorValues = RGB(((m_dwColorKey >> 11) & 0x1F) << 3, ((m_dwColorKey >> 5) & 0x3F) << 2, (m_dwColorKey & 0x1F) << 3);
				else // 555
					colorValues = RGB(((m_dwColorKey >> 10) & 0x1F) << 3, ((m_dwColorKey >> 5) & 0x1F) << 3, (m_dwColorKey & 0x1F) << 3);
			}
			else if(DevMode.dmBitsPerPel == 24 || DevMode.dmBitsPerPel == 32)
			{
				switch(GetScreen())
				{
				case ScreenBGR888:
					colorValues = RGB((colorValues >> 16) & 0xff, (colorValues >> 8) & 0xff, colorValues & 0xff);
					break;
				case ScreenRGB888:
					break;
				}
			}

		}
		m_dwColorKeyRGB = colorValues;
		m_fColorKeyDirty = FALSE;
	}

	return m_dwColorKeyRGB;
}

// Adds a DirectShow filter graph to the Running Object Table,
// allowing GraphEdit to "spy" on a remote filter graph.
HRESULT CMainFrame::AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
	HRESULT hr = E_FAIL;

	m_fGraphEditSpy = TRUE;

	if(m_fGraphEditSpy)
	{
		CComPtr <IMoniker>              pMoniker;
		CComPtr <IRunningObjectTable>   pROT;
		WCHAR wsz[128];

		if (FAILED(GetRunningObjectTable(0, &pROT)))
			return E_FAIL;

		wsprintfW(wsz, L"FilterGraph %08x pid %08x\0", (DWORD_PTR) pUnkGraph, 
				GetCurrentProcessId());

		hr = CreateItemMoniker(L"!", wsz, &pMoniker);
		if (SUCCEEDED(hr)) 
			hr = pROT->Register(ROTFLAGS_REGISTRATIONKEEPSALIVE, pUnkGraph, 
								pMoniker, pdwRegister);
	}        
    return hr;
}

// Removes a filter graph from the Running Object Table
void CMainFrame::RemoveGraphFromRot(DWORD pdwRegister)
{
    CComPtr<IRunningObjectTable> pROT;

    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) 
        pROT->Revoke(pdwRegister);

}

//HRESULT CMainFrame::AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
//{
//    IMoniker * pMoniker;
//    IRunningObjectTable *pROT;
//    if (FAILED(GetRunningObjectTable(0, &pROT))) {
//        return E_FAIL;
//    }
//    WCHAR wsz[128];
//	swprintf(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
////    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
//    HRESULT hr = CreateItemMoniker(L"!", wsz, &pMoniker);
//    if (SUCCEEDED(hr)) {
//        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
//        pMoniker->Release();
//    }
//    pROT->Release();
//    return hr;
//}
//
//void CMainFrame::RemoveGraphFromRot(DWORD pdwRegister)
//{
//    IRunningObjectTable *pROT;
//    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) {
//        pROT->Revoke(pdwRegister);
//        pROT->Release();
//    }
//}

void CMainFrame::MediaEvent()
{
	TRACE("CMainFrame::MediaEvent\n");

	if(m_pMediaEventEx == NULL || m_fIsAPicture)
		return;

//	HRESULT hr;
    long lEventCode, lParam1, lParam2;
	if(FAILED(m_pMediaEventEx->GetEvent(&lEventCode, &lParam1, &lParam2, 100))) //INFINITE )); //100));
		return;

	TRACE("\tEvent Code %d (0x%08x)\n", lEventCode, lEventCode);

	BOOL fNextFile = FALSE;

	switch(lEventCode)
	{
	case EC_CLOCK_CHANGED:
		SetTimers();
		break;
	case EC_PAUSED:
	case EC_COMPLETE:
		TRACE("XXXXXXXXXXXXXXX m_enTimeFormat = %d\n", m_MediaTime.m_enTimeFormat);
		if(m_pMediaSeeking != NULL && m_pMediaControl != NULL)
		{
			if(m_nRepeatABMode == 2 && lEventCode == EC_COMPLETE)
			{
				TRACE("MediaEvent() a = %I64d, b = %I64d\n", m_llRepeatAPosition, m_llRepeatBPosition);
				m_pMediaSeeking->SetPositions(
					&m_llRepeatAPosition, AM_SEEKING_AbsolutePositioning,
					&m_llRepeatBPosition, AM_SEEKING_AbsolutePositioning);
				m_pMediaControl->Run();

				CString szPlayingStatus;
				szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PLAYING);
				m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);

				m_SubtitleManager.Start();
				break;
			}

			{
				LONGLONG llPosition = 0, llStopPosition = 0;
				if(m_MediaTime.m_enTimeFormat != TimeFormatNone &&
					SUCCEEDED(m_pMediaSeeking->GetPositions(&llPosition, &llStopPosition)) &&
					SUCCEEDED(m_pMediaSeeking->GetDuration(&llStopPosition)))
				{
					TRACE(_T("llPosition = %I64d, llStopPosition = %I64d\n"), llPosition, llStopPosition);
					fNextFile = (llPosition >= (llStopPosition - 1));
				}
			}
		}
		break;
	}

	m_pMediaEventEx->FreeEventParams(lEventCode, lParam1, lParam2);

	if(fNextFile)
	{
		if((!m_fScreenSaverMode && m_SettingsCurrent.m_fPlayRandom) || 
			(m_fScreenSaverMode && m_SettingsCurrent.m_fScreensaverRandom))
			m_wndPlayListDlgBar.SetCurrentFile(m_wndPlayListDlgBar.GetFile(rand() % m_wndPlayListDlgBar.GetCount()));
			
		OnControlNextFile();
		if(m_fDesktopMode)
		{
			while(!m_fOverlayUsed)
				OnControlNextFile();
		}
	}
}

void CMainFrame::UpdateMainTitle(void)
{
    CString szTitle;

    // If no file is loaded, just show the application title
    if (m_szFileName == _T(""))
        szTitle.LoadString(IDR_MAINFRAME);

    // Otherwise, show useful information, including filename and play state
    else
    {
		CString szAudio, szVideo, szMuted, szPaused;
		szAudio.LoadString(IDS_STRING_AUDIO);
		szVideo.LoadString(IDS_STRING_VIDEO);
		szMuted.LoadString(IDS_STRING_MUTED);
		szPaused.LoadString(IDS_STRING_PAUSED);
        // Update the window title to show muted/normal status
        szTitle.Format(_T("%s [%s] %s%s"),
                m_szFileName,
                m_bAudioOnly ? szAudio : szVideo,
                (m_SettingsCurrent.m_fSoundMute) ? szMuted : _T(""),
                (m_enCurrentPlayState == PSPaused) ? szPaused : _T(""));
    }

    SetWindowText(szTitle);
}

void CMainFrame::UpdateDestinationView()
{
	CRect rect;
	m_wndView.GetWindowRect(&rect);
	UpdateDestinationView(rect.Width(), rect.Height());
}

void CMainFrame::UpdateDestinationView(int nWidth, int nHeight)
{
	if(m_pBasicVideo == NULL)
		return;

	CRect rectNative(0, 0, 0, 0); // Original video size
	if(m_fUseVMR)
		m_pVMRWindowlessControl->GetNativeVideoSize(&rectNative.right, &rectNative.bottom, NULL, NULL);
	else
		m_pBasicVideo->GetVideoSize(&rectNative.right, &rectNative.bottom);

	if(!m_fMaintainAspectRatio)
	{
		CRect rectSource(rectNative);
		CRect rectDestination(0, 0, nWidth, nHeight);
		if(m_fUseVMR)
			m_pVMRWindowlessControl->SetVideoPosition(&rectSource, &rectDestination); 
		else
		{
			m_pBasicVideo->SetSourcePosition(rectSource.left, rectSource.top, rectSource.Width(), rectSource.Height());
			m_pBasicVideo->SetDestinationPosition(rectDestination.left, rectDestination.top, rectDestination.Width(), rectDestination.Height());
		}
		return;
	}

	CRect rectScreen(0, 0, nWidth, nHeight); // Destination screen size
	double rScreenAspectRatio = (double)nWidth / (double)nHeight;
	double rNativeAspectRatio = (double)rectNative.Width() / (double)rectNative.Height();
	double rModifierAspectRatio = m_rCurrentAspectRatio / rNativeAspectRatio * (m_fRatio169 ? 3.0 / 4.0 : 1.0);

	if(m_fWindowFit)
	{
		if(((double)rectNative.Width() / (double)rectNative.Height()) * rModifierAspectRatio >= rScreenAspectRatio || m_fWindowWidthFit)
			m_rZoom = (double)rectScreen.Width() / (double)rectNative.Width();
		else
			m_rZoom = (double)rectScreen.Height() / (double)rectNative.Height() * rModifierAspectRatio;
	}

	CRect rectZoomedScreen(0, 0,
		(int)((double)nWidth / m_rZoom),
		(int)(((double)nHeight / m_rZoom) * rModifierAspectRatio));
	CRect rectSource(rectZoomedScreen);
	CRect rectDestination(0, 0, 0, 0);

	// Take the zoomed part of the source rectangle
	// rectSource clipped by rectNative
	if(rectSource.top < rectNative.top)
		rectSource.top = rectNative.top;
	if(rectSource.bottom > rectNative.bottom)
		rectSource.bottom = rectNative.bottom;
	if(rectSource.left < rectNative.left)
		rectSource.left = rectNative.left;
	if(rectSource.right > rectNative.right)
		rectSource.right = rectNative.right;


	double rSourceAspectRatio = (double)rectSource.Width() / (double)rectSource.Height();
	double rDestinationAspectRatio = rScreenAspectRatio; //rSourceAspectRatio; // * rModifierAspectRatio;
	double rModifierAspectRatioForVerticalScroll = 1.0;
	rDestinationAspectRatio = rSourceAspectRatio * rModifierAspectRatio;
	rModifierAspectRatioForVerticalScroll = rModifierAspectRatio;

	// Zoom and modify the aspect ratio
	if(m_fWindowFit && !m_fWindowWidthFit && rDestinationAspectRatio < rScreenAspectRatio)
	{
		rectDestination.bottom = rectScreen.Height();
		rectDestination.right = (long)((double)rectDestination.Height() * rDestinationAspectRatio);
	}
	else
	{
		if(m_fWindowFit)
			rectDestination.right = rectScreen.Width();
		else
			rectDestination.right = (long)((double)rectSource.Width() * m_rZoom);
		rectDestination.bottom = (long)((double)rectDestination.Width() / rDestinationAspectRatio);
	}

	// Horizontal scroll
	if(rectZoomedScreen.Width() <= rectNative.Width())
	{
		if(m_fViewCenter)
			m_nOffsetCenterX = rectNative.Width() / 2;
		else
		{
			if(m_nOffsetCenterX > rectNative.Width() - rectZoomedScreen.Width() / 2)
				m_nOffsetCenterX = rectNative.Width() - rectZoomedScreen.Width() / 2;
			if(m_nOffsetCenterX < rectZoomedScreen.Width() / 2)
				m_nOffsetCenterX = rectZoomedScreen.Width() / 2;
		}
		rectSource.left += m_nOffsetCenterX - rectZoomedScreen.Width() / 2;
		rectSource.right += m_nOffsetCenterX - rectZoomedScreen.Width() / 2;
	}
	else
	{
		if(m_fViewCenter)
			m_nOffsetCenterX = rectZoomedScreen.Width() / 2;
		else
		{
			if(m_nOffsetCenterX > rectZoomedScreen.Width() - rectNative.Width() / 2)
				m_nOffsetCenterX = rectZoomedScreen.Width() - rectNative.Width() / 2;
			if(m_nOffsetCenterX < rectNative.Width() / 2)
				m_nOffsetCenterX = rectNative.Width() / 2;
		}
		rectDestination.left += (int)((double)(m_nOffsetCenterX - rectNative.Width() / 2) * m_rZoom);
		rectDestination.right += (int)((double)(m_nOffsetCenterX - rectNative.Width() / 2) * m_rZoom);
	}

	// Vertical scroll
	if(rectZoomedScreen.Height() <= rectNative.Height())
	{
		if(m_fViewCenter)
			m_nOffsetCenterY = rectNative.Height() / 2;
		else
		{
			if(m_nOffsetCenterY > rectNative.Height() - rectZoomedScreen.Height() / 2)
				m_nOffsetCenterY = rectNative.Height() - rectZoomedScreen.Height() / 2;
			if(m_nOffsetCenterY < rectZoomedScreen.Height() / 2)
				m_nOffsetCenterY = rectZoomedScreen.Height() / 2;
		}
		rectSource.top += m_nOffsetCenterY - rectZoomedScreen.Height() / 2;
		rectSource.bottom += m_nOffsetCenterY - rectZoomedScreen.Height() / 2;
	}
	else
	{
		if(m_fViewCenter)
			m_nOffsetCenterY = rectZoomedScreen.Height() / 2;
		else
		{
			if(m_nOffsetCenterY > rectZoomedScreen.Height() - rectNative.Height() / 2)
				m_nOffsetCenterY = rectZoomedScreen.Height() - rectNative.Height() / 2;
			if(m_nOffsetCenterY < rectNative.Height() / 2)
				m_nOffsetCenterY = rectNative.Height() / 2;
		}
		rectDestination.top += (int)((double)(m_nOffsetCenterY - rectNative.Height() / 2) * m_rZoom / rModifierAspectRatioForVerticalScroll);
		rectDestination.bottom += (int)((double)(m_nOffsetCenterY - rectNative.Height() / 2) * m_rZoom / rModifierAspectRatioForVerticalScroll);
	}

	TRACE("Zoom %f, Fit = %d, Center = %d\n", m_rZoom, m_fWindowFit, m_fViewCenter);
	TRACE("rectNative      x=%d, y=%d, w=%d, h=%d, ratio=%f\n", rectNative.left, rectNative.top, rectNative.Width(), rectNative.Height(), rNativeAspectRatio);
	TRACE("rectZoomed      x=%d, y=%d, w=%d, h=%d, ratio=%f\n", rectZoomedScreen.left, rectZoomedScreen.top, rectZoomedScreen.Width(), rectZoomedScreen.Height(), (double)rectZoomedScreen.Width() / (double)rectZoomedScreen.Height());
	TRACE("rectSource      x=%d, y=%d, w=%d, h=%d, ratio=%f\n", rectSource.left, rectSource.top, rectSource.Width(), rectSource.Height(), rSourceAspectRatio);
	TRACE("rectDestination x=%d, y=%d, w=%d, h=%d, ratio=%f\n", rectDestination.left, rectDestination.top, rectDestination.Width(), rectDestination.Height(), rDestinationAspectRatio);
	TRACE("rectScreen      x=%d, y=%d, w=%d, h=%d, ratio=%f\n", rectScreen.left, rectScreen.top, rectScreen.Width(), rectScreen.Height(), rScreenAspectRatio);
	TRACE("\n");

	// Apply changed
	if(m_fUseVMR)
	{
		m_pVMRWindowlessControl->SetVideoPosition(&rectSource, &rectDestination); 
	}
	else
	{
		m_pBasicVideo->SetSourcePosition(rectSource.left, rectSource.top, rectSource.Width(), rectSource.Height());
		m_pBasicVideo->SetDestinationPosition(rectDestination.left, rectDestination.top, rectDestination.Width(), rectDestination.Height());
	}
}

BOOL CMainFrame::OpenFile(LPCTSTR pszPath, BOOL fWithGUI, BOOL fAddFile, BOOL fSetCurrent)
{
	if(fWithGUI)
	{
		CString szFilter;
		szFilter.LoadString(IDS_STRING_OPENFILE_FILTER);

		CFileDialog FileDialog(TRUE, _T("*.*"), NULL,
			OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING,
			szFilter, this);

		char pszFile[30 * 1024];
		memset(pszFile, 0, sizeof(pszFile));

		FileDialog.m_ofn.nMaxFile = sizeof(pszFile);
		FileDialog.m_ofn.lpstrFile = pszFile;
		if(pszPath)
			FileDialog.m_ofn.lpstrInitialDir = pszPath;

		if(FileDialog.DoModal() == IDOK)
		{
			if(!fAddFile)
				m_wndPlayListDlgBar.DeleteAllFile();

			CWaitCursor wc;
			BOOL fFirst = fSetCurrent;
			POSITION pos = FileDialog.GetStartPosition();
			while(pos)
			{
				CString szFilePath = FileDialog.GetNextPathName(pos);
				CString szExtension(_T(""));
				int nPos = szFilePath.ReverseFind(_T('.'));
				if(nPos != -1)
				{
					szExtension = szFilePath.Mid(nPos + 1);
					szExtension.MakeLower();
				}
				if(szExtension == _T("txt"))
				{
					CStdioFile file;
					if(file.Open(szFilePath, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
					{
						CString szLine;
						while(file.ReadString(szLine))
						{
							szLine.TrimLeft();
							szLine.TrimRight();
							if(szLine != _T(""))
							{
								m_wndPlayListDlgBar.AddFileToTail(szLine, fFirst);
								AddFileToRecentFileList(szFilePath);
								fFirst = FALSE;
							}
						}
						file.Close();
					}
				}
				else
				{
					m_wndPlayListDlgBar.AddFileToTail(szFilePath, fFirst);
					AddFileToRecentFileList(szFilePath);
					fFirst = FALSE;
				}
			}
			return TRUE;
		}
#include <Cderr.h>
#include ".\mainfrm.h"
		DWORD dwResult = CommDlgExtendedError();
	}
	else
	{
		BOOL fFirst = fSetCurrent;
		if(!fAddFile)
			m_wndPlayListDlgBar.DeleteAllFile();
		CString szFilePath = pszPath;
		CString szExtension(_T(""));
		int nPos = szFilePath.ReverseFind(_T('.'));
		if(nPos != -1)
		{
			szExtension = szFilePath.Mid(nPos + 1);
			szExtension.MakeLower();
		}
		if(szExtension == _T("txt"))
		{
			CStdioFile file;
			if(file.Open(szFilePath, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				CString szLine;
				while(file.ReadString(szLine))
				{
					szLine.TrimLeft();
					szLine.TrimRight();
					if(szLine != _T(""))
					{
						m_wndPlayListDlgBar.AddFileToTail(szLine, fFirst);
						AddFileToRecentFileList(szFilePath);
						fFirst = FALSE;
					}
				}
				file.Close();
			}
		}
		else
		{
			m_wndPlayListDlgBar.AddFileToTail(szFilePath, fFirst);
			AddFileToRecentFileList(szFilePath);
		}
		return TRUE;
	}
	return FALSE;
}

void CMainFrame::AddFileToRecentFileList(LPCTSTR pszFilename)
{
	CString szFilePath(pszFilename);
	if(m_pRecentFileList)
		m_pRecentFileList->Add(szFilePath);
	if(m_pRecentDirectoryList)
	{
		int nPos = szFilePath.ReverseFind(_T('\\'));
		if(nPos != -1)
		{
			szFilePath = szFilePath.Left(nPos + 1) + CString(_T('.'));
			m_pRecentDirectoryList->Add(szFilePath);
		}
	}
}

void CMainFrame::OnFileOpenclip() 
{
	if(OpenFile(NULL, TRUE, FALSE, TRUE))
		PlayCurrent();
}

void CMainFrame::OnFileAddclip() 
{
	OpenFile(NULL, TRUE, TRUE, (m_wndPlayListDlgBar.GetCount() == 0));
}

void CMainFrame::OnDropFiles(HDROP hDropInfo)
{
	TCHAR szPathname[MAX_PATH + 1];
	UINT nNumFilesDropped = DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
	for(UINT nFile = 0 ; nFile < nNumFilesDropped; nFile++)
	{
		DragQueryFile(hDropInfo, nFile, szPathname, MAX_PATH + 1);
		m_wndPlayListDlgBar.AddFileToTail(szPathname, nFile == 0);
		AddFileToRecentFileList(szPathname);
	}
	if(nFile > 0)
		PlayCurrent();
}

void CMainFrame::CleanGraphBuilder() 
{
	// Get filter enumerator
	IEnumFilters *pEnum = NULL;
	if(m_pGraphBuilder != NULL && SUCCEEDED(m_pGraphBuilder->EnumFilters(&pEnum)))
	{
		CArray<IBaseFilter *, IBaseFilter *> aBaseFilter;
		// Enumerate all filters in the graph
		IBaseFilter * pFilter = NULL;
		ULONG cFetched;
		while(pEnum->Next(1, &pFilter, &cFetched) == S_OK)
		{
			CString szFiltername(_T(""));
			FILTER_INFO FilterInfo;
			if(SUCCEEDED(pFilter->QueryFilterInfo(&FilterInfo)))
			{
				// Add the filter name to the filters listbox
				TCHAR szName[256];
				WideCharToMultiByte(CP_ACP, 0, FilterInfo.achName, -1, szName, 256, 0, 0);
				szFiltername = szName;
				FilterInfo.pGraph->Release();
			}
			if(szFiltername.Find(_T("Video Renderer")) == -1 && !(m_SettingsCurrent.m_fUseOverlayWhenPossible && szFiltername.Find(_T("Overlay")) != -1))
				aBaseFilter.Add(pFilter);

		}
		pEnum->Release();
		pEnum = NULL;

		int nCount = aBaseFilter.GetSize();
		for(int i = 0; i < nCount; i++)
		{
			pFilter = aBaseFilter[i];
			m_pGraphBuilder->RemoveFilter(pFilter);
			pFilter->Release();
			pFilter = NULL;
		}
	}
}

void CMainFrame::CloseCurrentClip() 
{
	TRACE(_T("CMainFrame::CloseCurrentClip\n"));

	//m_wndView.m_wndOSDView.ShowWindow(SW_HIDE);

	if(m_pGraphBuilder == NULL)
		return;

	m_MediaTime.Close();

	UnLoadSubtitle(); //m_SubtitleManager.Stop();
	m_wndSeekingDlgBar.m_wndSliderBar.EnableWindow(FALSE);

	if(m_hFastForwardSeekingTimer != 0)
	{
		KillTimer(m_hFastForwardSeekingTimer);
		m_hFastForwardSeekingTimer = 0;
	}

	if(m_hFrameSeekingTimer != 0)
	{
		KillTimer(m_hFrameSeekingTimer);
		m_hFrameSeekingTimer = 0;
	}

	if(m_hSlideShowTimer != 0)
	{
		KillTimer(m_hSlideShowTimer);
		m_hSlideShowTimer = 0;
	}

	m_wndView.m_fEraseBkgnd = TRUE;

    HRESULT hr;

    if(m_pMediaControl != NULL)
	{
        hr = m_pMediaControl->Stop();
		CString szPlayingStatus;
		szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_STOP);
		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);
	}

    m_enCurrentPlayState = PSStopped;
    m_bAudioOnly = TRUE;

    // Relinquish ownership (IMPORTANT!) after hiding video window
	if(m_pVideoWindow != NULL)
    {
        hr = m_pVideoWindow->put_Visible(OAFALSE);
        hr = m_pVideoWindow->put_Owner(NULL);
    }


	//CComPtr<IBaseFilter> pDelayFilter = NULL;
	//m_pGraphBuilder->FindFilterByName(L"Delay Filter", &pDelayFilter);

	//if(pDelayFilter != NULL)
	//{
	//	hr = m_pGraphBuilder->RemoveFilter(pDelayFilter);
	//	pDelayFilter = NULL;
	//}


//#ifdef REGISTER_FILTERGRAPH
    if(m_dwGraphRegister)
    {
        RemoveGraphFromRot(m_dwGraphRegister);
        m_dwGraphRegister = 0;
    }
//#endif

	m_pAMStreamSelect = NULL;

	m_pMediaEventEx = NULL;
	m_pMediaSeeking = NULL;
	m_pMediaControl = NULL;
	m_pBasicAudio = NULL;
	m_pBasicVideo = NULL;
	m_pVMRWindowlessControl = NULL;
	m_pVideoWindow = NULL;
	m_pGraphBuilder = NULL;

	if(m_pDelayTransformFilter)
	{
		CBasePin * pInputPin = m_pDelayTransformFilter->GetPin(0);
		if(pInputPin != NULL) pInputPin->Release();
//		m_pDelayTransformFilter->Release();
		delete m_pDelayTransformFilter;
		m_pDelayTransformFilter = NULL;
	}

//	CleanGraphBuilder();

    // Clear file name to allow selection of new file with open dialog
    m_szFileName = _T("");

    // No current media state
    m_enCurrentPlayState = PSUnload;

	CMenu * pMenu = GetMenu();
	if(pMenu)
		UpdateMenu(pMenu);

    RECT rect;
    GetClientRect(&rect);
    InvalidateRect(&rect, TRUE);

    UpdateMainTitle();
}

void CMainFrame::OnFileCloseClip() 
{
	CloseCurrentClip();

	m_wndPlayListDlgBar.DeleteCurrentFile();
}

void CMainFrame::OnFileCloseSelectedClip() 
{
	if(m_wndPlayListDlgBar.IsCurrentSelected())
		CloseCurrentClip();

	m_wndPlayListDlgBar.DeleteSelectedFile();
}

void CMainFrame::OnFileCloseAllClip() 
{
	CloseCurrentClip();

	m_wndPlayListDlgBar.DeleteAllFile();
}

void CMainFrame::OnFileAddSubtitle() 
{
	CString szFilter;
	szFilter.LoadString(IDS_STRING_OPENSUBTITLE_FILTER);

	CFileDialog FileDialog(TRUE, _T("*.*"), NULL,
		OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING,
		szFilter, this);
	
	char pszFile[30 * 1024];
	memset(pszFile, 0, sizeof(pszFile));
	FileDialog.m_ofn.nMaxFile = sizeof(pszFile);
	FileDialog.m_ofn.lpstrFile = pszFile;
//	if(pszPath)
//		FileDialog.m_ofn.lpstrInitialDir = pszPath;

	if(FileDialog.DoModal() == IDOK)
	{
		BOOL fResult = FALSE;
		CWaitCursor wc;
		POSITION pos = FileDialog.GetStartPosition();
		int nFirstSubtitleComboIndex = -1;
		while(pos)
		{
			BOOL fLocalResult;
			CString szFilePath = FileDialog.GetNextPathName(pos);
			UINT nSubtitleIndex = -1;
			fResult |= (fLocalResult = m_SubtitleManager.AddSubtitle(szFilePath, &nSubtitleIndex));

			if(fLocalResult && nSubtitleIndex != -1)
			{
				CString szFile = szFilePath;
				int nPos = szFile.ReverseFind(_T('\\'));
				if(nPos != -1)
					szFile = szFile.Mid(nPos + 1);
				int nIndex = m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.AddString(szFile);
				if(nIndex >= 0)
				{
					m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.SetItemData(nIndex, nSubtitleIndex);
					if(nFirstSubtitleComboIndex == -1)
						nFirstSubtitleComboIndex = nIndex;
				}
			}
		}
		if(fResult && m_enCurrentPlayState == PSRunning && nFirstSubtitleComboIndex != -1)
		{
			//m_SubtitleManager.Select(nFirstSubtitleIndex); //Start();
			m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.SetCurSel(nFirstSubtitleComboIndex);
			m_wndSubtitleDlgBar.OnSelchangeComboSubtitle();
		}
	}
}

BOOL CMainFrame::LoadSubtitleFromMovieFileName(LPCTSTR pszFilename)
{
	CWaitCursor wc;

	CString szPath(pszFilename);

	int nPos = szPath.ReverseFind(_T('\\'));
	if(nPos != -1)
		szPath = szPath.Left(nPos + 1);
	else
		szPath = "";

	CString szFilename(pszFilename);
	szFilename = szFilename.Mid(nPos + 1);

	nPos = szFilename.ReverseFind(_T('.'));
	if(nPos != -1)
		szFilename = szFilename.Left(nPos);

	CString szFilenames(szPath);
	szFilenames += szFilename + ".*";

	int nFirstSubtitleComboIndex = -1;
	WIN32_FIND_DATA findFileData;
	HANDLE fd = FindFirstFile(szFilenames, &findFileData);
	if(fd != INVALID_HANDLE_VALUE)
	{
		BOOL fResult = FALSE;
		BOOL fFoundFile = FALSE;
		do
		{
			if(!(findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				BOOL fLocalResult;
				CString szFilePath = szPath + findFileData.cFileName;
				UINT nSubtitleIndex = -1;
				fResult |= (fLocalResult = m_SubtitleManager.AddSubtitle(szFilePath, &nSubtitleIndex));

				if(fLocalResult && nSubtitleIndex != -1)
				{
					CString szFile = szFilePath;
					int nPos = szFile.ReverseFind(_T('\\'));
					if(nPos != -1)
						szFile = szFile.Mid(nPos + 1);
					int nIndex = m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.AddString(szFile);
					if(nIndex >= 0)
					{
						m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.SetItemData(nIndex, nSubtitleIndex);
						if(nFirstSubtitleComboIndex == -1)
							nFirstSubtitleComboIndex = nIndex;
					}
				}
			}
		}
		while(FindNextFile(fd, &findFileData));
		FindClose(fd);

		if(fResult && /*m_enCurrentPlayState == PSRunning && */nFirstSubtitleComboIndex != -1)
		{
			//m_SubtitleManager.Select(nFirstSubtitleIndex); //Start();
			m_wndSubtitleDlgBar.m_wndComboBoxSubtitle.SetCurSel(nFirstSubtitleComboIndex);
			m_wndSubtitleDlgBar.OnSelchangeComboSubtitle();
		}

		return fFoundFile;
	}
	return FALSE;
}

void CMainFrame::UnLoadSubtitle()
{
	if(m_SubtitleManager.GetSubtitleCount() > 0)
	{
		m_SubtitleManager.Clear();
		m_wndSubtitleDlgBar.Clear();
	}
}

void CMainFrame::OnControlRepeatAb() 
{
	TRACE(_T("CMainFrame::OnControlRepeatAb m_nRepeatABMode = %d\n"), m_nRepeatABMode);
	if(m_pMediaSeeking != NULL && m_pMediaControl != NULL)
	{
		if(m_nRepeatABMode == 0)
		{
			m_nRepeatABMode = 1;
			m_pMediaSeeking->GetCurrentPosition(&m_llRepeatAPosition);

			m_wndSeekingDlgBar.m_wndSliderBar.SetSelection((int)(m_llRepeatAPosition / m_MediaTime.m_dwFrameStep), (int)((m_llRepeatAPosition + 1) / m_MediaTime.m_dwFrameStep));
			return;
		}
		if(m_nRepeatABMode == 1)
		{
			m_nRepeatABMode = 2;
			m_pMediaSeeking->GetCurrentPosition(&m_llRepeatBPosition);

			m_pMediaSeeking->SetPositions(
				&m_llRepeatAPosition, AM_SEEKING_AbsolutePositioning,
				&m_llRepeatBPosition, AM_SEEKING_AbsolutePositioning);

			m_wndSeekingDlgBar.m_wndSliderBar.SetSelection((int)(m_llRepeatAPosition / m_MediaTime.m_dwFrameStep), (int)(m_llRepeatBPosition / m_MediaTime.m_dwFrameStep));
			m_wndSeekingDlgBar.m_wndSliderBar.Invalidate();
			return;
		}
		if(m_nRepeatABMode == 2)
		{
			m_nRepeatABMode = 0;
			m_pMediaSeeking->GetCurrentPosition(&m_llRepeatAPosition);
			m_pMediaSeeking->GetDuration(&m_llRepeatBPosition);
			m_pMediaSeeking->SetPositions(
				&m_llRepeatAPosition, AM_SEEKING_AbsolutePositioning,
				&m_llRepeatBPosition, AM_SEEKING_AbsolutePositioning);
			m_pMediaControl->Run();

			CString szPlayingStatus;
			szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PLAYING);
			m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);

			m_SubtitleManager.Start();

			m_wndSeekingDlgBar.m_wndSliderBar.ClearSel(TRUE);

			return;
		}
	}
}

void CMainFrame::OnControlStepByStep() 
{
    // Get the Frame Stepping Interface
    CComPtr<IVideoFrameStep> pVideoFrameStep = NULL;

    HRESULT hr = m_pGraphBuilder->QueryInterface(__uuidof(IVideoFrameStep), (PVOID *)&pVideoFrameStep);
    if(FAILED(hr))
        return;

    // The graph must be paused for frame stepping to work
    if(m_enCurrentPlayState != PSPaused)
        OnControlPlayPause();

    // Step one frame
    hr = pVideoFrameStep->Step(1, NULL); 

    // Clean up
    //pVideoFrameStep->Release();
	pVideoFrameStep = NULL;
}

void CMainFrame::OnControlPlayPause() 
{
    HRESULT hr;
    
    if(m_pMediaControl == NULL)
        return;

    // Toggle play/pause behavior
    if(	(m_enCurrentPlayState == PSPaused) ||
		(m_enCurrentPlayState == PSStopped) ||
		(m_enCurrentPlayState == PSFastRewind) ||
		(m_enCurrentPlayState == PSFastForward))
    {
		LONGLONG llStopPosition;
		m_pMediaSeeking->GetDuration(&llStopPosition);
		m_pMediaSeeking->SetPositions(
			NULL, AM_SEEKING_NoPositioning,
			&llStopPosition, AM_SEEKING_AbsolutePositioning);

        hr = m_pMediaControl->Run();

		CString szPlayingStatus;
		szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PLAYING);
		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);

        m_enCurrentPlayState = PSRunning;

		m_SubtitleManager.Start();

		if(m_hSlideShowTimer)
			KillTimer(m_hSlideShowTimer);
		if(m_fIsAPicture && m_SettingsCurrent.m_fSlideshow && m_SettingsCurrent.m_nSlideshowWaitingTime)
			m_hSlideShowTimer = SetTimer(TIMER_SLIDESHOW, m_SettingsCurrent.m_nSlideshowWaitingTime, NULL);
    }
    else
    {
        hr = m_pMediaControl->Pause();

		CString szPlayingStatus;
		szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PAUSE);
		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);

        m_enCurrentPlayState = PSPaused;

		m_SubtitleManager.Pause();

		if(m_hSlideShowTimer)
			KillTimer(m_hSlideShowTimer);
    }

	if(m_hFastForwardSeekingTimer != 0)
	{
		KillTimer(m_hFastForwardSeekingTimer);
		m_hFastForwardSeekingTimer = 0;
	}

    UpdateMainTitle();

	if(!m_hFrameSeekingTimer)
		m_hFrameSeekingTimer = SetTimer(TIMER_FRAME_SEEKING, 100, NULL);
}

void CMainFrame::OnControlStop() 
{
    HRESULT hr;
    
    if(m_pMediaControl == NULL || m_pMediaSeeking == NULL)
        return;

    // Stop and reset postion to beginning
//    if((m_enCurrentPlayState == PSPaused) || (m_enCurrentPlayState == PSRunning))
    {
        LONGLONG pos = 0;
        hr = m_pMediaControl->Stop();

		CString szPlayingStatus;
		szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_STOP);
		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);

        m_enCurrentPlayState = PSStopped;
		m_SubtitleManager.Stop();

        hr = m_pMediaSeeking->SetPositions(&pos, AM_SEEKING_AbsolutePositioning ,
            NULL, AM_SEEKING_NoPositioning);

        // Display the first frame to indicate the reset condition
        hr = m_pMediaControl->Pause();

		szPlayingStatus.LoadString(IDS_STRING_PLAY_MOVIE_PAUSE);
		m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_STATUS), szPlayingStatus);
    }

    UpdateMainTitle();

	if(m_hFastForwardSeekingTimer != 0)
	{
		KillTimer(m_hFastForwardSeekingTimer);
		m_hFastForwardSeekingTimer = 0;
	}

	if(m_hFrameSeekingTimer != 0)
	{
		KillTimer(m_hFrameSeekingTimer);
		m_hFrameSeekingTimer = 0;
	}

	if(m_hSlideShowTimer != 0)
	{
		KillTimer(m_hSlideShowTimer);
		m_hSlideShowTimer = 0;
	}
}

void CMainFrame::OnControlFastRewind() 
{
	if(m_enCurrentPlayState == PSFastRewind)
		m_nFastForwardSpeed *= 2;
	else
		m_nFastForwardSpeed = 4;
	m_enCurrentPlayState = PSFastRewind;

	CString szText, szPower;
	szText.LoadString(IDM_CONTROL_FAST_REWIND);
	szPower.Format(_T(" x%d"), m_nFastForwardSpeed);
	szText += szPower;
	m_wndView.ShowText(szText);
	m_fShowDefaultOSD = FALSE;

	OnTimer(TIMER_FAST_FORWARD_SEEKING);
	if(!m_hFastForwardSeekingTimer)
		m_hFastForwardSeekingTimer = SetTimer(TIMER_FAST_FORWARD_SEEKING, m_nFastForwardSeekingTime, NULL);
}

void CMainFrame::OnControlFastForward() 
{
	if(m_enCurrentPlayState == PSFastForward)
		m_nFastForwardSpeed *= 2;
	else
		m_nFastForwardSpeed = 4;
	m_enCurrentPlayState = PSFastForward;

	CString szText, szPower;
	szText.LoadString(IDM_CONTROL_FAST_FORWARD);
	szPower.Format(_T(" x%d"), m_nFastForwardSpeed);
	szText += szPower;
	m_wndView.ShowText(szText);
	m_fShowDefaultOSD = FALSE;

	OnTimer(TIMER_FAST_FORWARD_SEEKING);
	if(!m_hFastForwardSeekingTimer)
		m_hFastForwardSeekingTimer = SetTimer(TIMER_FAST_FORWARD_SEEKING, m_nFastForwardSeekingTime, NULL);
}

void CMainFrame::OnControlPreviousFile() 
{
	LPCTSTR pszFilename = m_wndPlayListDlgBar.GetPreviousFile();
	if(!pszFilename)
		pszFilename = m_wndPlayListDlgBar.GetFile(m_wndPlayListDlgBar.GetCount() - 1);
	if(pszFilename)
		PlayFile(pszFilename, TRUE);
}

void CMainFrame::OnControlNextFile() 
{
	PlayNextFile();
}

void CMainFrame::OnControlPreviousFolderFile() 
{
	if(m_wndFileBrowserDlgBar.SelectPreviousEntry())
	{
		CTreeNode * pTreeNode = m_wndFileBrowserDlgBar.GetCurrentSelection();
		if(pTreeNode != NULL && pTreeNode->szPath != "")
			PlayFile(pTreeNode->szPath, FALSE);
	}
}

void CMainFrame::OnControlNextFolderFile() 
{
	if(m_wndFileBrowserDlgBar.SelectNextEntry())
	{
		CTreeNode * pTreeNode = m_wndFileBrowserDlgBar.GetCurrentSelection();
		if(pTreeNode != NULL && pTreeNode->szPath != "")
			PlayFile(pTreeNode->szPath, FALSE);
	}
}

void CMainFrame::OnViewMute() 
{
	m_SettingsCurrent.m_fSoundMute ^= TRUE;
	UpdateMainTitle();

	if(m_pGraphBuilder == NULL || m_pBasicAudio == NULL)
        return;

    // Set new volume
	m_pBasicAudio->put_Volume(!m_SettingsCurrent.m_fSoundMute ? m_SettingsCurrent.m_nSoundLevel : -10000L);
}

void CMainFrame::OnViewSizeHalf() 
{
	m_rZoom = 0.5;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewSizeThreequarter() 
{
	m_rZoom = 3.0 / 4.0;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewSizeNormal() 
{
	m_rZoom = 1.0;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewSizeDouble() 
{
	m_rZoom = 2.0;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewFullscreen() 
{
	static CRect m_rectWnd;
	static BOOL fShowStatusBar;
	static BOOL fShowToolBar;
	static BOOL fShowSeekingDlgBar;
	static BOOL fShowControlDlgBar;
	static BOOL fShowPlayListDlgBar;
	static BOOL fShowFileBrowserDlgBar;
	static BOOL fShowSubtitleDlgBar;

	if(!m_fFullscreen)
	{
		if(m_bAudioOnly)
			return;

		GetWindowRect(&m_rectWnd);

		CRect rectScreen(0, 0, 0, 0);
		SystemParametersInfo(SPI_GETWORKAREA, NULL, &rectScreen, 0);

		CWnd * pWnd = GetDesktopWindow();
		if(pWnd && ::IsWindow(pWnd->m_hWnd))
			pWnd->GetWindowRect(&rectScreen);

		if(m_fMultipleMonitorSupported)
		{
			HMONITOR hMonitor = MonitorFromRect(&m_rectWnd, MONITOR_DEFAULTTONEAREST);
			if(hMonitor)
			{
				MONITORINFO mi;
				mi.cbSize = sizeof(MONITORINFO);
				GetMonitorInfo(hMonitor, &mi);
				rectScreen = mi.rcMonitor;
			}
		}

		fShowStatusBar = m_wndStatusBar.IsVisible();
		fShowToolBar = m_wndToolBar.IsVisible();
		fShowSeekingDlgBar = m_wndSeekingDlgBar.IsVisible();
		fShowControlDlgBar = m_wndControlDlgBar.IsVisible();
		fShowPlayListDlgBar = m_wndPlayListDlgBar.IsVisible();
		fShowFileBrowserDlgBar = m_wndFileBrowserDlgBar.IsVisible();
		fShowSubtitleDlgBar = m_wndSubtitleDlgBar.IsVisible();

		m_wndStatusBar.ShowWindow(SW_HIDE);
		m_wndToolBar.ShowWindow(SW_HIDE);
		m_wndSeekingDlgBar.ShowWindow(SW_HIDE);
		m_wndControlDlgBar.ShowWindow(SW_HIDE);
		m_wndPlayListDlgBar.ShowWindow(SW_HIDE);
		m_wndFileBrowserDlgBar.ShowWindow(SW_HIDE);
		m_wndSubtitleDlgBar.ShowWindow(SW_HIDE);

		SetMenu(NULL);
		m_Menu.DestroyMenu();
		ModifyStyle(WS_CAPTION | WS_BORDER | WS_DLGFRAME | WS_SYSMENU | WS_THICKFRAME | WS_GROUP | WS_TABSTOP | WS_MINIMIZEBOX | WS_MAXIMIZEBOX, WS_POPUP);
		SetWindowPos(&wndTop, rectScreen.left, rectScreen.top, rectScreen.Width(), rectScreen.Height(), SWP_SHOWWINDOW);

		ShowCursor(FALSE);
		SetCapture();

		m_wndView.SetFocus();

		m_fFullscreen = TRUE;
	}
	else
	{
		ReleaseCapture();
		ShowCursor(TRUE);
		ModifyStyle(0, WS_VISIBLE | WS_CLIPSIBLINGS | WS_CAPTION | WS_BORDER | WS_DLGFRAME | WS_SYSMENU | WS_THICKFRAME | WS_GROUP | WS_TABSTOP | WS_MINIMIZEBOX | WS_MAXIMIZEBOX);
		m_Menu.LoadMenu(IDR_MAINFRAME);
		SetMenu(&m_Menu);

		m_wndStatusBar.ShowWindow(fShowStatusBar);
		m_wndToolBar.ShowWindow(fShowToolBar);
		m_wndSeekingDlgBar.ShowWindow(fShowSeekingDlgBar);
		m_wndControlDlgBar.ShowWindow(fShowControlDlgBar);
		m_wndPlayListDlgBar.ShowWindow(fShowPlayListDlgBar);
		m_wndFileBrowserDlgBar.ShowWindow(fShowFileBrowserDlgBar);
		m_wndSubtitleDlgBar.ShowWindow(fShowSubtitleDlgBar);

		SetWindowPos(&wndNoTopMost, m_rectWnd.left, m_rectWnd.top, m_rectWnd.Width(), m_rectWnd.Height(), SWP_SHOWWINDOW | SWP_FRAMECHANGED);
		RecalcLayout();
		m_fFullscreen = FALSE;
	}
}

void CMainFrame::OnViewDesktop() 
{
	INT nElements = COLOR_DESKTOP;

	if(!m_fDesktopMode)
	{
		if(m_hMutex)
		{
			ReleaseMutex(m_hMutex);
			CloseHandle(m_hMutex);
		}

		//if(m_fOSDPossible)
		//	m_wndView.m_wndOSDView.ShowWindow(SW_HIDE);

		if(!m_fFullscreen)
			OnViewFullscreen();

		SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, _T(""), 0);

		COLORREF colorValues = GetColorKey();
		m_colorOldValues = GetSysColor(nElements);
		if(m_colorOldValues != colorValues)
			SetSysColors(1, &nElements, &colorValues);
		m_fDesktopMode = TRUE;

		ShowWindow(SW_HIDE);

		Shell_NotifyIcon(NIM_ADD,&m_nidIconData);
	}
	else
	{
		Shell_NotifyIcon(NIM_DELETE,&m_nidIconData);

		ShowWindow(SW_SHOW);

		SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, NULL, 0);
		COLORREF colorOldValues = GetSysColor(nElements);
		if(m_colorOldValues != colorOldValues)
			SetSysColors(1, &nElements, &m_colorOldValues);
		m_fDesktopMode = FALSE;

		if(m_fFullscreen)
			OnViewFullscreen();

		//if(m_fOSDPossible)
		//	m_wndView.m_wndOSDView.ShowWindow(SW_SHOW);

		m_hMutex = CreateMutex(NULL, FALSE, _T("LePlayerDuBonheur"));
	}
}

void CMainFrame::OnUpdateViewDesktop(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fDesktopMode);
	pCmdUI->Enable(m_fOverlayUsed);
}

void CMainFrame::OnView169() 
{
	m_fRatio169 ^= TRUE;
	if(m_fRatio169)
	{
		m_fWindowWidthFit = TRUE;
		m_fWindowFit = TRUE;
	}
	UpdateDestinationView();
}

void CMainFrame::OnViewMaintainAspectRatio() 
{
	m_fMaintainAspectRatio ^= TRUE;
	UpdateDestinationView();
}

void CMainFrame::OnViewCenter() 
{
	m_fViewCenter ^= TRUE;
	UpdateDestinationView();
}

void CMainFrame::OnUpdateFileAddclip(CCmdUI* pCmdUI) 
{
//	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount());
}
void CMainFrame::OnUpdateFileCloseClip(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount()); //m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateFileCloseSelectedClip(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount()); //m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateFileCloseAllClip(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount()); //m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateFileAddSubtitle(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateControlPlayPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateControlStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateControlStepByStep(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateControlRepeatAb(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateControlPreviousFile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount() > 1 && m_wndPlayListDlgBar.GetPreviousFile());
}

void CMainFrame::OnUpdateControlNextFile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount() > 1 && m_wndPlayListDlgBar.GetNextFile());
}

void CMainFrame::OnUpdateControlPreviousFolderFile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE); //m_wndPlayListDlgBar.GetCount() > 1 && m_wndPlayListDlgBar.GetPreviousFile());
}

void CMainFrame::OnUpdateControlNextFolderFile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE); //m_wndPlayListDlgBar.GetCount() > 1 && m_wndPlayListDlgBar.GetNextFile());
}

void CMainFrame::OnUpdateControlFastRewind(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateControlFastForward(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateViewMute(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_SettingsCurrent.m_fSoundMute);
}

void CMainFrame::OnUpdateViewSizeHalf(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateViewSizeThreequarter(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateViewSizeNormal(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateViewSizeDouble(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_enCurrentPlayState != PSUnload);
}

void CMainFrame::OnUpdateViewFullscreen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_fDesktopMode);
	pCmdUI->SetCheck(m_fFullscreen);
}

void CMainFrame::OnUpdateView169(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fRatio169);
}

void CMainFrame::OnUpdateViewMaintainAspectRatio(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fMaintainAspectRatio);
}

void CMainFrame::OnUpdateViewCenter(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fViewCenter);
}

void CMainFrame::OnViewAspectRatioPlus() 
{
	m_rCurrentAspectRatio /= m_rAspectRatioStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewAspectRatioMoins() 
{
	m_rCurrentAspectRatio *= m_rAspectRatioStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewAspectRatioReset() 
{
	if(m_pBasicVideo)
	{
		long nSourceWidth = 0, nSourceHeight = 0;
		m_pBasicVideo->GetVideoSize(&nSourceWidth, &nSourceHeight);
		
		if(nSourceHeight)
			m_rCurrentAspectRatio = (double)nSourceWidth / (double)nSourceHeight;
		else
			m_rCurrentAspectRatio = 0.0;
		UpdateDestinationView();
	}
}

void CMainFrame::OnViewScrollDown() 
{
	m_fViewCenter = FALSE;
	int nOffsetStep = m_nOffsetStepNormal;
	if(GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1))
		nOffsetStep = m_nOffsetStepSlow;
	m_nOffsetCenterY += nOffsetStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewScrollUp() 
{
	m_fViewCenter = FALSE;
	int nOffsetStep = m_nOffsetStepNormal;
	if(GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1))
		nOffsetStep = m_nOffsetStepSlow;
	m_nOffsetCenterY -= nOffsetStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewScrollLeft() 
{
	m_fViewCenter = FALSE;
	int nOffsetStep = m_nOffsetStepNormal;
	if(GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1))
		nOffsetStep = m_nOffsetStepSlow;
	m_nOffsetCenterX -= nOffsetStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewScrollRight() 
{
	m_fViewCenter = FALSE;
	int nOffsetStep = m_nOffsetStepNormal;
	if(GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1))
		nOffsetStep = m_nOffsetStepSlow;
	m_nOffsetCenterX += nOffsetStep;
	UpdateDestinationView();
}

void CMainFrame::OnViewScrollReset() 
{
	m_fViewCenter = TRUE;
	UpdateDestinationView();
}

void CMainFrame::OnViewZoomPlus() 
{
	m_rZoom *= m_rZoomStep;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewZoomMoins() 
{
	m_rZoom /= m_rZoomStep;
	m_fWindowFit = FALSE;
	UpdateDestinationView();
}

void CMainFrame::OnViewZoomReset() 
{
//	m_fViewCenter = TRUE;
	if(m_rZoom == 1.0) // Go to fit mode
	{
		m_fWindowFit = TRUE;
		UpdateDestinationView();
//		m_fWindowFit = FALSE;
	}
	else // Go to scale 1:1 with free mode
	{
		m_rZoom = 1.0;
		m_fWindowFit = FALSE;
		UpdateDestinationView();
//		m_fWindowFit = TRUE;
	}
//	m_fViewCenter = FALSE;
}

BOOL CMainFrame::DestroyWindow() 
{
	CloseCurrentClip();

	if(m_fScreenSaverMode && m_fScreenSaverNoDisturb)
		return FALSE;

	if(!m_fScreenSaverMode)
	{
		// Write Miscellaneous settings
		//GetWindowRect(&m_SettingsCurrent.m_rectWindowPosition);
		//AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("PositionLeft"), m_SettingsCurrent.m_rectWindowPosition.left);
		//AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("PositionTop"), m_SettingsCurrent.m_rectWindowPosition.top);
		//AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("PositionRight"), m_SettingsCurrent.m_rectWindowPosition.right);
		//AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("PositionBottom"), m_SettingsCurrent.m_rectWindowPosition.bottom);
		WINDOWPLACEMENT windowPlacement;
		GetWindowPlacement(&windowPlacement);
		AfxGetApp()->WriteProfileBinary(INI_SETTINGS_MISC, _T("WindowPlacement"), (LPBYTE)&windowPlacement, sizeof(WINDOWPLACEMENT));
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("TopMost"), m_SettingsCurrent.m_fTopMost);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("PlayRandomly"), m_SettingsCurrent.m_fPlayRandom);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("Mute"), m_SettingsCurrent.m_fSoundMute);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("SoundLevel"), m_SettingsCurrent.m_nSoundLevel);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("SoundBalance"), m_SettingsCurrent.m_nSoundBalance);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("UseOverlayWhenPossible"), m_SettingsCurrent.m_fUseOverlayWhenPossible);
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("ShowOSD"), m_SettingsCurrent.m_fShowOSD);

		m_SettingsCurrent.m_nSortAttribute = m_wndFileBrowserDlgBar.m_LeftView.m_nSortAttribute;
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_MISC, _T("SortAttribute"), m_SettingsCurrent.m_nSortAttribute);

		CString sProfile(_T("BarState"));
		m_wndControlDlgBar.SaveState(sProfile);
		m_wndPlayListDlgBar.SaveState(sProfile);
		m_wndFileBrowserDlgBar.SaveState(sProfile);
		m_wndSubtitleDlgBar.SaveState(sProfile);;
		SaveBarState(sProfile);
		if(m_pRecentFileList)
			m_pRecentFileList->WriteList();
		if(m_pRecentDirectoryList)
			m_pRecentDirectoryList->WriteList();
	}
	else
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverCurrentItem"), m_wndPlayListDlgBar.GetCurrentFileId());
	
	if(m_nidIconData.hWnd && m_nidIconData.uID>0 && m_fDesktopMode)
		Shell_NotifyIcon(NIM_DELETE,&m_nidIconData);

	return CFrameWnd::DestroyWindow();
}

void CMainFrame::OnViewControlBar() 
{
	BOOL bShow = m_wndControlDlgBar.IsVisible();
	ShowControlBar(&m_wndControlDlgBar, !bShow, FALSE);
}

void CMainFrame::OnUpdateViewControlBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndControlDlgBar.IsVisible());
}

void CMainFrame::OnViewPlayListBar() 
{
	BOOL bShow = m_wndPlayListDlgBar.IsVisible();
	ShowControlBar(&m_wndPlayListDlgBar, !bShow, FALSE);
}

void CMainFrame::OnUpdateViewPlayListBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndPlayListDlgBar.IsVisible());
}

void CMainFrame::OnViewFileBrowserBar()
{
	BOOL bShow = m_wndFileBrowserDlgBar.IsVisible();
	ShowControlBar(&m_wndFileBrowserDlgBar, !bShow, FALSE);
}

void CMainFrame::OnUpdateViewFileBrowserBar(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_wndFileBrowserDlgBar.IsVisible());
}

void CMainFrame::OnViewSubtitleBar() 
{
	BOOL bShow = m_wndSubtitleDlgBar.IsVisible();
	ShowControlBar(&m_wndSubtitleDlgBar, !bShow, FALSE);
}

void CMainFrame::OnUpdateViewSubtitleBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndSubtitleDlgBar.IsVisible());
}

void CMainFrame::OnViewSeekingBar() 
{
	BOOL bShow = m_wndSeekingDlgBar.IsVisible();
	ShowControlBar(&m_wndSeekingDlgBar, !bShow, FALSE);
}

void CMainFrame::OnUpdateViewSeekingBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndSeekingDlgBar.IsVisible());
}

// This function is Copyright (c) 2000, Cristi Posea.
// See www.datamekanix.com for more control bars tips&tricks.
BOOL CMainFrame::VerifyBarState(LPCTSTR lpszProfileName)
{
    CDockState state;
    state.LoadState(lpszProfileName);

    for (int i = 0; i < state.m_arrBarInfo.GetSize(); i++)
    {
        CControlBarInfo* pInfo = (CControlBarInfo*)state.m_arrBarInfo[i];
        ASSERT(pInfo != NULL);
        int nDockedCount = pInfo->m_arrBarID.GetSize();
        if (nDockedCount > 0)
        {
            // dockbar
            for (int j = 0; j < nDockedCount; j++)
            {
                UINT nID = (UINT) pInfo->m_arrBarID[j];
                if (nID == 0) continue; // row separator
                if (nID > 0xFFFF)
                    nID &= 0xFFFF; // placeholder - get the ID
                if (GetControlBar(nID) == NULL)
                    return FALSE;
            }
        }
        
        if (!pInfo->m_bFloating) // floating dockbars can be created later
            if (GetControlBar(pInfo->m_nBarID) == NULL)
                return FALSE; // invalid bar ID
    }

    return TRUE;
}

void CMainFrame::OnViewSizeWindowFit() 
{
	m_fWindowWidthFit ^= TRUE;
	m_fWindowFit = TRUE;
//	m_fViewCenter = TRUE;
	UpdateDestinationView();
//	m_fViewCenter = FALSE;
}

void CMainFrame::OnUpdateViewSizeWindowFit(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fWindowFit);	
}

void CMainFrame::OnUpdateViewVolumeMoins(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateViewVolumePlus(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateViewBalanceLeft(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateViewBalanceRight(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateControlSpeedPlus(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateControlSpeedNormal(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::OnUpdateControlSpeedMoins(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CMainFrame::UpdateMenu(CMenu * pMenu)
{
	// Update the filter properties menu
	CMenu * pMenuControl = NULL;
	if(pMenu && (pMenuControl = pMenu->GetSubMenu(1)))
	{
		CMenu * pPropertyMenu = pMenuControl->GetSubMenu(pMenuControl->GetMenuItemCount() - 1);
		if(pPropertyMenu)
		{
			// Delete previous Filters
			for(int i = 0; i < 20; i++)
				pPropertyMenu->DeleteMenu(IDM_CONTROL_PROPERTY1 + i, MF_BYCOMMAND);
			
			// Get filter enumerator
			CComPtr<IEnumFilters> pEnum = NULL;
			if(m_pGraphBuilder != NULL && SUCCEEDED(m_pGraphBuilder->EnumFilters(&pEnum)))
			{
				CStringArray aszMenuItem;
				CUIntArray afMenuItem;

				// Enumerate all filters in the graph
				CComPtr<IBaseFilter> pFilter = NULL;
				ULONG cFetched;
				while(pEnum->Next(1, &pFilter, &cFetched) == S_OK)
				{
					FILTER_INFO FilterInfo;
					if(SUCCEEDED(pFilter->QueryFilterInfo(&FilterInfo)))
					{
						// Add the filter name to the filters listbox
						TCHAR szName[256];
						WideCharToMultiByte(CP_ACP, 0, FilterInfo.achName, -1, szName, 256, 0, 0);
						aszMenuItem.InsertAt(0, szName);
						afMenuItem.InsertAt(0, (UINT)SupportsPropertyPage(pFilter));
						FilterInfo.pGraph->Release();
					}
					pFilter = NULL;
				}
				pEnum = NULL;

				MENUITEMINFO menuItemInfo;
				menuItemInfo.cbSize = sizeof(MENUITEMINFO);
				menuItemInfo.fMask = MIIM_DATA;
				int nMaxFilter = (aszMenuItem.GetSize() > 20 ? 20 : aszMenuItem.GetSize());
				for(int i = 0; i < nMaxFilter; i++)
				{
					pPropertyMenu->AppendMenu(MF_GRAYED, IDM_CONTROL_PROPERTY1 + i, aszMenuItem[i]);
					menuItemInfo.dwItemData = afMenuItem[i];
					pPropertyMenu->SetMenuItemInfo(IDM_CONTROL_PROPERTY1 + i, &menuItemInfo);
				}
			}
		}

		int nLanguageMenuPosition = 13;
		CMenu * pLanguageMenu = pMenuControl->GetSubMenu(nLanguageMenuPosition);

		// Update the language menu
//			IDM_CONTROL_PLAY_RANDOM;
		if(pLanguageMenu)
		{
			// Delete previous Filters
			for(int i = 0; i < 20; i++)
				pLanguageMenu->DeleteMenu(IDM_CONTROL_LANGUAGE1 + i, MF_BYCOMMAND);
			
			DWORD nCount = 0;
			if(m_pAMStreamSelect != NULL)
			{
				m_pAMStreamSelect->Count(&nCount);
				for(DWORD i = 0; i < nCount; i++)
				{
					WCHAR * pszName;
//					AM_MEDIA_TYPE * pMediaType;
					DWORD dwFlags = 0;

//					if(SUCCEEDED(pAMStreamSelect->Info(i, &pMediaType, &dwFlags, &lcid, &dwGroup, (WCHAR **)&pszName, &pObject, &pUnk)))
					if(SUCCEEDED(m_pAMStreamSelect->Info(i, NULL, &dwFlags, NULL, NULL, &pszName, NULL, NULL)))
					{
						TCHAR pszStreamName[256];
#ifndef UNICODE
						WideCharToMultiByte(CP_ACP, 0, pszName, -1, pszStreamName, sizeof(pszStreamName), NULL, NULL);
#else
						lstrcpy(pszName, pszStreamName);
#endif
						pLanguageMenu->AppendMenu(MF_ENABLED, IDM_CONTROL_LANGUAGE1 + i, pszStreamName);

						TRACE(_T("Stream(%d) = %s\n"), i, pszStreamName);
						CoTaskMemFree((LPVOID)pszName);
//						DeleteMediaType(pMediaType);
					}
				}
			}
			pMenuControl->EnableMenuItem(nLanguageMenuPosition, MF_BYPOSITION | (nCount ? MF_ENABLED : MF_GRAYED));
		}
	}
}

void CMainFrame::ShowMenu(CPoint & point)
{
	TRACE("CMainFrame::ShowMenu\n");

	BOOL fShowCursor = m_fFullscreen;

	if(fShowCursor)
		ShowCursor(TRUE);

	CMenu menu;
	CMenu * pMenu;
	if(!GetMenu())
	{
		menu.LoadMenu(IDR_MAINFRAME);
		pMenu = &menu;
	}
	else
		pMenu = GetMenu();

	UpdateMenu(pMenu);

	CMenu * pMenuControl = NULL;
	if(pMenu &&
		(pMenuControl = pMenu->GetSubMenu(1)))
		pMenuControl->TrackPopupMenu(0, point.x, point.y, this);

	if(!GetMenu())
		menu.DestroyMenu();

	if(fShowCursor)
		ShowCursor(FALSE);
}

LRESULT CMainFrame::OnTrayNotify(WPARAM wParam, LPARAM lParam) 
{ 
    UINT uID; 
    UINT uMsg; 
	
    uID = (UINT) wParam; 
    uMsg = (UINT) lParam; 
	
	if (uID != 1)
		return 0;
	
	CPoint pt;	

    switch(uMsg) 
	{ 
	case WM_RBUTTONDOWN:
	case WM_CONTEXTMENU:
		GetCursorPos(&pt);
		ShowMenu(pt);
		break;
    } 
	return 0; 
}

LRESULT CMainFrame::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	CString szRemoteCommand;
	int nRepeatCount;

	if(m_fLircSupport && (message == WM_GIRDER_SEND_EVENT || m_RemoteControl.MessageCapture0(message, szRemoteCommand, nRepeatCount)))
	{
		if(message == WM_GIRDER_SEND_EVENT)
		{
			szRemoteCommand = (LPCTSTR)lParam;
			nRepeatCount = wParam;
			m_RemoteControl.MessageCapture1(message, szRemoteCommand, nRepeatCount);
		}

		UINT nIndex;
		if(m_mapLircCommandToRemoteCommandParameterIndex.Lookup(szRemoteCommand, nIndex))
		{
			if(nIndex < (UINT)m_SettingsCurrent.m_aLircCommandParameters.GetSize())
			{
				if(nRepeatCount == 0 ||
						(m_SettingsCurrent.m_aLircCommandParameters[nIndex].fAcceptRepeat && nRepeatCount != 0))
				{
					TRACE(_T("	->Sent\n"));
					PostMessage(WM_COMMAND, m_SettingsCurrent.m_aLircCommandParameters[nIndex].nWndCommand);
				}
			}
		}
		return S_OK;
	}

	if(message == WM_ACTIVATEAPP)
	{
		if(!m_fDesktopMode)
		{
			if (m_fScreenSaverMode && !wParam)
				DestroyWindow();
		}
	}

	if(message == WM_COPYDATA && lParam)
	{
		PCOPYDATASTRUCT pMyCDS = (PCOPYDATASTRUCT)lParam;
		if(pMyCDS->dwData == 217)
		{
			BOOL fSetCurrent = (m_wndPlayListDlgBar.GetCount() == 0);
//			TCHAR pszLongPath[2048];
//			if(GetLongPathName((LPCTSTR)pMyCDS->lpData, pszLongPath, sizeof(pszLongPath)))
//				OpenFile((LPCTSTR)pszLongPath, FALSE, TRUE, fSetCurrent);
//			else

			CString szMediaFiles = (LPCTSTR)pMyCDS->lpData;
			int nStart = 0, nPos;
			while((nPos = szMediaFiles.Find('\n', nStart)) != -1)
			{
				CString szMediaFile = szMediaFiles.Mid(nStart, nPos - nStart);
	            OpenFile(szMediaFile, FALSE, TRUE, fSetCurrent && nStart == 0);
				nStart = nPos + 1;
			}
			//TRACE("TTTTTTTTTTTTT (%s)\n", (LPCTSTR)pMyCDS->lpData);
			if(fSetCurrent)
			{
				PlayCurrent();
				SetFocus();
				SetForegroundWindow();
			}
		}
	}

	if(!m_fScreenSaverMode && m_fFullscreen && message >= WM_MOUSEFIRST && message <= WM_MOUSELAST)
		return m_wndView.WindowProc(message, wParam, lParam);

	return CFrameWnd::WindowProc(message, wParam, lParam);
}

// The GraphBuilder interface provides a FindFilterByName() method,
// which provides similar functionality to the method below.
// This local method is provided for educational purposes.
BOOL CMainFrame::FindFilterFromName(LPCTSTR szNameToFind, IBaseFilter ** ppResultFilter)
{
    CComPtr<IEnumFilters> pEnum = NULL;
    CComPtr<IBaseFilter> pFilter = NULL;
    ULONG cFetched;
    BOOL bFound = FALSE;

    // Get filter enumerator
    if(FAILED(m_pGraphBuilder->EnumFilters(&pEnum)))
        return FALSE;

    // Enumerate all filters in the graph
    while(SUCCEEDED(pEnum->Next(1, &pFilter, &cFetched)) && !bFound)
    {
		if(pFilter != NULL)
		{
			FILTER_INFO FilterInfo;
			if(FAILED(pFilter->QueryFilterInfo(&FilterInfo)))
			{
				pFilter = NULL;
				pEnum = NULL;
				return FALSE;
			}

			// Compare this filter's name with the one we want
			TCHAR szName[256];
			WideCharToMultiByte(CP_ACP, 0, FilterInfo.achName, -1, szName, 256, 0, 0);
			if(!lstrcmp(szName, szNameToFind))
			{
				bFound = TRUE;
			}

			FilterInfo.pGraph->Release();

			if(bFound)
			{
				if(ppResultFilter != NULL)
				{
					*ppResultFilter = pFilter;
					(*ppResultFilter)->AddRef();
				}
				pFilter = NULL;
				break;
			}

			pFilter = NULL;
		}
    }
    pEnum = NULL;

    return bFound;
}

BOOL CMainFrame::SupportsPropertyPage(IBaseFilter *pFilter) 
{
	if(pFilter != NULL)
	{
		CComPtr<ISpecifyPropertyPages> pSpecify = NULL;

		// Discover if this filter contains a property page
		if(SUCCEEDED(pFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpecify))) 
		{
			pSpecify = NULL;
			return TRUE;
		}
	}
	return FALSE;
}

void CMainFrame::OnViewVolumeMoins() 
{
	int nVolume = m_wndControlDlgBar.m_wndSliderCtrlVolume.GetPos() - 250;
	int nMaxVolume = m_wndControlDlgBar.m_wndSliderCtrlVolume.GetRangeMax();
	if(nVolume > nMaxVolume)
		nVolume = nMaxVolume;
	m_wndControlDlgBar.m_wndSliderCtrlVolume.SetPos(nVolume);
	m_wndControlDlgBar.UpdateVolume();
}

void CMainFrame::OnViewVolumePlus() 
{
	int nVolume = m_wndControlDlgBar.m_wndSliderCtrlVolume.GetPos() + 250;
	int nMinVolume = m_wndControlDlgBar.m_wndSliderCtrlVolume.GetRangeMin();
	if(nVolume < nMinVolume)
		nVolume = nMinVolume;
	m_wndControlDlgBar.m_wndSliderCtrlVolume.SetPos(nVolume);
	m_wndControlDlgBar.UpdateVolume();
}

void CMainFrame::OnViewBalanceLeft() 
{
	int nBalance = m_wndControlDlgBar.m_wndSliderCtrlBalance.GetPos() - 500;
	int nMinBalance = m_wndControlDlgBar.m_wndSliderCtrlBalance.GetRangeMin();
	if(nBalance < nMinBalance)
		nBalance = nMinBalance;
	m_wndControlDlgBar.m_wndSliderCtrlBalance.SetPos(nBalance);
	m_wndControlDlgBar.UpdateBalance();
}

void CMainFrame::OnViewBalanceRight() 
{
	int nBalance = m_wndControlDlgBar.m_wndSliderCtrlBalance.GetPos() + 500;
	int nMaxBalance = m_wndControlDlgBar.m_wndSliderCtrlBalance.GetRangeMax();
	if(nBalance > nMaxBalance)
		nBalance = nMaxBalance;
	m_wndControlDlgBar.m_wndSliderCtrlBalance.SetPos(nBalance);
	m_wndControlDlgBar.UpdateBalance();
}

void CMainFrame::OnControlSpeedPlus() 
{
	int nSpeed = m_wndControlDlgBar.m_wndSliderCtrlSpeed.GetPos();
	if(nSpeed == 0)		nSpeed = 1;
	else if(nSpeed < 0)	nSpeed /= 2;
	else				nSpeed *= 2;

	int nMaxSpeed = m_wndControlDlgBar.m_wndSliderCtrlSpeed.GetRangeMax();

	if(nSpeed > nMaxSpeed)
		nSpeed = nMaxSpeed;
	m_wndControlDlgBar.m_wndSliderCtrlSpeed.SetPos(nSpeed);
	m_wndControlDlgBar.UpdateSpeed();
}

void CMainFrame::OnControlSpeedNormal() 
{
	m_wndControlDlgBar.m_wndSliderCtrlSpeed.SetPos(10);
	m_wndControlDlgBar.UpdateSpeed();
}

void CMainFrame::OnControlSpeedMoins() 
{

	int nSpeed = m_wndControlDlgBar.m_wndSliderCtrlSpeed.GetPos();
	if(nSpeed == 0)		nSpeed = -1;
	else if(nSpeed < 0)	nSpeed *= 2;
	else				nSpeed /= 2;

	int nMinSpeed = m_wndControlDlgBar.m_wndSliderCtrlSpeed.GetRangeMin();
	if(nSpeed < nMinSpeed)
		nSpeed = nMinSpeed;
	m_wndControlDlgBar.m_wndSliderCtrlSpeed.SetPos(nSpeed);
	m_wndControlDlgBar.UpdateSpeed();
}

void CMainFrame::OnControlDelayNo() 
{
	m_wndControlDlgBar.m_wndSliderCtrlDelay.SetPos(0);
	m_wndControlDlgBar.UpdateDelay();
}

void CMainFrame::OnControlDelayPlus() 
{
	int nDelay = m_wndControlDlgBar.m_wndSliderCtrlDelay.GetPos() + 1;
	int nMaxDelay = m_wndControlDlgBar.m_wndSliderCtrlDelay.GetRangeMax();
	if(nDelay > nMaxDelay)
		nDelay = nMaxDelay;
	m_wndControlDlgBar.m_wndSliderCtrlDelay.SetPos(nDelay);
	m_wndControlDlgBar.UpdateDelay();
}

void CMainFrame::OnControlDelayMoins() 
{
	int nDelay = m_wndControlDlgBar.m_wndSliderCtrlDelay.GetPos() - 1;
	int nMinDelay = m_wndControlDlgBar.m_wndSliderCtrlDelay.GetRangeMin();
	if(nDelay < nMinDelay)
		nDelay = nMinDelay;
	m_wndControlDlgBar.m_wndSliderCtrlDelay.SetPos(nDelay);
	m_wndControlDlgBar.UpdateDelay();
}

void CMainFrame::OnControlLanguage(UINT nID)
{
	if(m_pAMStreamSelect)
	{
		int nIndex = nID - IDM_CONTROL_LANGUAGE1;
		if(nIndex >= 0 && nIndex <= 20)
			HRESULT hhhr = m_pAMStreamSelect->Enable(nIndex, AMSTREAMSELECTINFO_ENABLED);
	}
}

void CMainFrame::OnUpdateControlLanguage(CCmdUI* pCmdUI) 
{
	if(m_pAMStreamSelect)
	{
		int nIndex = pCmdUI->m_nID - IDM_CONTROL_LANGUAGE1;
		if(nIndex >= 0 && nIndex <= 20)
		{
			DWORD dwFlags = NULL;
			if(SUCCEEDED(m_pAMStreamSelect->Info(nIndex, NULL, &dwFlags, NULL, NULL, NULL, NULL, NULL)))
				pCmdUI->SetRadio(dwFlags & AMSTREAMSELECTINFO_ENABLED);
		}
	}
}

void CMainFrame::OnControlProperty(UINT nID)
{
	CMenu * pMenu, * pMenuControl;
	if( !m_fFullscreen &&
		(pMenu = AfxGetMainWnd()->GetMenu()) &&
		(pMenuControl = pMenu->GetSubMenu(1)) &&
		(pMenu = pMenuControl->GetSubMenu(pMenuControl->GetMenuItemCount() - 1)))
	{
		CString szMenuText;
		pMenu->GetMenuString(nID, szMenuText, MF_BYCOMMAND);
		CComPtr<IBaseFilter> pFilter = NULL;
		if(FindFilterFromName(szMenuText, &pFilter))
		{
		    HRESULT hr;

			// Discover if this filter contains a property page
		    CComPtr<ISpecifyPropertyPages> pSpecify = NULL;
			if(SUCCEEDED(hr = pFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpecify)))
			{
				FILTER_INFO FilterInfo;
				if(SUCCEEDED(hr = pFilter->QueryFilterInfo(&FilterInfo)))
				{
					CAUUID caGUID;
					if(SUCCEEDED(hr = pSpecify->GetPages(&caGUID)))
					{
						// Display the filter's property page
						OleCreatePropertyFrame(m_hWnd, 0, 0,
							FilterInfo.achName,     // Caption for the dialog box
							1,                      // Number of filters
							(IUnknown **)&pFilter.p,// Pointer to the filter 
							caGUID.cElems,          // Number of property pages
							caGUID.pElems,          // Pointer to property page CLSIDs
							0, 0, NULL);
						CoTaskMemFree(caGUID.pElems);
					}
					FilterInfo.pGraph->Release(); // Bug: if dialog open and the graph is closed
				}
				pSpecify = NULL;
			}
			pFilter = NULL;
		}
	}
}

void CMainFrame::OnUpdateControlProperty(CCmdUI* pCmdUI) 
{
	CString szFilter;
	if(pCmdUI->m_pMenu && m_pGraphBuilder != NULL)
	{
		pCmdUI->m_pMenu->GetMenuString(pCmdUI->m_nID, szFilter, MF_BYCOMMAND);
		MENUITEMINFO menuItemInfo;
		menuItemInfo.cbSize = sizeof(MENUITEMINFO);
		menuItemInfo.fMask = MIIM_DATA;
		pCmdUI->m_pMenu->GetMenuItemInfo(pCmdUI->m_nID, &menuItemInfo);
		BOOL fEnable = menuItemInfo.dwItemData;
		pCmdUI->Enable(fEnable);
	}
}

void CMainFrame::OnPreferences() 
{
	CSettingsDialog dlg;
	CString szString;

	CString szStringGeneral;
	szStringGeneral.LoadString(IDS_STRING_SETTINGS_GENERAL);
	dlg.AddPage(RUNTIME_CLASS(CSettingsGeneralPropertyPage), szStringGeneral, IDD_DIALOG_SETTINGS_GENERAL);

	szString.LoadString(IDS_STRING_SETTINGS_SCREENSAVER);
	dlg.AddPage(RUNTIME_CLASS(CSettingsScreensaverPropertyPage), szString, IDD_DIALOG_SETTINGS_SCREENSAVER, szStringGeneral);

	CString szStringOSD;
	szStringOSD.LoadString(IDS_STRING_SETTINGS_OSD);
	dlg.AddPage(RUNTIME_CLASS(CSettingsOSDPropertyPage), szStringOSD, IDD_DIALOG_SETTINGS_OSD);

	szString.LoadString(IDS_STRING_SETTINGS_TIME_LINE);
	dlg.AddPage(RUNTIME_CLASS(CSettingsTimeLinePropertyPage), szString, IDD_DIALOG_SETTINGS_TIME_LINE, szStringOSD);

	szString.LoadString(IDS_STRING_SETTINGS_SUBTITLE);
	dlg.AddPage(RUNTIME_CLASS(CSettingsSubtitlePropertyPage), szString, IDD_DIALOG_SETTINGS_SUBTITLE, szStringOSD);

	szString.LoadString(IDS_STRING_SETTINGS_COMMAND);
	dlg.AddPage(RUNTIME_CLASS(CSettingsLircPropertyPage), szString, IDD_DIALOG_SETTINGS_LIRC);

//	dlg.AddPage(NULL, _T("Tree Item"), 0);

	szString.LoadString(IDS_STRING_SETTINGS);
	dlg.SetTitle(szString);
//	dlg.SetLogoText(_T("SettingsDialog"));    

	m_SettingsTemp.m_fGeneralDirty =
		m_SettingsTemp.m_fScreensaverDirty =
		m_SettingsTemp.m_fLircDirty =
		m_SettingsTemp.m_fOSDDirty =
		m_SettingsTemp.m_fSubtitleDirty =
		m_SettingsTemp.m_fTimeLineDirty = FALSE;

	UINT nResult = dlg.DoModal();
	if(nResult == IDAPPLY || nResult == IDOK)
		Preferences(FALSE);

	dlg.DestroyPages();

	if(m_wndControlDlgBar.m_wndSliderCtrlDelay.m_hWnd)
	{
		m_wndControlDlgBar.m_wndSliderCtrlDelay.EnableWindow(m_SettingsCurrent.m_fUseDelayFilter);
		m_wndControlDlgBar.m_wndStaticDelay.EnableWindow(m_SettingsCurrent.m_fUseDelayFilter);
	}

	if(m_SettingsCurrent.m_szGirderPluginFilename == _T(""))
	{
		m_RemoteControl.Stop();
		m_RemoteControl.CleanUp();
		m_fLircSupport = FALSE;
	}
	else
	{
		m_fLircSupport = TRUE;
		if(m_RemoteControl.Init(m_hWnd, m_SettingsCurrent.m_szGirderPluginFilename))
		{
			m_fLircInitialize = TRUE;
			if(!m_RemoteControl.Start())
			{
				CString szString;
				szString.LoadString(IDS_STRING_GIRDER_CANNOT_START);
				AfxMessageBoxEx(szString + m_SettingsCurrent.m_szGirderPluginFilename);
				m_fLircSupport = FALSE;
			}
		}
		else
		{
			CString szString;
			szString.LoadString(IDS_STRING_GIRDER_CANNOT_INITIALIZE);
			AfxMessageBoxEx(szString + m_SettingsCurrent.m_szGirderPluginFilename);
			m_fLircSupport = FALSE;
		}
	}
}

void CMainFrame::Preferences(BOOL bLoad)
{
	if(bLoad)
	{
		// Get Miscellaneous settings
		m_SettingsCurrent.m_fTopMost = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("TopMost"), FALSE);
		m_SettingsCurrent.m_fPlayRandom = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("PlayRandomly"), FALSE);
		m_SettingsCurrent.m_fSoundMute = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("Mute"), FALSE);
		m_SettingsCurrent.m_nSoundLevel = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("SoundLevel"), 0);
		m_SettingsCurrent.m_nSoundBalance = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("SoundBalance"), 0);
		m_SettingsCurrent.m_fUseOverlayWhenPossible = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("UseOverlayWhenPossible"), 1);
		m_SettingsCurrent.m_fShowOSD = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("ShowOSD"), 1);
		m_SettingsCurrent.m_nSortAttribute = AfxGetApp()->GetProfileInt(INI_SETTINGS_MISC, _T("SortAttribute"), CNetExplorerDoc::SortAscending);

		// Get General settings
		m_SettingsCurrent.m_fOnlyOneInstance = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("OnlyOneInstance"), TRUE);
		m_SettingsCurrent.m_fSlideshow = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("Slideshow"), TRUE);
		m_SettingsCurrent.m_nSlideshowWaitingTime = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("SlideshowWaitingTime"), 4000);
		m_SettingsCurrent.m_fUseDelayFilter = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("UseDelayFilter"), FALSE);

		// Get Screensaver settings
		m_SettingsCurrent.m_fScreensaverInstall = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverInstall"), FALSE);
		m_SettingsCurrent.m_szScreensaverPlaylistFile = AfxGetApp()->GetProfileString(INI_SETTINGS_GENERAL, _T("ScreensaverPlaylistFile"));
		m_SettingsCurrent.m_fScreensaverMute = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverMute"), FALSE);
		m_SettingsCurrent.m_fScreensaverRandom = AfxGetApp()->GetProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverRandom"), FALSE);

		// Get OSD settings
		if(!AfxGetProfileFont(INI_SETTINGS_OSD, _T("Font"), m_SettingsCurrent.m_fontOSD, GetDC()))
		{
			LOGFONT lf;
			CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT))->GetLogFont(&lf);
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			m_SettingsCurrent.m_fontOSD.CreateFontIndirect(&lf);
		}

		int nRed, nGreen, nBlue;
		CString szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("FontColor"), _T("#0000ff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorOSDFont = RGB(nRed, nGreen, nBlue);

		szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("FontBackgroundColor"), _T("#ffffff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorOSDBackgroundFont = RGB(nRed, nGreen, nBlue);

		m_SettingsCurrent.m_fOSDTransparent = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("FontTransparent"), TRUE);

		m_SettingsCurrent.m_nOSDPosX = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("OSDPosX"), 20);
		m_SettingsCurrent.m_nOSDPosY = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("OSDPosY"), 20);

		// Get Subtitle settings
		if(!AfxGetProfileFont(INI_SETTINGS_OSD, _T("SubtitleFont"), m_SettingsCurrent.m_fontSubtitle, GetDC()))
		{
			LOGFONT lf;
			CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT))->GetLogFont(&lf);
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			if(m_SettingsCurrent.m_fontSubtitle.m_hObject)
				m_SettingsCurrent.m_fontSubtitle.DeleteObject();
			m_SettingsCurrent.m_fontSubtitle.CreateFontIndirect(&lf);
		}

		szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("SubtitleFontColor"), _T("#ffffff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorSubtitleFont = RGB(nRed, nGreen, nBlue);

		szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("SubtitleFontBackgroundColor"), _T("#ffffff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorSubtitleBackgroundFont = RGB(nRed, nGreen, nBlue);

		m_SettingsCurrent.m_fSubtitleTransparent = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("SubtitleFontTransparent"), TRUE);

		m_SettingsCurrent.m_nSubtitlePosY = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("SubtitlePosY"), 60);

		// Get TimeLine settings
		if(!AfxGetProfileFont(INI_SETTINGS_OSD, _T("TimeLineFont"), m_SettingsCurrent.m_fontTimeLine, GetDC()))
		{
			LOGFONT lf;
			CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT))->GetLogFont(&lf);
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			if(m_SettingsCurrent.m_fontTimeLine.m_hObject)
				m_SettingsCurrent.m_fontTimeLine.DeleteObject();
			m_SettingsCurrent.m_fontTimeLine.CreateFontIndirect(&lf);
		}

		szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("TimeLineFontColor"), _T("#ffffff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorTimeLineFont = RGB(nRed, nGreen, nBlue);

		szFontColor = AfxGetApp()->GetProfileString(INI_SETTINGS_OSD, _T("TimeLineFontBackgroundColor"), _T("#ffffff"));
		if(sscanf(szFontColor, _T("#%02x%02x%02x"), &nRed, &nGreen, &nBlue) == 3)
			m_SettingsCurrent.m_colorTimeLineBackgroundFont = RGB(nRed, nGreen, nBlue);

		m_SettingsCurrent.m_fTimeLineTransparent = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("TimeLineFontTransparent"), TRUE);

		m_SettingsCurrent.m_nTimeLinePosY = AfxGetApp()->GetProfileInt(INI_SETTINGS_OSD, _T("TimeLinePosY"), 60);

		// Get Lirc settings
		ResetLircCommand(m_SettingsCurrent.m_aLircCommandParameters);

		int nNumKey = AfxGetApp()->GetProfileInt(INI_SETTINGS_LIRC, _T("NumKey"), 0);
		if(nNumKey)
		{
			for(int i = 0; i < m_SettingsCurrent.m_aLircCommandParameters.GetSize(); i++)
			{
				CString szKeyName, szKeyValue;
				TCHAR pszRemoteCommand[128];
				szKeyName.Format(_T("Key_%02d"), m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
				szKeyValue.Format(_T("%s %d %08x"), m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName, m_SettingsCurrent.m_aLircCommandParameters[i].fAcceptRepeat, m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
				szKeyValue = AfxGetApp()->GetProfileString(INI_SETTINGS_LIRC, szKeyName, szKeyValue);
				sscanf(szKeyValue, _T("%s %d %08x"), pszRemoteCommand, &m_SettingsCurrent.m_aLircCommandParameters[i].fAcceptRepeat, &m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
				m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName = pszRemoteCommand;
			}
		}

		m_SettingsCurrent.m_szGirderPluginFilename = AfxGetApp()->GetProfileString(INI_SETTINGS_LIRC, _T("GirderPluginFilename"));

		for(int i = 0; i < m_SettingsCurrent.m_aLircCommandParameters.GetSize(); i++)
			m_mapLircCommandToRemoteCommandParameterIndex[m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName] = i;
	}
	else
	{
		// Write General settings
		if(m_SettingsTemp.m_fGeneralDirty)
		{
			m_SettingsCurrent.m_fOnlyOneInstance = m_SettingsTemp.m_fOnlyOneInstance;
			m_SettingsCurrent.m_fSlideshow = m_SettingsTemp.m_fSlideshow;
			m_SettingsCurrent.m_nSlideshowWaitingTime = m_SettingsTemp.m_nSlideshowWaitingTime;
			m_SettingsCurrent.m_fUseDelayFilter = m_SettingsTemp.m_fUseDelayFilter;

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("OnlyOneInstance"), m_SettingsCurrent.m_fOnlyOneInstance);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("Slideshow"), m_SettingsCurrent.m_fSlideshow);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("SlideshowWaitingTime"), m_SettingsCurrent.m_nSlideshowWaitingTime);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("UseDelayFilter"), m_SettingsCurrent.m_fUseDelayFilter);
		}

		// Write Screensaver settings
		if(m_SettingsTemp.m_fScreensaverDirty)
		{
			m_SettingsCurrent.m_fScreensaverInstall = m_SettingsTemp.m_fScreensaverInstall;
			m_SettingsCurrent.m_szScreensaverPlaylistFile = m_SettingsTemp.m_szScreensaverPlaylistFile;
			m_SettingsCurrent.m_fScreensaverMute = m_SettingsTemp.m_fScreensaverMute;
			m_SettingsCurrent.m_fScreensaverRandom = m_SettingsTemp.m_fScreensaverRandom;

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverInstall"), m_SettingsCurrent.m_fScreensaverInstall);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_GENERAL, _T("ScreensaverPlaylistFile"), m_SettingsCurrent.m_szScreensaverPlaylistFile);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverMute"), m_SettingsCurrent.m_fScreensaverMute);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_GENERAL, _T("ScreensaverRandom"), m_SettingsCurrent.m_fScreensaverRandom);
		}

		// Write OSD settings
		if(m_SettingsTemp.m_fOSDDirty)
		{
			LOGFONT lf;
			m_SettingsTemp.m_fontOSD.GetLogFont(&lf);
			m_SettingsCurrent.m_fontOSD.DeleteObject();
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			m_SettingsCurrent.m_fontOSD.CreateFontIndirect(&lf);
			m_SettingsCurrent.m_colorOSDFont = m_SettingsTemp.m_colorOSDFont;
			m_SettingsCurrent.m_colorOSDBackgroundFont = m_SettingsTemp.m_colorOSDBackgroundFont;
			m_SettingsCurrent.m_fOSDTransparent = m_SettingsTemp.m_fOSDTransparent;
			m_SettingsCurrent.m_nOSDPosX = m_SettingsTemp.m_nOSDPosX;
			m_SettingsCurrent.m_nOSDPosY = m_SettingsTemp.m_nOSDPosY;

			AfxWriteProfileFont(INI_SETTINGS_OSD, _T("Font"), m_SettingsCurrent.m_fontOSD, GetDC());
			
			CString szFontColor;
			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorOSDFont & 0xff, (m_SettingsCurrent.m_colorOSDFont >> 8) & 0xff, (m_SettingsCurrent.m_colorOSDFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("FontColor"), szFontColor);

			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorOSDBackgroundFont & 0xff, (m_SettingsCurrent.m_colorOSDBackgroundFont >> 8) & 0xff, (m_SettingsCurrent.m_colorOSDBackgroundFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("FontBackgroundColor"), szFontColor);

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("FontTransparent"), m_SettingsCurrent.m_fOSDTransparent);

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("OSDPosX"), m_SettingsCurrent.m_nOSDPosX);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("OSDPosY"), m_SettingsCurrent.m_nOSDPosY);
		}

		// Write Subtitle settings
		if(m_SettingsTemp.m_fSubtitleDirty)
		{
			LOGFONT lf;
			m_SettingsTemp.m_fontSubtitle.GetLogFont(&lf);
			m_SettingsCurrent.m_fontSubtitle.DeleteObject();
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			m_SettingsCurrent.m_fontSubtitle.CreateFontIndirect(&lf);
			m_SettingsCurrent.m_colorSubtitleFont = m_SettingsTemp.m_colorSubtitleFont;
			m_SettingsCurrent.m_colorSubtitleBackgroundFont = m_SettingsTemp.m_colorSubtitleBackgroundFont;
			m_SettingsCurrent.m_fSubtitleTransparent = m_SettingsTemp.m_fSubtitleTransparent;
			m_SettingsCurrent.m_nSubtitlePosY = m_SettingsTemp.m_nSubtitlePosY;

			AfxWriteProfileFont(INI_SETTINGS_OSD, _T("SubtitleFont"), m_SettingsCurrent.m_fontSubtitle, GetDC());

			CString szFontColor;
			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorSubtitleFont & 0xff, (m_SettingsCurrent.m_colorSubtitleFont >> 8) & 0xff, (m_SettingsCurrent.m_colorSubtitleFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("SubtitleFontColor"), szFontColor);

			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorSubtitleBackgroundFont & 0xff, (m_SettingsCurrent.m_colorSubtitleBackgroundFont >> 8) & 0xff, (m_SettingsCurrent.m_colorSubtitleBackgroundFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("SubtitleFontBackgroundColor"), szFontColor);

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("SubtitleFontTransparent"), m_SettingsCurrent.m_fSubtitleTransparent);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("SubtitlePosY"), m_SettingsCurrent.m_nSubtitlePosY);
		}

		// Write TimeLine settings
		if(m_SettingsTemp.m_fTimeLineDirty)
		{
			LOGFONT lf;
			m_SettingsTemp.m_fontTimeLine.GetLogFont(&lf);
			m_SettingsCurrent.m_fontTimeLine.DeleteObject();
			lf.lfQuality |= ANTIALIASED_QUALITY; // | CLEARTYPE_QUALITY;
			m_SettingsCurrent.m_fontTimeLine.CreateFontIndirect(&lf);
			m_SettingsCurrent.m_colorTimeLineFont = m_SettingsTemp.m_colorTimeLineFont;
			m_SettingsCurrent.m_colorTimeLineBackgroundFont = m_SettingsTemp.m_colorTimeLineBackgroundFont;
			m_SettingsCurrent.m_fTimeLineTransparent = m_SettingsTemp.m_fTimeLineTransparent;
			m_SettingsCurrent.m_nTimeLinePosY = m_SettingsTemp.m_nTimeLinePosY;

			AfxWriteProfileFont(INI_SETTINGS_OSD, _T("TimeLineFont"), m_SettingsCurrent.m_fontTimeLine, GetDC());

			CString szFontColor;
			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorTimeLineFont & 0xff, (m_SettingsCurrent.m_colorTimeLineFont >> 8) & 0xff, (m_SettingsCurrent.m_colorTimeLineFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("TimeLineFontColor"), szFontColor);

			szFontColor.Format(_T("#%02x%02x%02x"), m_SettingsCurrent.m_colorTimeLineBackgroundFont & 0xff, (m_SettingsCurrent.m_colorTimeLineBackgroundFont >> 8) & 0xff, (m_SettingsCurrent.m_colorTimeLineBackgroundFont >> 16) & 0xff);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_OSD, _T("TimeLineFontBackgroundColor"), szFontColor);

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("TimeLineFontTransparent"), m_SettingsCurrent.m_fTimeLineTransparent);
			AfxGetApp()->WriteProfileInt(INI_SETTINGS_OSD, _T("TimeLinePosY"), m_SettingsCurrent.m_nTimeLinePosY);
		}

		// Write LIRC settings
		if(m_SettingsTemp.m_fLircDirty)
		{
			if(m_SettingsTemp.m_szGirderPluginFilename == _T("") && m_SettingsCurrent.m_szGirderPluginFilename != _T(""))
				m_SettingsCurrent.m_szGirderPluginFilename = m_SettingsTemp.m_szGirderPluginFilename;
			else if(m_SettingsTemp.m_szGirderPluginFilename != _T(""))
				m_SettingsCurrent.m_szGirderPluginFilename = m_SettingsTemp.m_szGirderPluginFilename;

			AfxGetApp()->WriteProfileString(INI_SETTINGS_LIRC, _T("GirderPluginFilename"), m_SettingsCurrent.m_szGirderPluginFilename);

			AfxGetApp()->WriteProfileInt(INI_SETTINGS_LIRC, _T("NumKey"), m_SettingsCurrent.m_aLircCommandParameters.GetSize());
			for(int i = 0; i < m_SettingsCurrent.m_aLircCommandParameters.GetSize(); i++)
			{
				m_SettingsCurrent.m_aLircCommandParameters[i] = m_SettingsTemp.m_aLircCommandParameters[i];
				m_mapLircCommandToRemoteCommandParameterIndex[m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName] = i;

				CString szKeyName, szKeyValue;
				szKeyName.Format(_T("Key_%02d"), m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
				szKeyValue.Format(_T("%s %d %08x"), m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName, m_SettingsCurrent.m_aLircCommandParameters[i].fAcceptRepeat, m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
				AfxGetApp()->WriteProfileString(INI_SETTINGS_LIRC, szKeyName, szKeyValue);
			}
		}
	}
}


void CMainFrame::ResetLircCommand(CArray<RemoteCommandParameter, RemoteCommandParameter&> & aLircCommandParameters)
{
	aLircCommandParameters.RemoveAll();

	RemoteCommandParameter rcp;
	int nCount = sizeof(g_arcp) / sizeof(RemoteCommandParameter2);
	for(int i = 0; i < nCount; i++)
	{
		rcp.szRemoteCommandName = g_arcp[i].pszRemoteCommandName;
		rcp.fAcceptRepeat = g_arcp[i].fAcceptRepeat;
		rcp.nWndCommand = g_arcp[i].nWndCommand;
		aLircCommandParameters.Add(rcp);
	}
}

LRESULT CMainFrame::OnCloseSettingDialog(WPARAM wParam, LPARAM lParam)
{
	// message received in response to modeless dialog OK
	//  and Cancel buttons
	if (!m_pSettingsDlg) return(-1);
	UINT nID = (UINT) wParam;

	if(nID == IDOK || nID == IDAPPLY)
	{
		AfxGetApp()->WriteProfileInt(INI_SETTINGS_LIRC, _T("NumKey"), m_SettingsCurrent.m_aLircCommandParameters.GetSize());
		for(int i = 0; i < m_SettingsCurrent.m_aLircCommandParameters.GetSize(); i++)
		{
			m_SettingsCurrent.m_aLircCommandParameters[i] = m_SettingsCurrent.m_aLircCommandParameters[i];
			m_mapLircCommandToRemoteCommandParameterIndex[m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName] = i;

			CString szKeyName, szKeyValue;
			szKeyName.Format(_T("Key_%02d"), m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
			szKeyValue.Format(_T("%s %d %08x"), m_SettingsCurrent.m_aLircCommandParameters[i].szRemoteCommandName, m_SettingsCurrent.m_aLircCommandParameters[i].fAcceptRepeat, m_SettingsCurrent.m_aLircCommandParameters[i].nWndCommand);
			AfxGetApp()->WriteProfileString(INI_SETTINGS_LIRC, szKeyName, szKeyValue);
		}
	}

	if ((nID == IDOK) || (nID == IDCANCEL))
	{
		m_pSettingsDlg->DestroyWindow();
		delete (m_pSettingsDlg);
		m_pSettingsDlg = NULL;
	}
	return 0L;
}

BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	BOOL fResult = CFrameWnd::OnCommand(wParam, lParam);

	CString szText;
	if(m_fShowDefaultOSD && LOWORD(wParam) != IDM_CONTROL_OSD_TEST && szText.LoadString(LOWORD(wParam)))
	{
		int nPos = szText.ReverseFind(_T('\n'));
		if(nPos != -1)
			szText = szText.Left(nPos);
		m_wndView.ShowText(szText);
	}

	m_fShowDefaultOSD = TRUE;

	return fResult;
}


void CMainFrame::OnFilePlaylistAdd() 
{
	if(OpenFile(NULL, TRUE, TRUE))
		PlayCurrent();
}

void CMainFrame::OnFilePlaylistLoad() 
{
	OpenFile(NULL, TRUE, FALSE);
}

void CMainFrame::OnFilePlaylistSave() 
{
	CString szFilter;
	szFilter.LoadString(IDS_STRING_OPENPLAYLIST_FILTER);
	CFileDialog FileDialog(FALSE, _T("*.txt"), NULL,
		OFN_OVERWRITEPROMPT | OFN_EXTENSIONDIFFERENT | OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING,
		szFilter, this);
	
	if(FileDialog.DoModal() == IDOK)
	{
		CWaitCursor wc;
		CStdioFile file;
		if(file.Open(FileDialog.GetPathName(), CFile::modeCreate
			| CFile::modeWrite | CFile::typeText ))
		{
			
			int nCount = m_wndPlayListDlgBar.GetCount();
			for(int nItem = 0; nItem < nCount; nItem++)
				file.WriteString(CString(m_wndPlayListDlgBar.GetFile(nItem) + CString(_T("\n"))));
			file.Close();
			AddFileToRecentFileList(FileDialog.GetPathName());
		}
	}
}

void CMainFrame::OnUpdateFilePlaylistAdd(CCmdUI* pCmdUI) 
{
}

void CMainFrame::OnUpdateFilePlaylistLoad(CCmdUI* pCmdUI) 
{
}

void CMainFrame::OnUpdateFilePlaylistSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_wndPlayListDlgBar.GetCount() != 0);
}

void CMainFrame::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	if(!m_fDesktopMode)
	{
		CFrameWnd::OnActivate(nState, pWndOther, bMinimized);
		
		if(m_fScreenSaverMode && !m_fScreenSaverNoDisturb && nState == WA_INACTIVE)
			DestroyWindow();
	}
}

void CMainFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);

	if(m_fScreenSaverMode)
		DestroyWindow();
}

void CMainFrame::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	CFrameWnd::OnLButtonDblClk(nFlags, point);
}

void CMainFrame::OnLButtonUp(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	CFrameWnd::OnLButtonUp(nFlags, point);
}

void CMainFrame::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CFrameWnd::OnLButtonDown(nFlags, point);

	if(m_fScreenSaverMode)
		DestroyWindow();
	else if(m_fFullscreen)
		m_wndView.OnLButtonDown(nFlags, point);
}

void CMainFrame::OnMButtonDown(UINT nFlags, CPoint point) 
{
	CFrameWnd::OnMButtonDown(nFlags, point);

	if(m_fScreenSaverMode)
		DestroyWindow();
}

void CMainFrame::OnMouseMove(UINT nFlags, CPoint point) 
{
	CFrameWnd::OnMouseMove(nFlags, point);

	if(m_fScreenSaverMode)
	{
		static CPoint ptLast = CPoint(-1, -1);
		if (ptLast == CPoint(-1,-1))
			ptLast = point;
		else if (ptLast != point)
			DestroyWindow();
	}
	else if(m_fFullscreen)
		m_wndView.OnMouseMove(nFlags, point);
}

BOOL CMainFrame::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	return m_wndView.OnMouseWheel(nFlags, zDelta, pt);
	//return CFrameWnd::OnMouseWheel(nFlags, zDelta, pt);
}

BOOL CMainFrame::OnNcActivate(BOOL bActive) 
{
	if(m_fScreenSaverMode && !bActive)
		return FALSE;
	
	return CFrameWnd::OnNcActivate(bActive);
}

void CMainFrame::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CFrameWnd::OnRButtonDown(nFlags, point);

	if(m_fScreenSaverMode)
		DestroyWindow();
	else
	{
		CPoint pt = point;
		ClientToScreen(&pt);
		ShowMenu(pt);
	}
}

BOOL CMainFrame::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if(m_fScreenSaverMode)
	{
		SetCursor(NULL);
		return TRUE;
	}

	return CFrameWnd::OnSetCursor(pWnd, nHitTest, message);
}

void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam) 
{
	if(m_fScreenSaverMode && ((nID == SC_SCREENSAVE) || (nID == SC_CLOSE)))
		return;
	
	CFrameWnd::OnSysCommand(nID, lParam);
}

void CMainFrame::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(m_fScreenSaverMode)
		DestroyWindow();

	CFrameWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);
}

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg) 
{
	if(m_fScreenSaverMode && pMsg &&
			pMsg->message >= WM_KEYFIRST && pMsg->message < WM_KEYLAST)
		return DestroyWindow();
	else
	{
		//if(m_fFullscreen)
		//	return m_wndView.PreTranslateMessage(pMsg);
		//else
		return CFrameWnd::PreTranslateMessage(pMsg);
	}
}

void CMainFrame::OnMruFileClean() 
{
	if(m_pRecentFileList)
	{
		int nCount = m_pRecentFileList->GetSize();
		for(int i = 0; i < nCount; i++)
			m_pRecentFileList->Remove(0);
	}
	if(m_pRecentDirectoryList)
	{
		int nCount = m_pRecentDirectoryList->GetSize();
		for(int i = 0; i < nCount; i++)
			m_pRecentDirectoryList->Remove(0);
	}
}

void CMainFrame::OnUpdateMruFileClean(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((m_pRecentFileList && (*m_pRecentFileList)[0].GetLength()) ||
		(m_pRecentDirectoryList && (*m_pRecentDirectoryList)[0].GetLength()));
}

void CMainFrame::OnViewTopmost() 
{
	m_SettingsCurrent.m_fTopMost ^= TRUE;

	if(m_SettingsCurrent.m_fTopMost)
		SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
	else
		SetWindowPos(&wndNoTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
}

void CMainFrame::OnUpdateViewTopmost(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_SettingsCurrent.m_fTopMost);
}

void CMainFrame::OnControlPlayRandom() 
{
	m_SettingsCurrent.m_fPlayRandom ^= TRUE;
}

void CMainFrame::OnUpdateControlPlayRandom(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_SettingsCurrent.m_fPlayRandom);
}

void CMainFrame::OnViewFullscreenOff() 
{
	if(m_fFullscreen)
		OnViewFullscreen();
}

void CMainFrame::OnControlOsdTest() 
{
//	CString szText = _T("Test");
//	m_wndView.ShowText(szText, 5000);
}

void CMainFrame::OnViewTimeline() 
{
	m_fViewTimeLine	^= TRUE;
	m_wndView.ShowTimeLine(m_fViewTimeLine);
}

void CMainFrame::OnUpdateViewUseOverlay(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_SettingsCurrent.m_fUseOverlayWhenPossible);	
}

void CMainFrame::OnViewUseOverlay() 
{
	m_SettingsCurrent.m_fUseOverlayWhenPossible ^= TRUE;
}

BOOL CMainFrame::PlayFile(LPCTSTR pszFilename, BOOL fInPlayList)
{
	m_fPlayListMode = fInPlayList;
	if(fInPlayList)
		m_wndPlayListDlgBar.SetCurrentFile(pszFilename);

	LoadSubtitleFromMovieFileName(pszFilename);

	return SUCCEEDED(PlayMovieInWindow(pszFilename));
}

BOOL CMainFrame::PlayCurrent()
{
	return PlayFile(m_wndPlayListDlgBar.GetCurrentFile(), TRUE);
}

BOOL CMainFrame::PlayNextFile()
{
	if(m_fPlayListMode)
	{
		LPCTSTR pszFilename = m_wndPlayListDlgBar.GetNextFile();
		if(!pszFilename)
			pszFilename = m_wndPlayListDlgBar.GetFile(0);

		if(pszFilename)
			return PlayFile(pszFilename, m_fPlayListMode);
	}
	else
	{
		while(m_wndFileBrowserDlgBar.SelectNextEntry())
		{
			CTreeNode * pTreeNode = m_wndFileBrowserDlgBar.GetCurrentSelection();
			if(pTreeNode != NULL && pTreeNode->szPath != "")
				if(PlayFile(pTreeNode->szPath, m_fPlayListMode))
					break;
		}
	}

	return FALSE;
}

void CMainFrame::OnViewOsd()
{
	m_SettingsCurrent.m_fShowOSD ^= TRUE;
	m_wndView.Invalidate();
}

void CMainFrame::OnUpdateViewOsd(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_SettingsCurrent.m_fShowOSD);
	pCmdUI->Enable(m_fOSDPossible);
}

void CMainFrame::OnUpdateViewTimeline(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_fViewTimeLine);
}

void CMainFrame::SetStatusTest(LPCTSTR pszText)
{
	if(m_wndStatusBar.m_hWnd)
		m_wndStatusBar.SetPaneText(0, pszText, TRUE);
}

void CMainFrame::OnControlLanguageToggle() 
{
	if(m_pAMStreamSelect)
	{
		DWORD nCount = 0;
		m_pAMStreamSelect->Count(&nCount);
		for(DWORD i = 0; i < nCount; i++)
		{
			DWORD dwFlags = NULL;
			if(SUCCEEDED(m_pAMStreamSelect->Info(i, NULL, &dwFlags, NULL, NULL, NULL, NULL, NULL)) && 
				(dwFlags & AMSTREAMSELECTINFO_ENABLED))
			{
					m_pAMStreamSelect->Enable((i + 1) % nCount, AMSTREAMSELECTINFO_ENABLED);
					break;
			}
		}
	}
}

void CMainFrame::OnUpdateControlLanguageToggle(CCmdUI* pCmdUI) 
{
	DWORD nCount = 0;
	if(m_pAMStreamSelect)
		m_pAMStreamSelect->Count(&nCount);

	pCmdUI->Enable(nCount != 0);
}

void CMainFrame::OnFilebrowserSynchroniseCurrentFile()
{
	m_wndFileBrowserDlgBar.m_LeftView.OnFilebrowserSynchroniseCurrentFile();
}

void CMainFrame::OnFilebrowserRefresh()
{
	m_wndFileBrowserDlgBar.m_LeftView.OnFilebrowserRefresh();
}

void CMainFrame::OnInitMenu(CMenu* pMenu)
{
	CFrameWnd::OnInitMenu(pMenu);

	CMenu * pMainMenu = GetMenu();
	if(pMainMenu) UpdateMenu(pMainMenu);
}
