//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__INCLUDED_)
#define AFX_MAINFRM_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <dshow.h>
//#include <comdef.h>

#include "ChildView.h"
#include "SeekingDialogBar.h"
#include "ControlDialogBar.h"
#include "PlayListDialogBar.h"
#include "FileBrowserDialogBar.h"
#include "SubtitleDialogBar.h"
#include "RemoteControl.h"
#include "SettingsData.h"
#include "MediaTime.h"

//
// Macros
//
#define SAFE_RELEASE(x) { if (x) x->Release(); x = NULL; }

#define JIF(x) if (FAILED(hr=(x))) \
    {TRACE(_T("FAILED(hr=0x%x) in ") _T(#x) _T("\n"), hr); goto Error;}

#define LIF(x) if (FAILED(hr=(x))) \
    {TRACE(_T("FAILED(hr=0x%x) in ") _T(#x) _T("\n"), hr);}

#define WM_TRAY_ICON_NOTIFY_MESSAGE (WM_USER + 1)
#define WM_GRAPHNOTIFY  (WM_USER + 13)

#define CLASSNAME       TEXT("PlayWndMediaPlayer")

typedef enum EnumPlayState { PSUnload, PSRunning, PSPaused, PSStopped, PSFastForward, PSFastRewind } EnumPlayState;

#include "SubtitleManager.h"

class CSettingsDialog;
class CRecentFileListEx;
class CDelayTransformFilter;

class CMainFrame : public CFrameWnd
{
	friend class CMPlayerXApp;
	friend class CChildView;
	friend class CControlDialogBar;
	friend class CSeekingDialogBar;
	friend class CSubtitleDialogBar;
	friend class CSettingsLircPropertyPage;
	friend class CPlayListDialogBar;
public:
	CMainFrame();
	BOOL GoScreenSaver();

protected: 
	DECLARE_DYNAMIC(CMainFrame)

// Attributes
public:

	long		m_nDelay;
	int			m_nRepeatABMode;
	LONGLONG	m_llRepeatAPosition;
	LONGLONG	m_llRepeatBPosition;
	BOOL		m_fOverlayUsed;
	BOOL		m_fIsAPicture;
	BOOL		m_fScreenSaverMode;
	BOOL		m_fScreenSaverNoDisturb;
	BOOL		m_fLircInitialize;
	BOOL		m_fLircSupport;
	BOOL		m_fFullscreen;
	BOOL		m_fDesktopMode;
	COLORREF	m_colorOldValues;
	BOOL		m_fColorKeyDirty;
	DWORD		m_dwColorKey;
	COLORREF	m_dwColorKeyRGB;
	DWORD		m_dwOldDesktopColor;
	BOOL		m_fMaintainAspectRatio;
	BOOL		m_fWindowFit;
	BOOL		m_fWindowWidthFit;
	BOOL		m_fRatio169;
	double		m_rCurrentAspectRatio;
	BOOL		m_fViewCenter;
	BOOL		m_fViewTimeLine;
	BOOL		m_fMultipleMonitorSupported;
	BOOL		m_fGraphEditSpy;
	BOOL		m_fRecentDirectoryListForFileBrowserDlgBar;
	BOOL		m_fUseVMR;
	BOOL		m_fPlayListMode;

	double		m_rZoom;
	double		m_rZoomStep;
	double		m_rAspectRatioStep;
	int			m_nOffsetCenterX;
	int			m_nOffsetCenterY;
	int			m_nOffsetStepSlow;
	int			m_nOffsetStepNormal;
	int			m_nOffsetStepQuick;

	CString		m_szFileName;
	BOOL		m_bAudioOnly;
	LONG		m_lVolume;
	EnumPlayState	m_enCurrentPlayState;
	int			m_nFastForwardSpeed;

	BOOL		m_fShowDefaultOSD;
	BOOL		m_fOSDPossible;

	CSettingsData m_SettingsCurrent;
	CSettingsData m_SettingsTemp;
	HANDLE m_hMutex;

// Operations
public:
	BOOL PlayCurrent();
	BOOL PlayFile(LPCTSTR pszFilename, BOOL fInPlayList = TRUE);
	BOOL PlayNextFile();
	HRESULT PlayMovieInWindow(LPCTSTR szFile);
	BOOL OpenFile(LPCTSTR pszPath, BOOL fWithGUI = TRUE, BOOL fAddFile = FALSE, BOOL fSetCurrent = FALSE);
	void AddFileToRecentFileList(LPCTSTR pszFilename);
	void CloseCurrentClip();
	void UpdateMainTitle();
	void UpdateDestinationView(int nWidth, int nHeight);
	void UpdateDestinationView();
	void MediaEvent();
	void ResetLircCommand(CArray<RemoteCommandParameter, RemoteCommandParameter&> & m_aLircCommandParameters);
	void UpdateTimeString(BOOL fSliderBarPosUpdate = TRUE);
	COLORREF GetColorKey();
	void Preferences(BOOL bLoad);
	void CleanGraphBuilder();
	void SetTimers();
	void SetStatusTest(LPCTSTR pszText);
	void ShowError(LPCTSTR pszMessage);
	void UpdateMenu(CMenu * pMenu);
	void ShowMenu(CPoint & point);
	BOOL LoadSubtitleFromMovieFileName(LPCTSTR pszFilename);
	void UnLoadSubtitle();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
	virtual BOOL DestroyWindow();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMainFrame();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

//protected:
	BOOL VerifyBarState(LPCTSTR lpszProfileName);
	BOOL SupportsPropertyPage(IBaseFilter *pFilter);
//	IBaseFilter * FindFilterFromName(LPCTSTR szNameToFind);
	BOOL FindFilterFromName(LPCTSTR szNameToFind, IBaseFilter ** ppResultFilter);
	BOOL InsertFilter(IBaseFilter * pFilterToInsert, BOOL fBefore, IBaseFilter * pBaseFilter);
//	IPin * FindPin(IBaseFilter * pBaseFilter, BOOL fInput = TRUE);


	// control bar embedded members
	CStatusBar  m_wndStatusBar;
	CToolBar    m_wndToolBar;
	CSeekingDialogBar	m_wndSeekingDlgBar;
	CControlDialogBar	m_wndControlDlgBar;
	CPlayListDialogBar	m_wndPlayListDlgBar;
	CFileBrowserDialogBar m_wndFileBrowserDlgBar;
	CSubtitleDialogBar	m_wndSubtitleDlgBar;

	CSubtitleManager m_SubtitleManager;
	CRemoteControl m_RemoteControl;
	CMap<CString, LPCTSTR, UINT, UINT> m_mapLircCommandToRemoteCommandParameterIndex;

	CMediaTime m_MediaTime;

	CSettingsDialog * m_pSettingsDlg;

	CRecentFileListEx* m_pRecentFileList;
	CRecentFileListEx* m_pRecentDirectoryList;

	CChildView	m_wndView;
	CMenu m_Menu;
	HICON m_hIcon;
	NOTIFYICONDATA	m_nidIconData;

	UINT		m_hFrameSeekingTimer;
	UINT		m_hFastForwardSeekingTimer;
	UINT		m_nFastForwardSeekingTime;
	UINT		m_hSlideShowTimer;

	DWORD		m_dwGraphRegister;
	HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister);
	void RemoveGraphFromRot(DWORD pdwRegister);

	// DirectShow interfaces
	CComPtr<IGraphBuilder> m_pGraphBuilder;
	CComPtr<IMediaControl> m_pMediaControl;
	CComPtr<IMediaEventEx> m_pMediaEventEx;
	CComPtr<IVMRWindowlessControl> m_pVMRWindowlessControl;
	CComPtr<IVideoWindow> m_pVideoWindow;
	CComPtr<IBasicAudio> m_pBasicAudio;
	CComPtr<IBasicVideo> m_pBasicVideo;
	CComPtr<IMediaSeeking> m_pMediaSeeking;
	CComPtr<IAMStreamSelect> m_pAMStreamSelect;
	CDelayTransformFilter * m_pDelayTransformFilter;

	// Generated message map functions
protected:
	//{{AFX_MSG(CMainFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSetFocus(CWnd *pOldWnd);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnClose();
	afx_msg void OnFileOpenclip();
	afx_msg void OnFileCloseClip();
	afx_msg void OnControlPlayPause();
	afx_msg void OnControlStop();
	afx_msg void OnViewMute();
	afx_msg void OnViewSizeHalf();
	afx_msg void OnViewSizeThreequarter();
	afx_msg void OnViewSizeNormal();
	afx_msg void OnViewSizeDouble();
public:
	afx_msg void OnViewFullscreen();
protected:
	afx_msg void OnView169();
	afx_msg void OnViewMaintainAspectRatio();
	afx_msg void OnViewCenter();
	afx_msg void OnUpdateFileCloseClip(CCmdUI* pCmdUI);
	afx_msg void OnUpdateControlPlayPause(CCmdUI* pCmdUI);
	afx_msg void OnUpdateControlStop(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewMute(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewSizeHalf(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewSizeThreequarter(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewSizeNormal(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewSizeDouble(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewFullscreen(CCmdUI* pCmdUI);
	afx_msg void OnUpdateView169(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewMaintainAspectRatio(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewCenter(CCmdUI* pCmdUI);
	afx_msg void OnViewAspectRatioPlus();
	afx_msg void OnViewAspectRatioMoins();
	afx_msg void OnViewAspectRatioReset();
	afx_msg void OnViewScrollDown();
	afx_msg void OnViewScrollUp();
	afx_msg void OnViewScrollLeft();
	afx_msg void OnViewScrollRight();
	afx_msg void OnViewScrollReset();
	afx_msg void OnViewZoomPlus();
	afx_msg void OnViewZoomMoins();
	afx_msg void OnViewZoomReset();
	afx_msg void OnViewControlBar();
	afx_msg void OnUpdateViewControlBar(CCmdUI* pCmdUI);
	afx_msg void OnViewPlayListBar();
	afx_msg void OnUpdateViewPlayListBar(CCmdUI* pCmdUI);
	afx_msg void OnViewSeekingBar();
	afx_msg void OnUpdateViewSeekingBar(CCmdUI* pCmdUI);
	afx_msg void OnControlFastRewind();
	afx_msg void OnUpdateControlFastRewind(CCmdUI* pCmdUI);
	afx_msg void OnControlPreviousFile();
	afx_msg void OnUpdateControlPreviousFile(CCmdUI* pCmdUI);
	afx_msg void OnControlNextFile();
	afx_msg void OnUpdateControlNextFile(CCmdUI* pCmdUI);
	afx_msg void OnControlPreviousFolderFile();
	afx_msg void OnUpdateControlPreviousFolderFile(CCmdUI* pCmdUI);
	afx_msg void OnControlNextFolderFile();
	afx_msg void OnUpdateControlNextFolderFile(CCmdUI* pCmdUI);
	afx_msg void OnControlFastForward();
	afx_msg void OnUpdateControlFastForward(CCmdUI* pCmdUI);
	afx_msg void OnViewSizeWindowFit();
	afx_msg void OnUpdateViewSizeWindowFit(CCmdUI* pCmdUI);
	afx_msg void OnFileAddclip();
	afx_msg void OnUpdateFileAddclip(CCmdUI* pCmdUI);
	afx_msg void OnViewVolumeMoins();
	afx_msg void OnUpdateViewVolumeMoins(CCmdUI* pCmdUI);
	afx_msg void OnViewVolumePlus();
	afx_msg void OnUpdateViewVolumePlus(CCmdUI* pCmdUI);
	afx_msg void OnViewBalanceLeft();
	afx_msg void OnUpdateViewBalanceLeft(CCmdUI* pCmdUI);
	afx_msg void OnViewBalanceRight();
	afx_msg void OnUpdateViewBalanceRight(CCmdUI* pCmdUI);
	afx_msg void OnFileCloseAllClip();
	afx_msg void OnUpdateFileCloseAllClip(CCmdUI* pCmdUI);
	afx_msg void OnFileCloseSelectedClip();
	afx_msg void OnUpdateFileCloseSelectedClip(CCmdUI* pCmdUI);
	afx_msg void OnControlSpeedPlus();
	afx_msg void OnControlSpeedNormal();
	afx_msg void OnControlSpeedMoins();
	afx_msg void OnUpdateControlSpeedPlus(CCmdUI* pCmdUI);
	afx_msg void OnUpdateControlSpeedNormal(CCmdUI* pCmdUI);
	afx_msg void OnUpdateControlSpeedMoins(CCmdUI* pCmdUI);
	afx_msg void OnPreferences();
	afx_msg void OnViewDesktop();
	afx_msg void OnUpdateViewDesktop(CCmdUI* pCmdUI);
	afx_msg void OnFilePlaylistAdd();
	afx_msg void OnUpdateFilePlaylistAdd(CCmdUI* pCmdUI);
	afx_msg void OnFilePlaylistLoad();
	afx_msg void OnUpdateFilePlaylistLoad(CCmdUI* pCmdUI);
	afx_msg void OnFilePlaylistSave();
	afx_msg void OnUpdateFilePlaylistSave(CCmdUI* pCmdUI);
	afx_msg void OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg BOOL OnNcActivate(BOOL bActive);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnMruFileClean();
	afx_msg void OnUpdateMruFileClean(CCmdUI* pCmdUI);
	afx_msg void OnViewTopmost();
	afx_msg void OnUpdateViewTopmost(CCmdUI* pCmdUI);
	afx_msg void OnControlPlayRandom();
	afx_msg void OnUpdateControlPlayRandom(CCmdUI* pCmdUI);
	afx_msg void OnViewFullscreenOff();
	afx_msg void OnControlStepByStep();
	afx_msg void OnControlRepeatAb();
	afx_msg void OnUpdateControlStepByStep(CCmdUI* pCmdUI);
	afx_msg void OnUpdateControlRepeatAb(CCmdUI* pCmdUI);
	afx_msg void OnControlOsdTest();
	afx_msg void OnViewTimeline();
	afx_msg void OnUpdateViewUseOverlay(CCmdUI* pCmdUI);
	afx_msg void OnViewUseOverlay();
	afx_msg void OnFileAddSubtitle();
	afx_msg void OnUpdateFileAddSubtitle(CCmdUI* pCmdUI);
	afx_msg void OnViewSubtitleBar();
	afx_msg void OnUpdateViewSubtitleBar(CCmdUI* pCmdUI);
	afx_msg void OnControlDelayNo();
	afx_msg void OnControlDelayPlus();
	afx_msg void OnControlDelayMoins();
	afx_msg void OnControlLanguageToggle();
	afx_msg void OnUpdateControlLanguageToggle(CCmdUI* pCmdUI);
	//}}AFX_MSG
	afx_msg LRESULT OnCloseSettingDialog(WPARAM wParam, LPARAM lParam);
    afx_msg BOOL OnToolTipText(UINT, NMHDR* pNMHDR, LRESULT* pResult);
//	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg LRESULT OnTrayNotify(WPARAM wParam, LPARAM lParam);
	afx_msg void OnControlProperty(UINT nID);
	afx_msg void OnUpdateControlProperty(CCmdUI* pCmdUI);
	afx_msg void OnMRUFileHandler(UINT i);
	afx_msg void OnMRUDirectoryHandler(UINT i);
	afx_msg void OnUpdateMRUFileHandler(CCmdUI* pCmdUI);
	afx_msg void OnUpdateMRUDirectoryHandler(CCmdUI* pCmdUI);
	afx_msg void OnControlLanguage(UINT nID);
	afx_msg void OnUpdateControlLanguage(CCmdUI* pCmdUI);
	afx_msg void OnViewOsd();
	afx_msg void OnUpdateViewOsd(CCmdUI *pCmdUI);
	afx_msg void OnUpdateViewTimeline(CCmdUI *pCmdUI);
	afx_msg void OnDropFiles(HDROP hDropInfo);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnViewFileBrowserBar();
	afx_msg void OnUpdateViewFileBrowserBar(CCmdUI *pCmdUI);

	DECLARE_MESSAGE_MAP()
//public:
//	afx_msg void OnMove(int x, int y);
public:
	afx_msg void OnFilebrowserSynchroniseCurrentFile();
	afx_msg void OnFilebrowserRefresh();
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnInitMenu(CMenu* pMenu);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__INCLUDED_)
