//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// MediaTime.cpp: implementation of the CMediaTime class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MediaTime.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMediaTime::CMediaTime()
{
	m_enTimeFormat = TimeFormatNone;
	m_szDuration = _T("");
	m_llDuration = 0;
	m_timeAvgTimePerFrame = 1.0;
	m_fUseFalseTime = FALSE;
	m_dwFrameStep = 1;

	m_pMediaSeeking = NULL;
}

CMediaTime::~CMediaTime()
{
	m_pMediaSeeking = NULL;
}

BOOL CMediaTime::Init(IMediaSeeking * pMediaSeeking)
{
	m_pMediaSeeking = pMediaSeeking;

	if(m_pMediaSeeking == NULL)
		return FALSE;

	const GUID * pTimeFormat = NULL;
	if(SUCCEEDED(m_pMediaSeeking->QueryPreferredFormat((GUID *)pTimeFormat)))
	{
		TRACE("TimeFormat -> Preferred\n");

		if(pTimeFormat == &TIME_FORMAT_MEDIA_TIME)
			m_enTimeFormat = TimeFormatMediaTime;
		else if(pTimeFormat == &TIME_FORMAT_FRAME)
			m_enTimeFormat = TimeFormatFrame;
		else if(pTimeFormat == &TIME_FORMAT_SAMPLE)
			m_enTimeFormat = TimeFormatSample;
		else if(pTimeFormat == &TIME_FORMAT_FIELD)
			m_enTimeFormat = TimeFormatField;
		else if(pTimeFormat == &TIME_FORMAT_BYTE)
			m_enTimeFormat = TimeFormatByte;
		else if(pTimeFormat == &TIME_FORMAT_NONE)
			m_enTimeFormat = TimeFormatNone;
	}
	else
/*		if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_MEDIA_TIME) == S_OK)
	{
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
		m_enTimeFormat = TimeFormatMediaTime;
	}
	else */
	if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_FRAME) == S_OK)
	{
		TRACE("TimeFormat -> TIME_FORMAT_FRAME\n");
		pTimeFormat = &TIME_FORMAT_FRAME;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_FRAME);
		m_enTimeFormat = TimeFormatFrame;
	}
	else if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_SAMPLE) == S_OK)
	{
		TRACE("TimeFormat -> TIME_FORMAT_SAMPLE\n");
		pTimeFormat = &TIME_FORMAT_SAMPLE;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_SAMPLE);
		m_enTimeFormat = TimeFormatSample;
	}
	else if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_FIELD) == S_OK)
	{
		TRACE("TimeFormat -> TIME_FORMAT_FIELD\n");
		pTimeFormat = &TIME_FORMAT_FIELD;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_FIELD);
		m_enTimeFormat = TimeFormatField;
	}
	else if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_MEDIA_TIME) == S_OK)
	{
		TRACE("TimeFormat -> TIME_FORMAT_MEDIA_TIME\n");
		pTimeFormat = &TIME_FORMAT_MEDIA_TIME;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
		m_enTimeFormat = TimeFormatMediaTime;
	}
	else if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_BYTE) == S_OK)
	{
		TRACE("TimeFormat -> TIME_FORMAT_BYTE\n");
		pTimeFormat = &TIME_FORMAT_BYTE;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_BYTE);
		m_enTimeFormat = TimeFormatByte;
	}
	else
	{
		TRACE("TimeFormat -> TIME_FORMAT_NONE\n");
		pTimeFormat = &TIME_FORMAT_NONE;
		m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_NONE);
		m_enTimeFormat = TimeFormatNone;
	}

	m_szDuration = _T("");
	LONGLONG llDuration = 0;
	m_timeAvgTimePerFrame = 1.0;
	m_fUseFalseTime = FALSE;

	// Read the file's duration
	if(SUCCEEDED(m_pMediaSeeking->GetDuration(&llDuration)))
	{
		m_llDuration = llDuration;

		LONGLONG llCurrent = 0;
		m_pMediaSeeking->SetPositions(
			&llCurrent, AM_SEEKING_AbsolutePositioning,
			&llDuration, AM_SEEKING_AbsolutePositioning);

		if(m_enTimeFormat == TimeFormatMediaTime)
		{
			m_dwFrameStep = 10000;

			// Convert the LONGLONG position into human-readable format
			unsigned long nDurationTotalMS = (unsigned long)(llDuration / 10000); // 100ns -> ms
			int nDurationMS = nDurationTotalMS % 1000;
			int nDurationSeconds = nDurationTotalMS / 1000;
			int nDurationMinutes = nDurationSeconds / 60;
			nDurationSeconds %= 60;

			// Update the display
			m_szDuration.Format(_T("%02d:%02d.%03d"),
				nDurationMinutes, nDurationSeconds, nDurationMS);
		}
		else if(m_enTimeFormat == TimeFormatFrame ||
				m_enTimeFormat == TimeFormatSample ||
				m_enTimeFormat == TimeFormatField ||
				m_enTimeFormat == TimeFormatByte)
		{
			if(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_MEDIA_TIME) == S_OK)
			{
				LONGLONG llNewDuration = 0;
				if(SUCCEEDED(m_pMediaSeeking->ConvertTimeFormat(&llNewDuration, &TIME_FORMAT_MEDIA_TIME, llDuration, NULL)))
					m_timeAvgTimePerFrame = (double)llNewDuration / (double)llDuration;
//					else
//						m_pBasicVideo->get_AvgTimePerFrame(&m_timeAvgTimePerFrame);

				m_fUseFalseTime = TRUE;
				m_dwFrameStep = 1;

				// Convert the LONGLONG position into human-readable format
				unsigned long nDurationTotalMS = (unsigned long)((LONGLONG)(llDuration * m_timeAvgTimePerFrame) / 10000); // 100ns -> ms
				int nDurationMS = nDurationTotalMS % 1000;
				int nDurationSeconds = nDurationTotalMS / 1000;
				int nDurationMinutes = nDurationSeconds / 60;
				nDurationSeconds %= 60;

				// Update the display
				m_szDuration.Format(_T("%02d:%02d.%03d"),
					nDurationMinutes, nDurationSeconds, nDurationMS);
			}
			else
			{
				m_dwFrameStep = 1;
				m_szDuration.Format(_T("%I64d"), llDuration);
			}
		}
	}
	return TRUE;
}

void CMediaTime::Close()
{
	m_pMediaSeeking = NULL;
}

BOOL CMediaTime::GetCurrentPosition(REFERENCE_TIME & timeCurrent)
{
	LONGLONG llPosition = 0;
	
	if(SUCCEEDED(m_pMediaSeeking->GetCurrentPosition(&llPosition)))
	{
		if(m_enTimeFormat == TimeFormatMediaTime)
		{
			timeCurrent = llPosition;
			return TRUE;
		}
		else if(m_enTimeFormat == TimeFormatFrame ||
			m_enTimeFormat == TimeFormatSample ||
			m_enTimeFormat == TimeFormatField ||
			m_enTimeFormat == TimeFormatByte)
		{
			if(SUCCEEDED(m_pMediaSeeking->IsFormatSupported(&TIME_FORMAT_MEDIA_TIME)))
			{
				LONGLONG llNewPosition = 0;
				if(SUCCEEDED(m_pMediaSeeking->ConvertTimeFormat(&llNewPosition, &TIME_FORMAT_MEDIA_TIME, llPosition, NULL)))
				{
					timeCurrent = llNewPosition;
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

LONGLONG CMediaTime::GetTimeString(CString & szResult)
{
	if(m_pMediaSeeking != NULL)
	{
		LONGLONG llPosition = 0, llStopPosition = 0;
		HRESULT hr = m_pMediaSeeking->GetPositions(&llPosition, &llStopPosition);

		CString szTime(_T(""));
		switch(m_enTimeFormat)
		{
		case TimeFormatMediaTime:
			{
				// Convert the LONGLONG position into human-readable format
				unsigned long nTotalMS = (unsigned long)(llPosition / (LONGLONG)10000); // 100ns -> ms
				int nMS = nTotalMS % 1000;
				int nSeconds = nTotalMS / 1000;
				int nMinutes = nSeconds / 60;
				nSeconds %= 60;
				
				// Update the display
				szTime.Format(_T("%02d:%02d.%03d / %s"),
					nMinutes, nSeconds, nMS, m_szDuration);
			}
			break;
			
		case TimeFormatFrame:
		case TimeFormatSample:
		case TimeFormatField:
		case TimeFormatByte:
			if(m_fUseFalseTime)
			{
				// Convert the LONGLONG position into human-readable format
				unsigned long nTotalMS = (unsigned long)((LONGLONG)(llPosition * m_timeAvgTimePerFrame) / 10000); // 100ns -> ms
				int nMS = nTotalMS % 1000;
				int nSeconds = nTotalMS / 1000;
				int nMinutes = nSeconds / 60;
				nSeconds %= 60;
				
				// Update the display
				szTime.Format(_T("%02d:%02d.%03d / %s"),
					nMinutes, nSeconds, nMS, m_szDuration);
			}
			else
				szTime.Format(_T("%I64d / %s"), llPosition, m_szDuration);
			break;
		case TimeFormatNone:
			break;
		}
		
		if(szTime != _T(""))
		{
			szResult = szTime;
			return llPosition;
		}
	}
	return (LONGLONG)-1;
}