//    NetExplorer - Windows LAN explorer application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// NetExplorerDoc.cpp : implementation of the CNetExplorerDoc class
//

#include "stdafx.h"
#include "MPlayerX.h"
#include "NetExplorerDoc.h"
#include <string.h>
#include <lm.h>
#include "LeftView.h"
#include "QArray.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int anType2Images[] = {
	11,	// Desktop,
	10,	// WorkArea
	12,	// Drive
	1,	// NetNeighborhood
	2,	// Network
	3,	// NetProtocol
	6,	// NetDirectory
	14,	// NetShare
	4,	// Domain
	5,	// Server
	14,	// Share
	6,	// Directory
	7,	// SelectedDirectory
	9	// File
};

int CTreeNode::Type2ImageNumber(enType nType)
{
	return anType2Images[nType];
}

CTreeNode::CTreeNode(const CTreeNode & TreeNode)
{
	*this = TreeNode;
}

void CTreeNode::operator =(const CTreeNode & TreeNode)
{
	pParent = TreeNode.pParent;
	nType = TreeNode.nType;
	nFieldMask = TreeNode.nFieldMask;
	szName = TreeNode.szName;
	szPath = TreeNode.szPath;
	szComment = TreeNode.szComment;
	tCreationTime = TreeNode.tCreationTime;
	tLastAccessTime = TreeNode.tLastAccessTime;
	tLastWriteTime = TreeNode.tLastWriteTime;
	dwFileAttributes = TreeNode.dwFileAttributes;
	nFileSizeHigh = TreeNode.nFileSizeHigh;
	nFileSizeLow = TreeNode.nFileSizeLow;
	lParam = TreeNode.lParam;
	lParamEnumerate = TreeNode.lParamEnumerate;

//	pNetExplorerView = TreeNode.pNetExplorerView;
	pLeftView = TreeNode.pLeftView;
	hParent = TreeNode.hParent;
	hItem = TreeNode.hItem;
	hDummyItem = TreeNode.hDummyItem;
}

BOOL CTreeNode::operator ==(const CTreeNode & TreeNode)
{
	return nType == TreeNode.nType && szName == TreeNode.szName && szPath == TreeNode.szPath;
}

/////////////////////////////////////////////////////////////////////////////
// CNetExplorerDoc

CNetExplorerDoc::CNetExplorerDoc()
{

}

CNetExplorerDoc::~CNetExplorerDoc()
{
	POSITION pos = m_ListNETRESOURCE.GetHeadPosition();
	LPNETRESOURCE lpnr;
	for(int i = 0; i < m_ListNETRESOURCE.GetCount(); i++)
		if(lpnr = m_ListNETRESOURCE.GetNext(pos)) {
			if(lpnr->lpLocalName)
				free(lpnr->lpLocalName);
			if(lpnr->lpComment)
				free(lpnr->lpComment);
			if(lpnr->lpRemoteName)
				free(lpnr->lpRemoteName);
			if(lpnr->lpProvider)
				free(lpnr->lpProvider);

			delete lpnr;
		}

	CMap<CString, LPCTSTR, CStringList *, CStringList *>::CPair* pCurVal = m_mapHiddensByParentName.PGetFirstAssoc();
	while(pCurVal != NULL)
	{
		delete pCurVal->value;
		pCurVal = m_mapHiddensByParentName.PGetNextAssoc(pCurVal);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CNetExplorerDoc commands

int CNetExplorerDoc::EnumerateDesktop(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	CTreeNode * pTreeNode;

	HKEY hKeyReg;
	char pszBuf[2048];
	pszBuf[0] = '\0';

	if(RegOpenKeyEx(HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
		0, KEY_ALL_ACCESS, &hKeyReg) == ERROR_SUCCESS)
	{
		ULONG nBufSize = sizeof(pszBuf);
		ULONG nType = REG_SZ;
		RegQueryValueEx(hKeyReg, "Personal", NULL, &nType, (LPBYTE)pszBuf, &nBufSize);
		RegCloseKey(hKeyReg);
	}

	if(pszBuf[0] != '\0')
	{
		pTreeNode = new CTreeNode;
		pTreeNode->nType = CTreeNode::Directory;
		pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
		pTreeNode->szName = "My Documents";
		pTreeNode->szPath = CString(pszBuf) + "\\";
		pTreeNode->szComment = "My Documents";
		pTreeNode->pParent = pParentTreeNode;
		if(!pfnResult(pTreeNode, pParentTreeNode)) return FALSE;
	}

	pTreeNode = new CTreeNode;
	pTreeNode->nType = CTreeNode::WorkArea;
	pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
	pTreeNode->szName = "Work Area";
	pTreeNode->szPath = pTreeNode->szName;
	pTreeNode->szComment = "Work Area";
	pTreeNode->pParent = pParentTreeNode;
	if(!pfnResult(pTreeNode, pParentTreeNode)) return FALSE;

	pTreeNode = new CTreeNode;
	pTreeNode->nType = CTreeNode::NetNeighborhood;
	pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
	pTreeNode->szName = "Network Neighborhood";
	pTreeNode->szPath = pTreeNode->szName;
	pTreeNode->szComment = "Network Neighborhood";
	pTreeNode->pParent = pParentTreeNode;
	if(!pfnResult(pTreeNode, pParentTreeNode)) return FALSE;

	pTreeNode = new CTreeNode;
	pTreeNode->nType = CTreeNode::Network;
	pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
	pTreeNode->szName = "All Network";
	pTreeNode->szPath = pTreeNode->szName;
	pTreeNode->szComment = pTreeNode->szName;
	pTreeNode->pParent = pParentTreeNode;
	if(!pfnResult(pTreeNode, pParentTreeNode)) return FALSE;

	return TRUE;
}

int CNetExplorerDoc::EnumerateWorkArea(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	CTreeNode * pTreeNode;
	
	DWORD nLogicalDrives = GetLogicalDrives();
	for(int i = 0; i < 8 * sizeof(nLogicalDrives); i++)
	{
		if(nLogicalDrives & (1 << i))
		{
			pTreeNode = new CTreeNode;
			pTreeNode->nType = CTreeNode::Drive;
			pTreeNode->nFieldMask = TN_NAME | TN_PATH;
			pTreeNode->szPath = CString((char)('A' + i)) + ":";
			pTreeNode->szName = "(" + pTreeNode->szPath + ")";
			pTreeNode->szPath += "\\";
			pTreeNode->pParent = pParentTreeNode;

			char pszName[16];
			switch(GetDriveType(pTreeNode->szPath + "\\"))
			{
			case DRIVE_UNKNOWN:
				break;
			case DRIVE_NO_ROOT_DIR:
				break;
			case DRIVE_REMOVABLE:
				break;
			case DRIVE_FIXED:
				if(GetVolumeInformation(pTreeNode->szPath + "\\", pszName, sizeof(pszName), NULL, NULL, NULL, NULL,0) && pszName[0])
					pTreeNode->szName = pszName + CString(" ") + pTreeNode->szName;
				break;
			case DRIVE_REMOTE:
				if(GetVolumeInformation(pTreeNode->szPath + "\\", pszName, sizeof(pszName), NULL, NULL, NULL, NULL,0) && pszName[0])
					pTreeNode->szName = pszName + CString(" ") + pTreeNode->szName;
				break;
			case DRIVE_CDROM:
				break;
			case DRIVE_RAMDISK:
				if(GetVolumeInformation(pTreeNode->szPath + "\\", pszName, sizeof(pszName), NULL, NULL, NULL, NULL,0) && pszName[0])
					pTreeNode->szName = pszName + CString(" ") + pTreeNode->szName;
				break;
			}

			if(!pfnResult(pTreeNode, pParentTreeNode)) return FALSE;
		}
	}
	return TRUE;
}

int CNetExplorerDoc::EnumerateNetNeighborhood(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	CTreeNode * pTreeNode;

	DWORD dwResult, dwResultEnum;
	HANDLE hEnum;
	DWORD cbBuffer = 16384;      // 16K is a good size
	DWORD cEntries = -1;         // enumerate all possible entries
	LPNETRESOURCE lpnrLocal;     // pointer to enumerated structures
	DWORD i;
	BOOL fStopNow = FALSE;

	// Call the WNetOpenEnum function to begin the enumeration.
	dwResult = WNetOpenEnum(RESOURCE_CONTEXT, //RESOURCE_GLOBALNET, // all network resources
		RESOURCETYPE_DISK, // RESOURCETYPE_ANY,   // all resources
		0,        // enumerate all resources
		NULL, &hEnum);
	
	if(dwResult != NO_ERROR) {
		// NetErrorHandler(hwnd, dwResult, (LPSTR)"WNetOpenEnum");
		TRACE("Error: %s (code %d)\n", (LPSTR)"WNetOpenEnum", dwResult);
		return FALSE;
	}

	// Call the GlobalAlloc function to allocate resources.
	lpnrLocal = (LPNETRESOURCE)GlobalAlloc(GPTR, cbBuffer);
	
	do {  
		// Initialize the buffer.
		ZeroMemory(lpnrLocal, cbBuffer);

		// Call the WNetEnumResource function to continue the enumeration.
		dwResultEnum = WNetEnumResource(hEnum, &cEntries, lpnrLocal, &cbBuffer); // buffer size

		// If the call succeeds, loop through the structures.
		if(dwResultEnum == NO_ERROR) {
			for(i = 0; i < cEntries && !fStopNow; i++) {
				if(lpnrLocal[i].dwDisplayType == RESOURCEDISPLAYTYPE_SERVER) {
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::Server;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName + 3;
					pTreeNode->szName = CString(lpnrLocal[i].lpRemoteName[2]) + strlwr((LPSTR)(LPCTSTR)pTreeNode->szName);
					pTreeNode->szPath = CString("\\\\") + pTreeNode->szName + "\\";
					pTreeNode->szComment = lpnrLocal[i].lpComment; //lpProvider;
					pTreeNode->pParent = pParentTreeNode;
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
				}
			}
		} else if (dwResultEnum != ERROR_NO_MORE_ITEMS) { // Process errors.
			//      NetErrorHandler(hwnd, dwResultEnum, (LPSTR)"WNetEnumResource");
			TRACE("Error: %s (code %d)\n", (LPSTR)"WNetEnumResource", dwResultEnum);
			break;
		}
	} while(dwResultEnum != ERROR_NO_MORE_ITEMS && !fStopNow);

	// Call the GlobalFree function to free the memory.
	GlobalFree((HGLOBAL)lpnrLocal);

	// Call WNetCloseEnum to end the enumeration.
	if((dwResult = WNetCloseEnum(hEnum)) != NO_ERROR) { 
		// Process errors.
		//    NetErrorHandler(hwnd, dwResult, (LPSTR)"WNetCloseEnum");
		TRACE("Error: %s (code %d)\n", (LPSTR)"WNetCloseEnum", dwResult);
	}

	if(fStopNow) return FALSE;
	return TRUE;
}

int CNetExplorerDoc::EnumerateNetwork(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	CTreeNode * pTreeNode;
	BOOL fStopNow = FALSE;

	LPNETRESOURCE lpnr = NULL;

	DWORD dwResult, dwResultEnum;
	HANDLE hEnum;
	DWORD cbBuffer = 16384;      // 16K is a good size
	DWORD cEntries = -1;         // enumerate all possible entries
	LPNETRESOURCE lpnrLocal;     // pointer to enumerated structures
	DWORD i;

	if(pParentTreeNode->nType != CTreeNode::Network && (pParentTreeNode->nFieldMask & TN_LPARAM))
		lpnr = (LPNETRESOURCE)pParentTreeNode->lParamEnumerate;

	// Call the WNetOpenEnum function to begin the enumeration.
	dwResult = WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_DISK, 0,
		lpnr,     // NULL first time the function is called
		&hEnum);  // handle to the resource

	if(dwResult != NO_ERROR) { //ERROR_NETNAME_DELETED
		// NetErrorHandler(hwnd, dwResult, (LPSTR)"WNetOpenEnum");
		TRACE("Error: %s (code %d)\n", (LPSTR)"WNetOpenEnum", dwResult);
		return FALSE;
	}

	// Call the GlobalAlloc function to allocate resources.
	lpnrLocal = (LPNETRESOURCE)GlobalAlloc(GPTR, cbBuffer);
	
	do {  
		// Initialize the buffer.
		ZeroMemory(lpnrLocal, cbBuffer);

		// Call the WNetEnumResource function to continue
		//  the enumeration.
		dwResultEnum = WNetEnumResource(hEnum,      // resource handle
			&cEntries,  // defined locally as -1
			lpnrLocal,  // LPNETRESOURCE
			&cbBuffer); // buffer size

		// If the call succeeds, loop through the structures.
		if(dwResultEnum == NO_ERROR) {
			for(i = 0; i < cEntries && !fStopNow; i++) {
				switch(lpnrLocal[i].dwDisplayType) {
				case RESOURCEDISPLAYTYPE_NETWORK:
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::NetProtocol;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName;
					pTreeNode->szPath = pTreeNode->szName;
					pTreeNode->pParent = pParentTreeNode;
					if(lpnrLocal[i].lpComment)
						pTreeNode->szComment = lpnrLocal[i].lpComment;
					else
						pTreeNode->szComment = "";
					if(lpnrLocal[i].dwUsage & RESOURCEUSAGE_CONTAINER) {
						pTreeNode->nFieldMask |= TN_LPARAM;
						LPNETRESOURCE lpnr2 = new NETRESOURCE;
						m_ListNETRESOURCE.AddTail(lpnr2);
						*lpnr2 = lpnrLocal[i];
						if(lpnr2->lpLocalName)
							lpnr2->lpLocalName = strdup(lpnrLocal[i].lpLocalName);
						if(lpnr2->lpComment)
							lpnr2->lpComment = strdup(lpnrLocal[i].lpComment);
						if(lpnr2->lpRemoteName)
							lpnr2->lpRemoteName = strdup(lpnrLocal[i].lpRemoteName);
						if(lpnr2->lpProvider)
							lpnr2->lpProvider = strdup(lpnrLocal[i].lpProvider);
						pTreeNode->lParamEnumerate = (LPARAM)lpnr2;
					}
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
					break;
				case RESOURCEDISPLAYTYPE_DOMAIN:
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::Domain;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName + 1;
					pTreeNode->szName = CString(lpnrLocal[i].lpRemoteName[0]) + strlwr((LPSTR)(LPCTSTR)pTreeNode->szName);
					pTreeNode->szPath = pTreeNode->szName;
					pTreeNode->pParent = pParentTreeNode;
					if(lpnrLocal[i].lpComment)
						pTreeNode->szComment = lpnrLocal[i].lpComment;
					else
						pTreeNode->szComment = "";
					if(lpnrLocal[i].dwUsage & RESOURCEUSAGE_CONTAINER) {
						pTreeNode->nFieldMask |= TN_LPARAM;
						LPNETRESOURCE lpnr2 = new NETRESOURCE;
						m_ListNETRESOURCE.AddTail(lpnr2);
						*lpnr2 = lpnrLocal[i];
						if(lpnr2->lpLocalName)
							lpnr2->lpLocalName = strdup(lpnrLocal[i].lpLocalName);
						if(lpnr2->lpComment)
							lpnr2->lpComment = strdup(lpnrLocal[i].lpComment);
						if(lpnr2->lpRemoteName)
							lpnr2->lpRemoteName = strdup(lpnrLocal[i].lpRemoteName);
						if(lpnr2->lpProvider)
							lpnr2->lpProvider = strdup(lpnrLocal[i].lpProvider);
						pTreeNode->lParamEnumerate = (LPARAM)lpnr2;
					}
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
					break;
				case RESOURCEDISPLAYTYPE_SERVER:
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::Server;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName + 3;
					pTreeNode->szName = CString(lpnrLocal[i].lpRemoteName[2]) + strlwr((LPSTR)(LPCTSTR)pTreeNode->szName);
					pTreeNode->szPath = CString("\\\\") + pTreeNode->szName + "\\";
					pTreeNode->pParent = pParentTreeNode;
					if(lpnrLocal[i].lpComment)
						pTreeNode->szComment = lpnrLocal[i].lpComment;
					else
						pTreeNode->szComment = "";
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
					break;
				case RESOURCEDISPLAYTYPE_DIRECTORY:
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::NetDirectory;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName;
					pTreeNode->szPath = pTreeNode->szName;
					pTreeNode->pParent = pParentTreeNode;
					if(lpnrLocal[i].lpComment)
						pTreeNode->szComment = lpnrLocal[i].lpComment;
					else
						pTreeNode->szComment = "";
					if(lpnrLocal[i].dwUsage & RESOURCEUSAGE_CONTAINER) {
						pTreeNode->nFieldMask |= TN_LPARAM;
						LPNETRESOURCE lpnr2 = new NETRESOURCE;
						m_ListNETRESOURCE.AddTail(lpnr2);
						*lpnr2 = lpnrLocal[i];
						if(lpnr2->lpLocalName)
							lpnr2->lpLocalName = strdup(lpnrLocal[i].lpLocalName);
						if(lpnr2->lpComment)
							lpnr2->lpComment = strdup(lpnrLocal[i].lpComment);
						if(lpnr2->lpRemoteName)
							lpnr2->lpRemoteName = strdup(lpnrLocal[i].lpRemoteName);
						if(lpnr2->lpProvider)
							lpnr2->lpProvider = strdup(lpnrLocal[i].lpProvider);
						pTreeNode->lParamEnumerate = (LPARAM)lpnr2;
					}
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
					break;
				case RESOURCEDISPLAYTYPE_SHARE:
					pTreeNode = new CTreeNode;
					pTreeNode->nType = CTreeNode::NetShare;
					pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
					pTreeNode->szName = lpnrLocal[i].lpRemoteName;
					pTreeNode->szPath = pTreeNode->szName + '\\';
					pTreeNode->pParent = pParentTreeNode;
					int nPos = pTreeNode->szName.ReverseFind('\\');
					if(nPos != -1)
						pTreeNode->szName = pTreeNode->szName.Mid(nPos + 1);
					if(lpnrLocal[i].lpComment)
						pTreeNode->szComment = lpnrLocal[i].lpComment;
					else
						pTreeNode->szComment = "";
					if(!pfnResult(pTreeNode, pParentTreeNode))
						fStopNow = TRUE;
					break;
				}
			}
		} else if (dwResultEnum != ERROR_NO_MORE_ITEMS) { // Process errors.
			//      NetErrorHandler(hwnd, dwResultEnum, (LPSTR)"WNetEnumResource");
			printf("Error: %s (code %d)\n", (LPSTR)"WNetEnumResource", dwResultEnum);
			break;
		}
	} while(dwResultEnum != ERROR_NO_MORE_ITEMS && !fStopNow);

	// Call the GlobalFree function to free the memory.
	GlobalFree((HGLOBAL)lpnrLocal);

	// Call WNetCloseEnum to end the enumeration.
	dwResult = WNetCloseEnum(hEnum);
	
	if(dwResult != NO_ERROR) { 
		// Process errors.
		//    NetErrorHandler(hwnd, dwResult, (LPSTR)"WNetCloseEnum");
		printf("Error: %s (code %d)\n", (LPSTR)"WNetCloseEnum", dwResult);
	}

	// Hidden server
	CStringList * pStringListHiddens = m_mapHiddensByParentName[pParentTreeNode->szName];
	if(pStringListHiddens != NULL)
	{
		for(POSITION pos = pStringListHiddens->GetHeadPosition(); pos != NULL;)
		{
			CString & szHidden = pStringListHiddens->GetNext(pos);
			pTreeNode = new CTreeNode;
			pTreeNode->nType = CTreeNode::Server;
			pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
			pTreeNode->szName = szHidden;
			pTreeNode->szPath = CString(_T("\\\\")) + pTreeNode->szName + CString(_T("\\"));
			pTreeNode->szComment = _T("");
			pTreeNode->dwPermissions = 0;
			pTreeNode->pParent = pParentTreeNode;
			if(!pfnResult(pTreeNode, pParentTreeNode))
				fStopNow = TRUE;
		}
	}

	if(fStopNow) return FALSE;
	return TRUE;
}

int CNetExplorerDoc::EnumerateShare(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	BOOL fStopNow = FALSE;
	CTreeNode * pTreeNode;


	// Hidden shares
	CStringList * pStringListHiddens = m_mapHiddensByParentName[pParentTreeNode->szName];
	if(pStringListHiddens != NULL)
	{
		for(POSITION pos = pStringListHiddens->GetHeadPosition(); pos != NULL;)
		{
			CString & szHidden = pStringListHiddens->GetNext(pos);
			pTreeNode = new CTreeNode;
			pTreeNode->nType = CTreeNode::Share;
			pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
			pTreeNode->szName = szHidden;
			pTreeNode->szPath = pParentTreeNode->szPath + pTreeNode->szName + CString(_T("\\"));
			pTreeNode->szComment = _T("");
			pTreeNode->dwPermissions = 0;
			pTreeNode->pParent = pParentTreeNode;
			if(!pfnResult(pTreeNode, pParentTreeNode))
				fStopNow = TRUE;
		}
	}


	// Real shares
	NET_API_STATUS res;
	DWORD er = 0, tr = 0, resume = 0, i;

	// Convert ANSI user name and domain to Unicode
	WCHAR wszServer[256];
	MultiByteToWideChar(CP_ACP, 0, pParentTreeNode->szPath, strlen(pParentTreeNode->szPath) + 1, wszServer, sizeof(wszServer) / sizeof(wszServer[0]));

	do
	{
		{
			SHARE_INFO_1 * pShareInfo1, * pOriginalShareInfo1; // NT & 95/98
			try
			{
				res = NetShareEnum(/*(LPSTR)(LPCTSTR)*/wszServer, 1, (LPBYTE *) &pOriginalShareInfo1, -1, &er, &tr, &resume);
			}
			catch(int nError)
			{
				TRACE("Error: NetShareEnum (%n 0x%08x)\n", nError, nError);
			}
			catch(char * pszError)
			{
				TRACE("Error: NetShareEnum (%s)\n", pszError);
			}
			catch(...)
			{
				TRACE("Error: NetShareEnum Unkown error\n");
			}
			if(res == ERROR_SUCCESS || res == ERROR_MORE_DATA) {
				pShareInfo1 = pOriginalShareInfo1;
				for(i = 1; i <= er && !fStopNow; i++) {
					if(pShareInfo1->shi1_type == STYPE_DISKTREE) {
						char pszDest[256];
						pTreeNode = new CTreeNode;
						pTreeNode->nType = CTreeNode::Share;
						pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_COMMENT;
						WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)pShareInfo1->shi1_netname, -1, pszDest, 256, NULL, NULL);
						pTreeNode->szName = pszDest;
						pTreeNode->szPath = pParentTreeNode->szPath + pTreeNode->szName + "\\";
						WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)pShareInfo1->shi1_remark, -1, pszDest, 256, NULL, NULL);
						pTreeNode->szComment = pszDest;
						pTreeNode->dwPermissions = 0; //pShareInfo1->shi1_permissions;
						pTreeNode->pParent = pParentTreeNode;
						if(!pfnResult(pTreeNode, pParentTreeNode))
							fStopNow = TRUE;
					}
					pShareInfo1++;
				}
				NetApiBufferFree(pOriginalShareInfo1);
			}
			else 
				TRACE("Error: %ld\n",res);
		}
	} while(res == ERROR_MORE_DATA && !fStopNow);

	if(fStopNow) return FALSE;
	return TRUE;
}


int CNetExplorerDoc::CTreeNodeCompareByName(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2)
{
	return pTreeNode1->szName.Compare(pTreeNode2->szName);
}

int CNetExplorerDoc::CTreeNodeCompareByExtension(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2)
{
	int nResult = pTreeNode1->szExtension.Compare(pTreeNode2->szExtension);
	if(nResult == 0)
		nResult = pTreeNode1->szName.Compare(pTreeNode2->szName);
	return nResult;
}

int CNetExplorerDoc::CTreeNodeCompareByDate(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2)
{
	int nResult = 0;
	double rSpan = (pTreeNode1->tLastWriteTime - pTreeNode2->tLastWriteTime).m_span;
	if(rSpan == 0.0)
		nResult = 0;
	else if(rSpan > 0.0)
		nResult = 1;
	else if(rSpan < 0.0)
		nResult = -1;
	if(nResult == 0)
		nResult = pTreeNode1->szName.Compare(pTreeNode2->szName);

	return nResult;
}

void CNetExplorerDoc::QuickSortRecursive(CTreeNode* *pArr, int d, int h, BOOL bAscending, int (*pTreeNodeComparer)(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2))
{
	int i,j;
	CTreeNode* str;

	i = h;
	j = d;

	str = pArr[((int)((d + h) / 2))];

	do
	{
		if(bAscending)
		{
			while((*pTreeNodeComparer)(pArr[j], str) < 0) j++;
			while((*pTreeNodeComparer)(pArr[i], str) > 0) i--;
		}
		else
		{
			while((*pTreeNodeComparer)(pArr[j], str) > 0) j++;
			while((*pTreeNodeComparer)(pArr[i], str) < 0) i--;
		}

		if(i >= j)
		{
			if(i != j)
			{
				CTreeNode * zal = pArr[i];
				pArr[i] = pArr[j];
				pArr[j] = zal;
			}

			i--;
			j++;
		}
	}
	while(j <= i);

	if(d < i) QuickSortRecursive(pArr, d, i, bAscending, pTreeNodeComparer);
	if(j < h) QuickSortRecursive(pArr, j, h, bAscending, pTreeNodeComparer);
}

void CNetExplorerDoc::QuickSort(CArray<CTreeNode*> * pArray, BOOL bAscending, int (*pTreeNodeComparer)(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2))
{
	if(pArray->GetCount() > 1)
		QuickSortRecursive(pArray->GetData(), 0, pArray->GetCount() - 1, bAscending, pTreeNodeComparer);
}

int CNetExplorerDoc::EnumerateFiles(int nFileTypeMask, enum SortType nSortAttribute, CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *))
{
	HANDLE fd;
	WIN32_FIND_DATA findFileData;
	CTreeNode * pTreeNode;

	if(!pfnResult || !nFileTypeMask)
		return FALSE;

	CString szFilenames(pParentTreeNode->szPath), szFilename;
	szFilenames += "*.*";

	BOOL fStopNow = FALSE;

	fd = FindFirstFile(szFilenames, &findFileData);
	if(fd != INVALID_HANDLE_VALUE)
	{
		CArray<CTreeNode*> aFiles;
		CArray<CTreeNode*> aDirectories;
		do
		{
			if((nFileTypeMask & ENUM_FILES) && !(findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
//DWORD dwFileAttributes;
				pTreeNode = new CTreeNode;
				pTreeNode->nType = CTreeNode::File;
				pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_SIZE | TN_DATE;
				pTreeNode->szName = findFileData.cFileName;
				pTreeNode->szPath = pParentTreeNode->szPath + pTreeNode->szName;
				int nPos = pTreeNode->szName.ReverseFind(_T('.'));
				if(nPos != -1)
					pTreeNode->szExtension = pTreeNode->szName.Mid(nPos + 1);
				else
					pTreeNode->szExtension = "";
				pTreeNode->szExtension.MakeUpper();
				pTreeNode->dwFileAttributes = findFileData.dwFileAttributes;
				pTreeNode->nFileSizeHigh = findFileData.nFileSizeHigh;
				pTreeNode->nFileSizeLow = findFileData.nFileSizeLow;
				pTreeNode->tCreationTime = findFileData.ftCreationTime;
				pTreeNode->tLastAccessTime = findFileData.ftLastAccessTime;
				pTreeNode->tLastWriteTime = findFileData.ftLastWriteTime;
				pTreeNode->pParent = pParentTreeNode;
				aFiles.Add(pTreeNode);

//				TRACE("\tFILE %s\n", findFileData.cFileName);
			}
			else if((nFileTypeMask & ENUM_DIRECTORIES) && strcmp(findFileData.cFileName, ".") && strcmp(findFileData.cFileName, "..") && (findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				pTreeNode = new CTreeNode;
				pTreeNode->nType = CTreeNode::Directory;
				pTreeNode->nFieldMask = TN_PATH | TN_NAME | TN_DATE;
				pTreeNode->szName = findFileData.cFileName;
				pTreeNode->szPath = pParentTreeNode->szPath + pTreeNode->szName + "\\";
				int nPos = pTreeNode->szName.ReverseFind(_T('.'));
				if(nPos != -1)
					pTreeNode->szExtension = pTreeNode->szName.Mid(nPos + 1);
				else
					pTreeNode->szExtension = "";
				pTreeNode->szExtension.MakeUpper();
				pTreeNode->dwFileAttributes = findFileData.dwFileAttributes;
				pTreeNode->tCreationTime = findFileData.ftCreationTime;
				pTreeNode->tLastAccessTime = findFileData.ftLastAccessTime;
				pTreeNode->tLastWriteTime = findFileData.ftLastWriteTime;
				pTreeNode->pParent = pParentTreeNode;
				aDirectories.Add(pTreeNode);

//				TRACE("\tDIRECTORY %s\n", findFileData.cFileName);
			}

		}
		while(/*!fStopNow && */FindNextFile(fd, &findFileData));
		FindClose(fd);

		if(nSortAttribute & CNetExplorerDoc::SortByName)
		{
			QuickSort(&aFiles, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByName);
			QuickSort(&aDirectories, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByName);
		}
		else if(nSortAttribute & CNetExplorerDoc::SortByExtension)
		{
			QuickSort(&aFiles, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByExtension);
			QuickSort(&aDirectories, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByExtension);
		}
		else if(nSortAttribute & CNetExplorerDoc::SortByDate)
		{
			QuickSort(&aFiles, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByDate);
			QuickSort(&aDirectories, nSortAttribute & CNetExplorerDoc::SortAscending, CNetExplorerDoc::CTreeNodeCompareByDate);
		}

		for(int i = 0; i < aDirectories.GetCount() && !fStopNow; i++)
		{
			pTreeNode = aDirectories[i];
			if(!pfnResult(pTreeNode, pParentTreeNode))
				fStopNow = TRUE;
		}
		for(int i = 0; i < aFiles.GetCount() && !fStopNow; i++)
		{
			pTreeNode = aFiles[i];
			if(!pfnResult(pTreeNode, pParentTreeNode))
				fStopNow = TRUE;
		}
	}

	if(fStopNow)
		return FALSE;
	return TRUE;
}
