//    NetExplorer - Windows LAN explorer application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// NetExplorerDoc.h : interface of the CNetExplorerDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_NETEXPLORERDOC_H__INCLUDED_)
#define AFX_NETEXPLORERDOC_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//class CNetExplorerView;
class CLeftView;

#define ENUM_FILES			1
#define ENUM_DIRECTORIES	2

#define TN_NAME		1
#define TN_PATH		2
#define TN_COMMENT	4
#define TN_SIZE		8
#define TN_DATE		16
#define TN_LPARAM	32
#define TN_SEARCH	64

class CTreeNode;

typedef CArray<CTreeNode *, CTreeNode *> CTreeNodeArray;

class CTreeNode
{
public:
	enum enType {
		Desktop = 0, WorkArea, Drive,
		NetNeighborhood, Network, NetProtocol, NetDirectory, NetShare,
		Domain, Server, Share, Directory, SelectedDirectory, File
	};

	CTreeNode() {}
	CTreeNode(const CTreeNode & TreeNode);
	void operator =(const CTreeNode & TreeNode);
	BOOL operator ==(const CTreeNode & TreeNode);
	static int Type2ImageNumber(enType nType);

	CTreeNode * pParent;
	enum enType nType;
	DWORD nFieldMask;
	CString szName;
	CString szExtension;
	CString szPath;
	CString szComment;
	CString szSearchPath;

	COleDateTime tCreationTime;
	COleDateTime tLastAccessTime;
	COleDateTime tLastWriteTime;

	DWORD dwFileAttributes;
	DWORD dwPermissions;
	DWORD nFileSizeHigh;
	DWORD nFileSizeLow;

	DWORD lParam;
	DWORD lParamEnumerate;

	// IHM particularity
//	CNetExplorerView * pNetExplorerView;
	CLeftView * pLeftView;
	HTREEITEM hParent;
	HTREEITEM hItem;
	HTREEITEM hDummyItem;
};

typedef CList<CTreeNode *, CTreeNode *> CTreeNodeList;

class CNetExplorerDoc : public CObject //CDocument
{
public:
	CNetExplorerDoc();
	virtual ~CNetExplorerDoc();
//	DECLARE_DYNCREATE(CNetExplorerDoc)

// Attributes
public:
	CMap<CString, LPCTSTR, CStringList *, CStringList *> m_mapHiddensByParentName;

// Operations
public:
	enum SortType
	{
		NoSort = 0x00,
		SortByName = 0x01,
		SortByExtension = 0x02,
		SortByDate = 0x04,
		SortAscending = 0x08,
	};
	int EnumerateDesktop(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));
	int EnumerateWorkArea(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));
	int EnumerateNetNeighborhood(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));
	int EnumerateShare(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));
	int EnumerateFiles(int nFileTypeMask, enum SortType nSortAttribute, CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));
	int EnumerateNetwork(CTreeNode * pParentTreeNode, BOOL (* pfnResult)(CTreeNode *, CTreeNode *));

	static int CTreeNodeCompareByName(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2);
	static int CTreeNodeCompareByExtension(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2);
	static int CTreeNodeCompareByDate(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2);
	static void QuickSortRecursive(CTreeNode* *pArr, int d, int h, BOOL bAscending, int (*pTreeNodeComparer)(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2));
	static void QuickSort(CArray<CTreeNode*> * pArray, BOOL bAscending, int (*pTreeNodeComparer)(CTreeNode* pTreeNode1, CTreeNode* pTreeNode2));

//// Overrides
//	// ClassWizard generated virtual function overrides
//	//{{AFX_VIRTUAL(CNetExplorerDoc)
//	public:
//	virtual BOOL OnNewDocument();
//	virtual void Serialize(CArchive& ar);
//	//}}AFX_VIRTUAL

//#ifdef _DEBUG
//	virtual void AssertValid() const;
//	virtual void Dump(CDumpContext& dc) const;
//#endif

protected:
	CList<LPNETRESOURCE, LPNETRESOURCE&> m_ListNETRESOURCE;

//// Generated message map functions
//protected:
//	//{{AFX_MSG(CNetExplorerDoc)
//		// NOTE - the ClassWizard will add and remove member functions here.
//		//    DO NOT EDIT what you see in these blocks of generated code !
//	//}}AFX_MSG
//	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_NETEXPLORERDOC_H__INCLUDED_)
