//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// PlayListDialogBar.cpp : implementation file
//

#include "stdafx.h"
#include "MPlayerX.h"
#include "PlayListDialogBar.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPlayListDialogBar

CPlayListDialogBar::CPlayListDialogBar()
{
	m_dwCurrentFile = NULL;
}

CPlayListDialogBar::~CPlayListDialogBar()
{
}

#define LVS_EX_LABELTIP         0x00004000
#define LVN_ENDSCROLL            (LVN_FIRST-81)

BEGIN_MESSAGE_MAP(CPlayListDialogBar, CSizingControlBarG)
	//{{AFX_MSG_MAP(CPlayListDialogBar)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_NOTIFY(NM_DBLCLK, IDC_LISTCTRL_PLAY_LIST, OnDblclkList)
	ON_NOTIFY(NM_RCLICK, IDC_LISTCTRL_PLAY_LIST, OnRclickList)
	ON_NOTIFY(LVN_ENDSCROLL, IDC_LISTCTRL_PLAY_LIST, OnEndScrollList)
	ON_NOTIFY(LVN_BEGINDRAG, IDC_LISTCTRL_PLAY_LIST, OnBegindragList)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPlayListDialogBar message handlers

int CPlayListDialogBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CSizingControlBarG::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if(!m_wndPlayListListCtrl.Create(WS_CHILD | WS_VISIBLE | LVS_REPORT | LVS_NOCOLUMNHEADER,
			CRect(0, 0, 0, 0), this, IDC_LISTCTRL_PLAY_LIST))
		return -1;
	
	m_wndPlayListListCtrl.SetExtendedStyle(LVS_EX_LABELTIP);
	m_wndPlayListListCtrl.ModifyStyleEx(0, WS_EX_ACCEPTFILES);

	m_ImageList.Create(IDB_BITMAP_PLAY_LIST, 16, 2, RGB(255, 255, 255));
	m_wndPlayListListCtrl.SetImageList(&m_ImageList, LVSIL_SMALL);

	m_wndPlayListListCtrl.InsertColumn(0, _T(""), LVCFMT_LEFT, -1);
	
	//// Set up initial variables
	m_bDragging = false;
	m_nDragIndex = -1;
	m_nDropIndex = -1;
	m_pDragImage = NULL;

	return 0;
}

void CPlayListDialogBar::OnDestroy()
{
	CSizingControlBarG::OnDestroy();

	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		TCHAR * pszPathname = (TCHAR *)m_wndPlayListListCtrl.GetItemData(i);
		if(pszPathname)
			delete [] pszPathname;
	}
}

void CPlayListDialogBar::UpdateColumnWidth()
{
	int nColumnWidth = -1;
	int nFirst = m_wndPlayListListCtrl.GetTopIndex();
	int nLast = nFirst + m_wndPlayListListCtrl.GetCountPerPage();
	for(int i = nFirst; i <= nLast; i++)
	{
		CString szString = m_wndPlayListListCtrl.GetItemText(i, 0);
		int nItemWidth = m_wndPlayListListCtrl.GetStringWidth(szString);
		if(nItemWidth > nColumnWidth)
			nColumnWidth = nItemWidth;
	}
	m_wndPlayListListCtrl.SetColumnWidth(0, nColumnWidth + 25);
}

void CPlayListDialogBar::OnSize(UINT nType, int cx, int cy) 
{
	CSizingControlBarG::OnSize(nType, cx, cy);

	m_wndPlayListListCtrl.MoveWindow(0, 0, cx, cy);
	UpdateColumnWidth();
}

void CPlayListDialogBar::DeleteCurrentFile()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int nItem = 0; nItem < nCount; nItem++)
	{
		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(nItem);
		if(dwFile == m_dwCurrentFile)
		{
			TCHAR * pszPathname = (TCHAR *)dwFile;
			if(pszPathname)
				delete [] pszPathname;

			m_wndPlayListListCtrl.DeleteItem(nItem);
			return;
		}
	}
}

void CPlayListDialogBar::DeleteSelectedFile()
{
	POSITION pos;
	while (pos = m_wndPlayListListCtrl.GetFirstSelectedItemPosition())
	{
		int nItem = m_wndPlayListListCtrl.GetNextSelectedItem(pos);
		TCHAR * pszPathname = (TCHAR *)m_wndPlayListListCtrl.GetItemData(nItem);
		if(pszPathname)
			delete [] pszPathname;

		m_wndPlayListListCtrl.DeleteItem(nItem);
	}
}

void CPlayListDialogBar::DeleteAllFile()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		TCHAR * pszPathname = (TCHAR *)m_wndPlayListListCtrl.GetItemData(i);
		if(pszPathname)
			delete [] pszPathname;
	}
	m_wndPlayListListCtrl.DeleteAllItems();
}

BOOL CPlayListDialogBar::AddFileToTail(LPCTSTR pszFilename, BOOL fSetCurrent)
{
	CString szFilename(pszFilename);
	int nPos = szFilename.ReverseFind(_T('\\'));
	if(nPos != -1)
		szFilename = szFilename.Mid(nPos + 1);
	
	int iItem = m_wndPlayListListCtrl.InsertItem(m_wndPlayListListCtrl.GetItemCount(), szFilename);
	m_wndPlayListListCtrl.SetItemText(iItem, 1, _T(""));
	TCHAR * pszFileData = _tcsdup(pszFilename);
	m_wndPlayListListCtrl.SetItemData(iItem, (DWORD)pszFileData);
	if(fSetCurrent)
		m_dwCurrentFile = (DWORD)pszFileData;

//	m_wndPlayListListCtrl.SetColumnWidth(0, -1);
	UpdateColumnWidth();

	return TRUE;
}

int CPlayListDialogBar::GetCurrentFileId()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(i);
		if(dwFile == m_dwCurrentFile)
			return i;
	}
	return 0;
}

LPCTSTR CPlayListDialogBar::GetCurrentFile()
{
	return (LPCTSTR)m_dwCurrentFile;
}

LPCTSTR CPlayListDialogBar::GetPreviousFile()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(i);
		if(dwFile == m_dwCurrentFile)
		{
			if(i == 0)
				break;
			return (LPCTSTR)m_wndPlayListListCtrl.GetItemData(i - 1);
		}
	}
	return NULL;
}

LPCTSTR CPlayListDialogBar::GetNextFile()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(i);
		if(dwFile == m_dwCurrentFile)
		{
			if(i + 1 == nCount)
				break;
			return (LPCTSTR)m_wndPlayListListCtrl.GetItemData(i + 1);
		}
	}
	return NULL;
}

void CPlayListDialogBar::SetCurrentFile(LPCTSTR pszFilename)
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int i = 0; i < nCount; i++)
	{
		LVITEM lvi;
		memset(&lvi, 0, sizeof(lvi));
		lvi.mask = LVIF_IMAGE;
		lvi.iItem = i;

		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(i);
		if(dwFile == (DWORD)pszFilename)
		{
			m_wndPlayListListCtrl.EnsureVisible(i, FALSE);
			m_dwCurrentFile = (DWORD)pszFilename;
			lvi.iImage = 1;
		}
		else
			lvi.iImage = 0;

		m_wndPlayListListCtrl.SetItem(&lvi);
	}
}

BOOL CPlayListDialogBar::IsCurrentSelected()
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	for(int nItem = 0; nItem < nCount; nItem++)
	{
		DWORD dwFile = m_wndPlayListListCtrl.GetItemData(nItem);
		if(dwFile == m_dwCurrentFile)
			return m_wndPlayListListCtrl.GetItemState(nItem, LVIS_SELECTED) == LVIS_SELECTED;
	}

	return FALSE;
}

LPCTSTR CPlayListDialogBar::GetFile(int nItem)
{
	int nCount = m_wndPlayListListCtrl.GetItemCount();
	if(nItem >= 0 && nItem < nCount)
		return (LPCTSTR)m_wndPlayListListCtrl.GetItemData(nItem);

	return NULL;
}

void CPlayListDialogBar::OnDblclkList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	GetCursorPos(&point);
	m_wndPlayListListCtrl.ScreenToClient(&point);
	int nItem = m_wndPlayListListCtrl.HitTest(point);
	if(nItem != -1)
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame)
			pMainFrame->PlayFile((LPCTSTR)m_wndPlayListListCtrl.GetItemData(nItem), true);
	}	
	*pResult = 0;
}

void CPlayListDialogBar::OnRclickList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	GetCursorPos(&point);
	CMenu menu;
	menu.LoadMenu(IDR_MENU_PLAYLIST);
	CMenu * pMenu = NULL;
	if(pMenu = menu.GetSubMenu(0))
		pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
			point.x, point.y, AfxGetMainWnd());
	
	*pResult = 0;
}

void CPlayListDialogBar::OnEndScrollList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateColumnWidth();

	*pResult = 0;
}

void CPlayListDialogBar::OnBegindragList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	m_nDragIndex = pNMListView->iItem;

	//// Create a drag image
	POINT pt;
	int nOffset = -8; //offset in pixels for drag image (positive is up and to the left; neg is down and to the right)
	if(m_wndPlayListListCtrl.GetSelectedCount() > 1) //more than one item is selected
	pt.x = nOffset;
	pt.y = nOffset;

    IMAGEINFO       imf;
    int             iHeight;
	BOOL bFirst = TRUE;
	int iPos = m_wndPlayListListCtrl.GetNextItem(-1, LVNI_SELECTED);
	while(iPos != -1)
	{
		if(bFirst)
		{
			// For the first selected item, we simply create a single-line drag image
			m_pDragImage = m_wndPlayListListCtrl.CreateDragImage(iPos, &pt);
			ImageList_GetImageInfo(m_pDragImage->m_hImageList, 0, &imf);
			iHeight = imf.rcImage.bottom;
			bFirst = FALSE;
		}
		else
		{
			// For the rest selected items, we create a single-line drag image, then
			// append it to the bottom of the complete drag image
			CImageList * pOneImageList = m_wndPlayListListCtrl.CreateDragImage(iPos, &pt);
			HIMAGELIST hTempImageList = ImageList_Merge(m_pDragImage->m_hImageList, 0, pOneImageList->m_hImageList, 0, 0, iHeight);
			delete pOneImageList;
			delete m_pDragImage;
			m_pDragImage = new CImageList();
			m_pDragImage->Attach(hTempImageList);
			ImageList_GetImageInfo(m_pDragImage->m_hImageList, 0, &imf);
			iHeight = imf.rcImage.bottom;
		}
		iPos = m_wndPlayListListCtrl.GetNextItem(iPos, LVNI_SELECTED);
	}

	if(!m_pDragImage) return;

	//// Change the cursor to the drag image
	////	(still must perform DragMove() in OnMouseMove() to show it moving)
	m_pDragImage->BeginDrag(0, CPoint(nOffset, nOffset));
	m_pDragImage->DragEnter(GetDesktopWindow(), pNMListView->ptAction);
	
	//// Set dragging flag and others
	m_bDragging = TRUE;	//we are in a drag and drop operation
	m_nDropIndex = -1;	//we don't have a drop index yet

	m_wndPlayListListCtrl.SetFocus();

	SetCapture ();

	*pResult = 0;
}


void CPlayListDialogBar::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bDragging)
	{
		//// Move the drag image
		CPoint pt(point);	//get our current mouse coordinates
		ClientToScreen(&pt); //convert to screen coordinates
		m_pDragImage->DragMove(pt); //move the drag image to those coordinates
		// Unlock window updates (this allows the dragging image to be shown smoothly)
		m_pDragImage->DragShowNolock(false);

		//// Get the CWnd pointer of the window that is under the mouse cursor
		CWnd* pDropWnd = WindowFromPoint (pt);
		if(!pDropWnd) return;

		// Convert from screen coordinates to drop target client coordinates
		pDropWnd->ScreenToClient(&pt);

		//If we are hovering over a CListCtrl we need to adjust the highlights
		if(pDropWnd->m_hWnd == m_wndPlayListListCtrl.m_hWnd) //pDropWnd->IsKindOf(RUNTIME_CLASS (CListCtrl)))
		{			
			//Note that we can drop here
			SetCursor(LoadCursor(NULL, IDC_ARROW));
			UINT uFlags;
			CListCtrl* pList = (CListCtrl*)pDropWnd;

			// Turn off hilight for previous drop target
			pList->SetItemState (m_nDropIndex, 0, LVIS_DROPHILITED);
			// Redraw previous item
			pList->RedrawItems (m_nDropIndex, m_nDropIndex);
			
			// Get the item that is below cursor
			m_nDropIndex = ((CListCtrl*)pDropWnd)->HitTest(pt, &uFlags);
			// Highlight it
			pList->SetItemState(m_nDropIndex, LVIS_DROPHILITED, LVIS_DROPHILITED);
			// Redraw item
			pList->RedrawItems(m_nDropIndex, m_nDropIndex);
			pList->UpdateWindow();
		}
		else
		{
			//If we are not hovering over a CListCtrl, change the cursor
			// to note that we cannot drop here
			SetCursor(LoadCursor(NULL, IDC_NO));
		}
		// Lock window updates
		m_pDragImage->DragShowNolock(true);
	}
		
	CSizingControlBarG::OnMouseMove(nFlags, point);
}


void CPlayListDialogBar::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(m_bDragging)
	{
		// Release mouse capture, so that other controls can get control/messages
		ReleaseCapture();

		// Note that we are NOT in a drag operation
		m_bDragging = FALSE;

		// End dragging image
		m_pDragImage->DragLeave(GetDesktopWindow());
		m_pDragImage->EndDrag();
		delete m_pDragImage;

		CPoint pt(point);
		ClientToScreen(&pt);

		CWnd* pDropWnd = WindowFromPoint (pt);
		if(!pDropWnd) return;

		if(pDropWnd->m_hWnd == m_wndPlayListListCtrl.m_hWnd)
		{
			// Unhilight the drop target
			m_wndPlayListListCtrl.SetItemState(m_nDropIndex, 0, LVIS_DROPHILITED);

			//Stuff for storing the Items in memory
			CList<LVITEM*, LVITEM*> listItems; //declare a linked list of LV_ITEMS

			//Retrieve the selected items, store them in memory and delete them from the CListCtrl
			POSITION pos = m_wndPlayListListCtrl.GetFirstSelectedItemPosition(); //iterator for the CListCtrl
			while(pos)
			{
				m_nDragIndex = m_wndPlayListListCtrl.GetNextSelectedItem(pos);

				LVITEM * pItem = new LV_ITEM;
				ZeroMemory(pItem, sizeof (LV_ITEM));
				pItem->iItem = m_nDragIndex;
				pItem->mask = LVIF_IMAGE | LVIF_PARAM;
				m_wndPlayListListCtrl.GetItem(pItem);

				pItem->mask |= LVIF_TEXT;
				pItem->pszText = _tcsdup((LPCTSTR)m_wndPlayListListCtrl.GetItemText(m_nDragIndex, 0));
				pItem->cchTextMax = _tcslen(pItem->pszText);

				listItems.AddTail(pItem);
			}

			
			//Delete the selected items
			pos = m_wndPlayListListCtrl.GetFirstSelectedItemPosition();
			while(pos)
			{
				pos = m_wndPlayListListCtrl.GetFirstSelectedItemPosition();
				m_nDragIndex = m_wndPlayListListCtrl.GetNextSelectedItem(pos);

				m_wndPlayListListCtrl.DeleteItem(m_nDragIndex); //since we are MOVING, delete the item
				if(m_nDragIndex < m_nDropIndex) m_nDropIndex--; //must decrement the drop index to account
																//for the deleted items
			}
			

			//Iterate through the items stored in memory and add them back into the CListCtrl at the drop index
			pos = listItems.GetHeadPosition();
			while(pos)
			{
				LVITEM * pItem = listItems.GetNext(pos); //retrieve subsequent items

				m_nDropIndex = (m_nDropIndex == -1) ? m_wndPlayListListCtrl.GetItemCount() : m_nDropIndex;
				pItem->iItem = m_nDropIndex;
				m_wndPlayListListCtrl.InsertItem(pItem); //add the item

				m_wndPlayListListCtrl.SetItemState(pItem->iItem, LVIS_SELECTED, LVIS_SELECTED); //highlight/select the item we just added

				delete [] pItem->pszText;
				delete pItem;
				m_nDropIndex++; //increment the index we are dropping at to keep the dropped items in the same order they were in in the Drag List
				//If we dont' increment this, the items are added in reverse order
			}
		}
	}
	
	CSizingControlBarG::OnLButtonUp(nFlags, point);
}

BOOL CPlayListDialogBar::PreTranslateMessage(MSG* pMsg)
{
	if(pMsg->message == WM_KEYDOWN && m_bDragging && pMsg->wParam == VK_ESCAPE)
	{
		ReleaseCapture();
		m_bDragging = FALSE;
		m_pDragImage->DragLeave(GetDesktopWindow());
		m_pDragImage->EndDrag();
		delete m_pDragImage;
		m_wndPlayListListCtrl.SetItemState(m_nDropIndex, 0, LVIS_DROPHILITED);
	}

	return CSizingControlBarG::PreTranslateMessage(pMsg);
}
