//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// RemoteControl.cpp: implementation of the CRemoteControl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RemoteControl.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
HWND CRemoteControl::m_hWnd = NULL;

CRemoteControl::CRemoteControl()
{
	m_hMapFile = NULL;
	m_hPlugin = NULL;
	m_lpMapAddress = NULL;
	m_hSMSem = NULL;
	m_timeStart = m_timeStop = 0;
	m_nRepeatCount = 0;
	m_szLastCommand = _T("");

	nGirderAPIVersion = 0;

	Girder0_identify_device = NULL;
	Girder0_support_device = NULL;
	Girder0_version_device = NULL;
	Girder0_get_device_num = NULL;
	Girder0_init_device = NULL;
	Girder0_clean_up_device = NULL;
	Girder0_start_device = NULL;
	Girder0_stop_device = NULL;
	Girder0_flush_device = NULL;
	Girder0_config_device = NULL;
	Girder0_compare_str = NULL;
	Girder0_learn_device = NULL;

	Girder1_gir_version = NULL;
	Girder1_gir_name = NULL;
	Girder1_gir_description = NULL;
	Girder1_gir_devicenum = NULL;
	Girder1_gir_requested_api = NULL;
	Girder1_gir_open = NULL;
	Girder1_gir_close = NULL;
	Girder1_gir_config = NULL;
	Girder1_gir_start = NULL;
	Girder1_gir_stop = NULL;

	if(m_hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,                    // Read/write permission. 
		0, 250, _T("GirderFileMap6111976")))
		if(m_lpMapAddress = MapViewOfFile(m_hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, 250))
			m_hSMSem = CreateSemaphore(NULL, 1, 1, _T("GirderSemaphore6111976"));
}

CRemoteControl::~CRemoteControl()
{
	CleanUp();

	if(m_hSMSem)
	{
		CloseHandle(m_hSMSem);
		m_hSMSem = NULL;
	}

	if(m_lpMapAddress)
	{
		UnmapViewOfFile(m_lpMapAddress);
		m_lpMapAddress = NULL;
	}

	if(m_hMapFile)
	{
		CloseHandle(m_hMapFile);
		m_hMapFile = NULL;
	}
}

PCHAR __stdcall MyTranslateFunct(PCHAR eng)
{
	return eng;
}

BOOL CRemoteControl::Init(HWND hWnd, LPCTSTR pszPluginPath)
{
	CleanUp();

	m_hWnd = hWnd;

	m_hPlugin = LoadLibrary(pszPluginPath); // "WinLIRC2GIrder.dll");
	if(m_hPlugin)
	{
		Girder1_gir_version = (void (__stdcall *)(char *, int))GetProcAddress(m_hPlugin, "gir_version");
		if(Girder1_gir_version)
		{
			nGirderAPIVersion = 1;

			Girder1_gir_name = (void (__stdcall *)(char *, int))GetProcAddress(m_hPlugin, "gir_name");
			Girder1_gir_description = (void (__stdcall *)(char *, int))GetProcAddress(m_hPlugin, "gir_description");
			Girder1_gir_devicenum = (int (__stdcall *)())GetProcAddress(m_hPlugin, "gir_devicenum");
			Girder1_gir_requested_api = (int (__stdcall *)(int))GetProcAddress(m_hPlugin, "gir_requested_api");
			Girder1_gir_open = (int (__stdcall *)(int, int, int, p_functions))GetProcAddress(m_hPlugin, "gir_open");
			Girder1_gir_close = (int (__stdcall *)())GetProcAddress(m_hPlugin, "gir_close");
			Girder1_gir_config = (void (__stdcall *)())GetProcAddress(m_hPlugin, "gir_config");
			Girder1_gir_start = (int (__stdcall *)())GetProcAddress(m_hPlugin, "gir_start");
			Girder1_gir_stop = (int (__stdcall *)())GetProcAddress(m_hPlugin, "gir_stop");
		}
		else
		{
			nGirderAPIVersion = 0;

			Girder0_identify_device = (void (__stdcall *)(PCHAR, BYTE))GetProcAddress(m_hPlugin, "identify_device");
			Girder0_support_device = (void (__stdcall *)(PCHAR, BYTE))GetProcAddress(m_hPlugin, "support_device");
			Girder0_version_device = (void (__stdcall *)(PCHAR, BYTE))GetProcAddress(m_hPlugin, "version_device");
			Girder0_get_device_num = (int (__stdcall *)())GetProcAddress(m_hPlugin, "get_device_num");
			Girder0_init_device = (void (__stdcall *)(HWND, Girder0_TranslateFunct))GetProcAddress(m_hPlugin, "init_device");
			Girder0_clean_up_device = (void (__stdcall *)())GetProcAddress(m_hPlugin, "clean_up_device");
			Girder0_start_device = (bool (__stdcall *)())GetProcAddress(m_hPlugin, "start_device");
			Girder0_stop_device = (bool (__stdcall *)())GetProcAddress(m_hPlugin, "stop_device");
			Girder0_flush_device = (bool (__stdcall *)())GetProcAddress(m_hPlugin, "flush_device");
			Girder0_config_device = (bool (__stdcall *)())GetProcAddress(m_hPlugin, "config_device");
			Girder0_compare_str = (bool (__stdcall *)(PCHAR, PCHAR))GetProcAddress(m_hPlugin, "compare_str");
			Girder0_learn_device = (void (__stdcall *)(PCHAR))GetProcAddress(m_hPlugin, "learn_device");
		}
	}
	else
		return FALSE;

	m_timeStart = GetTickCount();
	m_timeStart -= 10000;

	if(nGirderAPIVersion == 0 && Girder0_init_device && Girder0_start_device && Girder0_config_device)
	{
		Girder0_init_device(hWnd, MyTranslateFunct);
		return TRUE;
	}
	else if(nGirderAPIVersion  > 0 && Girder1_gir_open && Girder1_gir_start && Girder1_gir_config)
	{
		Girder1_api_functions.size = sizeof(t_functions);
		Girder1_api_functions.parse_reg_string = Girder1_parse_girder_reg;
		Girder1_api_functions.get_link_name = Girder1_get_link_name;
		Girder1_api_functions.set_command = Girder1_set_command;
		Girder1_api_functions.target_enum = Girder1_target_enum;
		Girder1_api_functions.realloc_pchar = Girder1_realloc_pchar;
		Girder1_api_functions.show_osd = Girder1_show_osd;
		Girder1_api_functions.hide_osd = Girder1_hide_osd;
		Girder1_api_functions.start_osd_draw = Girder1_start_osd_draw;
		Girder1_api_functions.stop_osd_draw = Girder1_stop_osd_draw;
		Girder1_api_functions.treepicker_show = Girder1_treepicker_show;
		Girder1_api_functions.register_cb = Girder1_register_cb;
		Girder1_api_functions.i18n_translate = Girder1_i18n_translate;
		Girder1_api_functions.get_osd_settings = Girder1_get_osd_settings;
		Girder1_api_functions.get_osd_font_name = Girder1_get_osd_fontname;
		Girder1_api_functions.gir_malloc = Girder1_gir_malloc;
		Girder1_api_functions.gir_free = Girder1_gir_free;
		Girder1_api_functions.get_int_var = Girder1_get_int_var;
		Girder1_api_functions.get_double_var = Girder1_get_double_var;
		Girder1_api_functions.get_string_var = Girder1_get_string_var;
		Girder1_api_functions.set_int_var = Girder1_set_int_var;
		Girder1_api_functions.set_double_var = Girder1_set_double_var;
		Girder1_api_functions.set_string_var = Girder1_set_string_var;
		Girder1_api_functions.delete_var = Girder1_delete_var;
		Girder1_api_functions.run_parser = Girder1_run_parser;
		Girder1_api_functions.send_event = Girder1_send_event;
		Girder1_api_functions.trigger_command = Girder1_trigger_command;
		Girder1_api_functions.parent_hwnd = hWnd;

//		Girder1_gir_requested_api(1);
		if(Girder1_gir_open(0, 2, 0, &Girder1_api_functions))
			return TRUE;
	}

	CleanUp();
	return FALSE;
}

BOOL CRemoteControl::CleanUp()
{
//	if(nGirderAPIVersion > 0 && Girder1_gir_close)
//		Girder1_gir_close();
//	else if(nGirderAPIVersion == 0 && Girder0_clean_up_device)
//		Girder0_clean_up_device();

	if(m_hPlugin)
	{
		FreeLibrary(m_hPlugin);
		m_hPlugin = NULL;
	}

	return TRUE;
}

BOOL CRemoteControl::GetPluginInfo(CString &szInfo)
{
	char pszText[250];
	if(nGirderAPIVersion == 0)
	{
		if(Girder0_identify_device)
		{
			Girder0_identify_device(pszText, (BYTE)sizeof(pszText));
			szInfo = pszText + CString("\n\r");
		}
		if(Girder0_support_device)
		{
			Girder0_support_device(pszText, (BYTE)sizeof(pszText));
			szInfo += pszText + CString("\n\r");
		}
		if(Girder0_version_device)
		{
			Girder0_version_device(pszText, (BYTE)sizeof(pszText));
			szInfo += pszText + CString("\n\r");
		}
		if(Girder0_get_device_num)
		{
			CString szNumber;
			szNumber.Format("%d", Girder0_get_device_num());
			szInfo += szNumber;
		}
	}
	else if(nGirderAPIVersion > 0)
	{
		if(Girder1_gir_name)
		{
			Girder1_gir_name(pszText, (BYTE)sizeof(pszText));
			szInfo = pszText + CString("\n\r");
		}
		if(Girder1_gir_description)
		{
			Girder1_gir_description(pszText, (BYTE)sizeof(pszText));
			szInfo += pszText + CString("\n\r");
		}
		if(Girder1_gir_version)
		{
			Girder1_gir_version(pszText, (BYTE)sizeof(pszText));
			szInfo += pszText + CString("\n\r");
		}
		if(Girder1_gir_devicenum)
		{
			CString szNumber;
			szNumber.Format("%d", Girder1_gir_devicenum());
			szInfo += szNumber;
		}
	}
	return TRUE;
}

BOOL CRemoteControl::Start()
{
	if(nGirderAPIVersion == 0 && Girder0_start_device)
		return Girder0_start_device();
	else if(nGirderAPIVersion > 0 && Girder1_gir_start)
		return Girder1_gir_start();

	return FALSE;
}

BOOL CRemoteControl::Stop()
{
	if(nGirderAPIVersion == 0 && Girder0_stop_device)
		return Girder0_stop_device();
	else if(nGirderAPIVersion > 0 && Girder1_gir_stop)
		return Girder1_gir_stop();

	return FALSE;
}


BOOL CRemoteControl::ShowConfig()
{
	if(nGirderAPIVersion == 0 && Girder0_config_device)
		return Girder0_config_device();
	else if(nGirderAPIVersion > 0 && Girder1_gir_config)
	{
		Girder1_gir_config();
		return TRUE;
	}

	return FALSE;
}

int WINAPI CRemoteControl::Girder1_send_event(const char * eventstring, void *payload, int len, int device)
{
	::SendMessage(m_hWnd, WM_GIRDER_SEND_EVENT, (WPARAM)0, (LPARAM)eventstring);
	TRACE("%s send_event eventstring %s, payload %08x, len %d, device %d\n",
		CTime::GetCurrentTime().Format("%x %X"), eventstring, payload, len, device);
	return 0;
}

BOOL CRemoteControl::MessageCapture0(UINT message, CString & szRemoteCommand, int & nRepeatCount)
{
	if(nGirderAPIVersion == 0 && message == WM_USER + 1030 && m_lpMapAddress)
	{
		m_timeStop = GetTickCount();
		char pszCommand[256];
		strncpy(pszCommand, (const char *)m_lpMapAddress, 250);
		pszCommand[251] = '\0';
		ReleaseSemaphore(m_hSMSem, 1, NULL); // +1
		if(m_szLastCommand == pszCommand)
			m_nRepeatCount++;

		if(m_timeStop - m_timeStart > 250)
			m_nRepeatCount = 0;
		szRemoteCommand = m_szLastCommand = pszCommand;
		nRepeatCount = m_nRepeatCount;
		TRACE("%s %d (difftime %d = Stop(%d) - Start(%d))\n", pszCommand, nRepeatCount, m_timeStop - m_timeStart, m_timeStop, m_timeStart);
		m_timeStart = m_timeStop;
		return TRUE;
	}
	return FALSE;
}

BOOL CRemoteControl::MessageCapture1(UINT message, CString & szRemoteCommand, int & nRepeatCount)
{
	if(nGirderAPIVersion > 0)
	{
		m_timeStop = GetTickCount();
		if(m_szLastCommand == szRemoteCommand)
			m_nRepeatCount++;

		if(m_timeStop - m_timeStart > 250)
			m_nRepeatCount = 0;
		m_szLastCommand = szRemoteCommand;
		nRepeatCount = m_nRepeatCount;
		TRACE("%s %d (difftime %d = Stop(%d) - Start(%d))\n", szRemoteCommand, nRepeatCount, m_timeStop - m_timeStart, m_timeStop, m_timeStart);
		m_timeStart = m_timeStop;
		return TRUE;
	}
	return FALSE;
}
