//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// RemoteControl.h: interface for the CRemoteControl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_REMOTECONTROL_H__INCLUDED_)
#define AFX_REMOTECONTROL_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WM_GIRDER_SEND_EVENT (WM_USER + 2)

//Girder v2

// the command structure that girder passes to the plugin.
typedef struct s_command 
{
   CRITICAL_SECTION  critical_section;
   PCHAR             name;
   int               actiontype;
   int               actionsubtype;
   PCHAR             svalue1;
   PCHAR             svalue2;
   PCHAR             svalue3;
   int               bvalue1;
   int               bvalue2;
   int               bvalue3;
   int               ivalue1;
   int               ivalue2;
   int               ivalue3;
   int               lvalue1;
   int               lvalue2;
   int               lvalue3;
   void              *binary;
   int               size;
} t_command, *p_command;


// Girder exports a few functions here are the required typedefs
typedef void   (WINAPI *t_target_callback)   (HWND hw, p_command command);
typedef void   (WINAPI *t_set_command)       (p_command command); 
typedef void   (WINAPI *t_target_enum)       (int id, t_target_callback callback); 
typedef void   (WINAPI *t_realloc_pchar)     (PCHAR * old, PCHAR newchar);
typedef void   (WINAPI *t_show_osd)          (int timer);
typedef void   (WINAPI *t_hide_osd)          ();
typedef int    (WINAPI *t_start_osd_draw)    (HWND *hw, HDC *h, int user);
typedef void   (WINAPI *t_stop_osd_draw)     (HDC h);
typedef int    (WINAPI *t_treepicker_show)   (HWND window, int id);
typedef int    (WINAPI *t_event_cb)          (PCHAR event_string, int device, void *payload, int len);
typedef int    (WINAPI *t_register_cb)       (int actionplugin, t_event_cb callback, PCHAR prefix, int device);
typedef int    (WINAPI *t_get_link_name)     (int lvalue, PCHAR szstore, int size);
typedef int    (WINAPI *t_parse_girder_reg)  (PCHAR orig, PCHAR szstore, int size);
typedef int    (WINAPI *t_i18n_translate)    (PCHAR orig, PCHAR szstore, int size);
typedef DWORD  (WINAPI *t_get_osd_settings)  (int setting);
typedef int    (WINAPI *t_get_osd_fontname)  (PCHAR szstore, int size);
typedef int    (WINAPI *t_run_parser)        (PCHAR str, int *error_value);
typedef int    (WINAPI *t_get_int_var)       (PCHAR name);
typedef double (WINAPI *t_get_double_var)    (PCHAR name);
typedef int    (WINAPI *t_get_string_var)    (PCHAR name, PCHAR szstore, int size);
typedef int    (WINAPI *t_set_int_var)       (PCHAR name, int value);
typedef int    (WINAPI *t_set_double_var)    (PCHAR name, double value);
typedef int    (WINAPI *t_set_string_var)    (PCHAR name, PCHAR value);
typedef int    (WINAPI *t_delete_var)        (PCHAR name);
typedef void * (WINAPI *t_gir_malloc)        (int size);
typedef void   (WINAPI *t_gir_free)          (void *data);
typedef int    (WINAPI *t_send_event)        (const char * eventstring, void *payload, int len, int device);
typedef int    (WINAPI *t_trigger_command)   (int command_id);


// this is the version 1 export function structure as passed to gir_open
typedef struct s_functions
{
   DWORD                size;
   t_parse_girder_reg   parse_reg_string;
   t_get_link_name      get_link_name;
   t_set_command        set_command;
   t_target_enum        target_enum;		
   t_realloc_pchar      realloc_pchar;
   t_show_osd           show_osd;
   t_hide_osd           hide_osd;
   t_start_osd_draw     start_osd_draw;
   t_stop_osd_draw      stop_osd_draw;
   t_treepicker_show    treepicker_show;
   t_register_cb        register_cb;
   t_i18n_translate     i18n_translate;
   t_get_osd_settings   get_osd_settings;
   t_get_osd_fontname   get_osd_font_name;
   t_gir_malloc         gir_malloc;
   t_gir_free           gir_free;         
   t_get_int_var        get_int_var;
   t_get_double_var     get_double_var;
   t_get_string_var     get_string_var;
   t_set_int_var        set_int_var;
   t_set_double_var     set_double_var;
   t_set_string_var     set_string_var;
   t_delete_var         delete_var;
   t_run_parser         run_parser;
   t_send_event         send_event;
   t_trigger_command    trigger_command;
   HWND                 parent_hwnd;
} t_functions, *p_functions; 


class CRemoteControl  
{
public:
	CRemoteControl();
	virtual ~CRemoteControl();
	BOOL GetPluginInfo(CString & szInfo);
	BOOL Init(HWND hWnd, LPCTSTR pszPluginPath);
	BOOL MessageCapture0(UINT message, CString & szRemoteCommand, int & nRepeatCount);
	BOOL MessageCapture1(UINT message, CString & szRemoteCommand, int & nRepeatCount);
	BOOL CleanUp();
	BOOL ShowConfig();
	BOOL Start();
	BOOL Stop();

protected:
	static HWND m_hWnd;

	HANDLE m_hMapFile;
	HMODULE m_hPlugin;
	LPVOID m_lpMapAddress;
	HANDLE m_hSMSem;
	DWORD m_timeStart, m_timeStop;
	int m_nRepeatCount;
	CString m_szLastCommand;

	int nGirderAPIVersion;

	void (__stdcall *Girder0_identify_device)(PCHAR Buffer, BYTE Length);
	void (__stdcall *Girder0_support_device)(PCHAR Buffer, BYTE Length);
	void (__stdcall *Girder0_version_device)(PCHAR Buffer, BYTE Length);
	int (__stdcall  *Girder0_get_device_num)();
	typedef PCHAR (__stdcall  *Girder0_TranslateFunct) (PCHAR eng);
	void (__stdcall *Girder0_init_device)(HWND hWindow, Girder0_TranslateFunct p);
	void (__stdcall *Girder0_clean_up_device)();
	bool (__stdcall *Girder0_start_device)();
	bool (__stdcall *Girder0_stop_device)();
	bool (__stdcall *Girder0_flush_device)();
	bool (__stdcall *Girder0_config_device)();
	bool (__stdcall *Girder0_compare_str)(PCHAR Orig, PCHAR Comp);
	void (__stdcall *Girder0_learn_device)(PCHAR old);

	void (__stdcall *Girder1_gir_version)(char *data, int size);
	void (__stdcall *Girder1_gir_name)(char *data, int size);
	void (__stdcall *Girder1_gir_description)(char *data, int size);
	int (__stdcall *Girder1_gir_devicenum)();
	int (__stdcall *Girder1_gir_requested_api)(int max_api);
	int (__stdcall *Girder1_gir_open)(int gir_major_ver, int gir_minor_ver, 
		int gir_micro_ver, p_functions api_functions );
	int (__stdcall *Girder1_gir_close)();
	void (__stdcall *Girder1_gir_config)();
	int (__stdcall *Girder1_gir_start)();
	int (__stdcall *Girder1_gir_stop)();



	static void  WINAPI Girder1_target_callback (HWND hw, p_command command) {}
	static void  WINAPI Girder1_set_command (p_command command) {}
	static void  WINAPI Girder1_target_enum (int id, t_target_callback callback) {}
	static void  WINAPI Girder1_realloc_pchar (PCHAR * old, PCHAR newchar) {}
	static void  WINAPI Girder1_show_osd (int timer) {}
	static void  WINAPI Girder1_hide_osd () {}
	static int   WINAPI Girder1_start_osd_draw (HWND *hw, HDC *h, int user) { return 0; }
	static void  WINAPI Girder1_stop_osd_draw (HDC h) {}
	static int   WINAPI Girder1_treepicker_show (HWND window, int id) { return 0; }
	static int   WINAPI Girder1_event_cb (PCHAR event_string, int device, void *payload, int len) { return 0; }
	static int   WINAPI Girder1_register_cb (int actionplugin, t_event_cb callback, PCHAR prefix, int device) { return 0; }
	static int   WINAPI Girder1_get_link_name (int lvalue, PCHAR szstore, int size) { return 0; }
	static int   WINAPI Girder1_parse_girder_reg (PCHAR orig, PCHAR szstore, int size) { return 0; }
	static int   WINAPI Girder1_i18n_translate (PCHAR orig, PCHAR szstore, int size) { return 0; }
	static DWORD WINAPI Girder1_get_osd_settings (int setting) { return 0; }
	static int   WINAPI Girder1_get_osd_fontname (PCHAR szstore, int size) { return 0; }
	static int   WINAPI Girder1_run_parser (PCHAR str, int *error_value) { return 0; }
	static int   WINAPI Girder1_get_int_var (PCHAR name) { return 0; }
	static double WINAPI Girder1_get_double_var (PCHAR name) { return 0.0; }
	static int   WINAPI Girder1_get_string_var (PCHAR name, PCHAR szstore, int size) { return 0; }
	static int   WINAPI Girder1_set_int_var (PCHAR name, int value) { return 0; }
	static int   WINAPI Girder1_set_double_var (PCHAR name, double value) { return 0; }
	static int   WINAPI Girder1_set_string_var (PCHAR name, PCHAR value) { return 0; }
	static int   WINAPI Girder1_delete_var (PCHAR name) { return 0; }
	static void *WINAPI Girder1_gir_malloc (int size) { return malloc(size); }
	static void  WINAPI Girder1_gir_free (void *data) { free(data); }
	static int   WINAPI Girder1_send_event (const char * eventstring, void *payload, int len, int device);
	static int   WINAPI Girder1_trigger_command (int command_id) { return 0; }

	t_functions Girder1_api_functions;
};

#endif // !defined(AFX_REMOTECONTROL_H__INCLUDED_)
