//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// SettingsLircPropertyPage.cpp : implementation file
//

#include "stdafx.h"
#include "mplayerx.h"
#include "MainFrm.h"
#include "SettingsLircPropertyPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSettingsLircPropertyPage property page

IMPLEMENT_DYNCREATE(CSettingsLircPropertyPage, CPropertyPage)

CSettingsLircPropertyPage::CSettingsLircPropertyPage() : CPropertyPage(CSettingsLircPropertyPage::IDD)
{
	//{{AFX_DATA_INIT(CSettingsLircPropertyPage)
	m_szRemoteCommand = _T("");
	m_fAcceptRepeat = FALSE;
	m_szGirderPluginFilePath = _T("");
	//}}AFX_DATA_INIT

	m_fLircInitialize = m_fLearnAllowed = m_fLircSupport = FALSE;
}

CSettingsLircPropertyPage::~CSettingsLircPropertyPage()
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && m_fLircInitialize)
		pMainFrame->m_RemoteControl.CleanUp();
}

void CSettingsLircPropertyPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSettingsLircPropertyPage)
	DDX_Control(pDX, IDC_BUTTON_GIRDER_PLUGIN_CONFIG, m_wndButtonConfig);
	DDX_Control(pDX, IDC_BUTTON_LEARN, m_wndButtonLearn);
	DDX_Control(pDX, IDC_BUTTON_GIRDER_PLUGIN_BROWSE, m_wndButtonPluginBrowse);
	DDX_Control(pDX, IDC_LIST_COMMAND, m_wndListBoxCommand);
	DDX_Text(pDX, IDC_EDIT_REMOTE_COMMAND, m_szRemoteCommand);
	DDX_Check(pDX, IDC_CHECK_ACCEPT_REPEAT, m_fAcceptRepeat);
	DDX_Text(pDX, IDC_EDIT_GIRDER_PLUGIN_FILE, m_szGirderPluginFilePath);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSettingsLircPropertyPage, CPropertyPage)
	//{{AFX_MSG_MAP(CSettingsLircPropertyPage)
	ON_LBN_SELCHANGE(IDC_LIST_COMMAND, OnSelchangeListCommand)
	ON_BN_CLICKED(IDC_CHECK_ACCEPT_REPEAT, OnCheckAcceptRepeat)
	ON_EN_CHANGE(IDC_EDIT_REMOTE_COMMAND, OnChangeEditRemoteCommand)
	ON_BN_CLICKED(IDC_BUTTON_RESET_MAPPING, OnButtonResetMapping)
	ON_BN_CLICKED(IDC_BUTTON_GIRDER_PLUGIN_BROWSE, OnButtonGirderPluginBrowse)
	ON_EN_CHANGE(IDC_EDIT_GIRDER_PLUGIN_FILE, OnChangeEditGirderPluginFile)
	ON_BN_CLICKED(IDC_BUTTON_LEARN, OnButtonLearn)
	ON_BN_CLICKED(IDC_BUTTON_GIRDER_PLUGIN_CONFIG, OnButtonGirderPluginConfig)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettingsLircPropertyPage message handlers

BOOL CSettingsLircPropertyPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		m_szGirderPluginFilePath = pMainFrame->m_SettingsTemp.m_szGirderPluginFilename = pMainFrame->m_SettingsCurrent.m_szGirderPluginFilename;

		m_wndListBoxCommand.ResetContent();
		pMainFrame->m_SettingsTemp.m_aLircCommandParameters.SetSize(pMainFrame->m_SettingsCurrent.m_aLircCommandParameters.GetSize());
		for(int i = 0; i < pMainFrame->m_SettingsCurrent.m_aLircCommandParameters.GetSize(); i++)
		{
			pMainFrame->m_SettingsTemp.m_aLircCommandParameters[i] = pMainFrame->m_SettingsCurrent.m_aLircCommandParameters[i];
			CString szString = (LPCTSTR)pMainFrame->m_SettingsTemp.m_aLircCommandParameters[i].nWndCommand;
			int nPos = szString.ReverseFind(_T('\n'));
			if(nPos != -1)
				szString = szString.Left(nPos);

			int nItem = m_wndListBoxCommand.AddString(szString);
			m_wndListBoxCommand.SetItemData(nItem, i);
		}
		m_wndListBoxCommand.SetCurSel(0);
		OnSelchangeListCommand();

		m_wndButtonLearn.EnableWindow(pMainFrame->m_fLircSupport);
		m_wndButtonConfig.EnableWindow(pMainFrame->m_fLircInitialize);

		if(pMainFrame->m_fLircInitialize)
			ConnectRemoteControl();
	}
	

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSettingsLircPropertyPage::OnSelchangeListCommand() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		int nItem = m_wndListBoxCommand.GetItemData(m_wndListBoxCommand.GetCurSel());
		m_szRemoteCommand = pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].szRemoteCommandName;
		m_fAcceptRepeat = pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].fAcceptRepeat;
		UpdateData(FALSE);
	}
}

void CSettingsLircPropertyPage::OnCheckAcceptRepeat() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		UpdateData();
		int nItem = m_wndListBoxCommand.GetItemData(m_wndListBoxCommand.GetCurSel());
		pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].fAcceptRepeat = m_fAcceptRepeat;
		pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
	}
}

void CSettingsLircPropertyPage::OnChangeEditRemoteCommand() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		UpdateData();
		int nItem = m_wndListBoxCommand.GetItemData(m_wndListBoxCommand.GetCurSel());
		pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].szRemoteCommandName = m_szRemoteCommand;
		pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
	}
}

void CSettingsLircPropertyPage::OnButtonResetMapping() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		pMainFrame->ResetLircCommand(pMainFrame->m_SettingsTemp.m_aLircCommandParameters);

		int nItem = m_wndListBoxCommand.GetItemData(m_wndListBoxCommand.GetCurSel());
		m_szRemoteCommand = pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].szRemoteCommandName;
		m_fAcceptRepeat = pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].fAcceptRepeat;
		pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
		UpdateData(FALSE);
	}
}

void CSettingsLircPropertyPage::OnButtonGirderPluginBrowse() 
{
	CString szFilter;
	szFilter.LoadString(IDS_STRING_OPENGIRDER_FILTER);

	CFileDialog FileDialog(TRUE, _T("*.dll"), NULL,
		OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING,
		szFilter, this);

	TCHAR pszOriginalPath[2048];
	pszOriginalPath[0] = '\0';

	if(m_szGirderPluginFilePath == _T(""))
	{
		CString szExecutable = AfxGetExecutableName();
		int nPos = szExecutable.ReverseFind('\\');
		if(nPos != -1)
			strncpy(pszOriginalPath, szExecutable.Left(nPos + 1), sizeof(pszOriginalPath));
		FileDialog.m_ofn.lpstrInitialDir  = pszOriginalPath;
	}
	else
		FileDialog.m_ofn.lpstrFile = (LPTSTR)(LPCTSTR)m_szGirderPluginFilePath;

	if(FileDialog.DoModal() == IDOK)
	{
		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		if(pMainFrame)
		{
			pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
			pMainFrame->m_SettingsTemp.m_szGirderPluginFilename = m_szGirderPluginFilePath = FileDialog.GetPathName();
			UpdateData(FALSE);
			ConnectRemoteControl();
		}
	}
}

void CSettingsLircPropertyPage::OnChangeEditGirderPluginFile() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		UpdateData();
		pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
		pMainFrame->m_SettingsTemp.m_szGirderPluginFilename = m_szGirderPluginFilePath;
	}
}

BOOL CSettingsLircPropertyPage::ConnectRemoteControl()
{
	m_fLircInitialize = FALSE;
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame)
	{
		m_fLircSupport = FALSE;
		if(m_szGirderPluginFilePath != _T(""))
		{
			if(pMainFrame->m_RemoteControl.Init(m_hWnd, m_szGirderPluginFilePath))
			{
				m_fLircInitialize = TRUE;
				if(pMainFrame->m_RemoteControl.Start())
					m_fLircSupport = TRUE;
				else
				{
					CString szString;
					szString.LoadString(IDS_STRING_GIRDER_CANNOT_START);
					AfxMessageBoxEx(szString + m_szGirderPluginFilePath);
				}
			}
			else
			{
				CString szString;
				szString.LoadString(IDS_STRING_GIRDER_CANNOT_INITIALIZE);
				AfxMessageBoxEx(szString + m_szGirderPluginFilePath);
			}
		}
	}

	m_wndButtonLearn.EnableWindow(m_fLircSupport);
	m_wndButtonConfig.EnableWindow(m_fLircInitialize);

	return m_fLircSupport;
}

void CSettingsLircPropertyPage::OnButtonLearn() 
{
	m_fLearnAllowed = m_wndButtonLearn.GetCheck();
}

LRESULT CSettingsLircPropertyPage::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	CString szRemoteCommand;
	int nRepeatCount;
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && (message == WM_GIRDER_SEND_EVENT || pMainFrame->m_RemoteControl.MessageCapture0(message, szRemoteCommand, nRepeatCount)))
	{
		if(message == WM_GIRDER_SEND_EVENT)
		{
			szRemoteCommand = (LPCTSTR)lParam;
			nRepeatCount = wParam;
			pMainFrame->m_RemoteControl.MessageCapture1(message, szRemoteCommand, nRepeatCount);
		}

		if(m_fLearnAllowed)
		{
			UpdateData();
			int nItem = m_wndListBoxCommand.GetItemData(m_wndListBoxCommand.GetCurSel());
			m_szRemoteCommand = pMainFrame->m_SettingsTemp.m_aLircCommandParameters[nItem].szRemoteCommandName = szRemoteCommand;
			pMainFrame->m_SettingsTemp.m_fLircDirty = TRUE;
			UpdateData(FALSE);
		}
		return TRUE;
	}

	return CPropertyPage::WindowProc(message, wParam, lParam);
}

void CSettingsLircPropertyPage::OnButtonGirderPluginConfig() 
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
	if(pMainFrame && m_fLircInitialize)
		pMainFrame->m_RemoteControl.ShowConfig();
}
