//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// SubtitleManager.cpp: implementation of the CSubtitleManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <dshow.h>
#include "SubtitleManager.h"

#define TIME_CONVERTER(x) (m_bTimeBased ? x : ((__int64)((double)x / m_dFramePerSecond)) * 10000000)

CArray<CSubtitle *, CSubtitle *> g_aSubtitle;

CSubtitle::CSubtitle(CMediaTime & MediaTime) :
		m_MediaTime(MediaTime)
{
	m_enSubtitleState = SubtitleStateUnkown;
	m_dFramePerSecond = 23.97; // Framerates: 23.97 FPS, 23.976 FPS, 25 FPS, 29.97 FPS for MicroDVD and SAMI)
	m_dDelay = 0.0;
	m_rect = CRect(10, 10, 300, 100);
	m_bSubtitleDisplayed = FALSE;
	m_nTimerCookie = NULL;
	m_bIsPlaying = FALSE;
	m_bTimeBased = TRUE;
}

CSubtitle::~CSubtitle()
{

}

EnumSubtitleType CSubtitle::GetTypeFromFilename(LPCTSTR pszFilename)
{
	CString szFilename(pszFilename);
	int nPos = szFilename.ReverseFind(_T('.'));
	CString szExtension;
	if(nPos != -1)
	{
		szExtension = szFilename.Mid(nPos + 1);

		if(szExtension == _T("txt"))
			return SubtitleTypeMicroDVD;
		if(szExtension == _T("srt"))
			return SubtitleTypeSubRip;
		if(szExtension == _T("sub"))
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				CString szLine;
				int nMax = 5; // Read nMax lines from the begining of the file
				while(file.ReadString(szLine) && nMax >= 0)
				{
					if(szLine != _T(""))
					{
						//{704}{767}{y:i}Tout ceci n'tait-il qu'un rve ?
						int nTime1, nTime2;
						if(_stscanf(szLine, _T("{%d}{%d}"), &nTime1, &nTime2) == 2)
							return SubtitleTypeMicroDVD;
						nMax--;
					}
				}
				file.Close();
			}

			return SubtitleTypeSubViewer;
//			return SubtitleTypeSubViewer2;
		}
		if(szExtension == _T("smi"))
			return SubtitleTypeSAMI;
		if(szExtension == _T("ssa"))
			return SubtitleTypeSubStationAlpha;
	}

	return SubtitleTypeUnkown;
}

BOOL CSubtitle::Load(LPCTSTR pszFilename)
{
	switch(GetTypeFromFilename(pszFilename))
	{
	case SubtitleTypeUnkown:
		break;
	case SubtitleTypeMicroDVD:
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				CString szLine;
				while(file.ReadString(szLine))
				{
					if(szLine != _T(""))
					{
						int nPos1 = szLine.Find(_T('}'));
						if(nPos1 != -1)
						{
							int nPos2 = szLine.Find(_T('}'), nPos1 + 1);
							if(nPos2 != -1)
							{
								CString szTime1, szTime2;
								szTime1 = szLine.Left(nPos1 + 1);
								szTime1.TrimLeft();
								szTime1.TrimRight();
								szTime1 = szTime1.Mid(1);
								szTime1 = szTime1.Left(szTime1.GetLength() - 1);
								szTime2 = szLine.Mid(nPos1 + 1, nPos2 - nPos1);
								szTime2.TrimLeft();
								szTime2.TrimRight();
								szTime2 = szTime2.Mid(1);
								szTime2 = szTime2.Left(szTime2.GetLength() - 1);

								int nStart = atoi(szTime1);
								int nStop = atoi(szTime2);
								int nCount = m_aSubtitle.GetSize();
								m_aSubtitle.SetSize(nCount + 1);

								REFERENCE_TIME t1, t2;

								t1 = nStart; //(__int64)((nStart / m_dFramePerSecond) * 10000000);
								t2 = nStop; //(__int64)((nStop / m_dFramePerSecond) * 10000000);
								if(t1 >= t2)
									t2 = t1 + 1;

								m_aSubtitle[nCount].m_timeShow = t1;
								m_aSubtitle[nCount].m_timeHide = t2;
								m_aSubtitle[nCount].m_szLine = szLine.Mid(nPos2 + 1);
								m_aSubtitle[nCount].m_szLine.Replace(_T("|"), _T("\n"));
							}
						}
					}
				}
				file.Close();
				m_bTimeBased = FALSE;
				return TRUE;
			}
		}
		break;
	case SubtitleTypeSubRip:
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				int nCount = 0, nLineIndex = 0;
				CString szLine;
				while(file.ReadString(szLine))
				{
					if(szLine != _T(""))
					{
						if(nLineIndex == 0)
						{
							nCount = m_aSubtitle.GetSize();
							m_aSubtitle.SetSize(nCount + 1);
						}
						else if(nLineIndex == 1)
						{
							szLine.TrimLeft();
							szLine.TrimRight();
							int nHour1, nMinutes1, nSecondes1, nMilliemes1;
							int nHour2, nMinutes2, nSecondes2, nMilliemes2;
							if(_stscanf(szLine, _T("%d:%d:%d,%d --> %d:%d:%d,%d"), 
								&nHour1, &nMinutes1, &nSecondes1, &nMilliemes1,
								&nHour2, &nMinutes2, &nSecondes2, &nMilliemes2) == 8)
							{
								REFERENCE_TIME t1, t2;

								t1 = ((((__int64)(nHour1 * 60 + nMinutes1) * 60 + nSecondes1) * 1000 + nMilliemes1) * 10000);
								t2 = ((((__int64)(nHour2 * 60 + nMinutes2) * 60 + nSecondes2) * 1000 + nMilliemes2) * 10000);
								if(t1 >= t2)
									t2 = t1 + 1;
								m_aSubtitle[nCount].m_timeShow = t1;
								m_aSubtitle[nCount].m_timeHide = t2;
							}
						}
						else if(nLineIndex == 2)
						{
							m_aSubtitle[nCount].m_szLine = szLine;
						}
						else if(nLineIndex > 2)
						{
							m_aSubtitle[nCount].m_szLine += "\n" + szLine;
						}
						nLineIndex++;
					}
					else
					{
						nLineIndex = 0;
					}
				}
				file.Close();
				m_bTimeBased = TRUE;
				return TRUE;
			}
		}
		break;
	case SubtitleTypeSubViewer:
	case SubtitleTypeSubViewer2:
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				int nCount = 0, nLineIndex = 0;
				CString szLine;
				while(file.ReadString(szLine))
				{
					if(szLine != _T(""))
					{
						int nPos = szLine.Find(_T("[DELAY]"));
						if(nPos != -1)
						{
							CString szDelay = szLine.Mid(nPos + 7);
							m_dDelay = atof(szDelay);
							continue;
						}

						if(nLineIndex == 0)
						{
							szLine.TrimLeft();
							szLine.TrimRight();
							int nHour1, nMinutes1, nSecondes1, nMilliemes1;
							int nHour2, nMinutes2, nSecondes2, nMilliemes2;
							if(_stscanf(szLine, _T("%d:%d:%d.%d,%d:%d:%d.%d"), 
								&nHour1, &nMinutes1, &nSecondes1, &nMilliemes1,
								&nHour2, &nMinutes2, &nSecondes2, &nMilliemes2) == 8)
							{
								nCount = m_aSubtitle.GetSize();
								m_aSubtitle.SetSize(nCount + 1);

								REFERENCE_TIME t1, t2;

								t1 = ((((__int64)(nHour1 * 60 + nMinutes1) * 60 + nSecondes1) * 1000 + nMilliemes1) * 10000);
								t2 = ((((__int64)(nHour2 * 60 + nMinutes2) * 60 + nSecondes2) * 1000 + nMilliemes2) * 10000);
								if(t1 >= t2)
									t2 = t1 + 1;
								m_aSubtitle[nCount].m_timeShow = t1;
								m_aSubtitle[nCount].m_timeHide = t2;

								nLineIndex++;
							}
							else
								nLineIndex = 0;
						}
						else if(nLineIndex == 1)
						{
							m_aSubtitle[nCount].m_szLine = szLine;
							m_aSubtitle[nCount].m_szLine.Replace(_T("[br]"), _T("\n"));
						}
					}
					else
					{
						nLineIndex = 0;
					}
				}
				file.Close();
				m_bTimeBased = TRUE;
				return TRUE;
			}
		}
		break;
	case SubtitleTypeSAMI:
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				int nCount = 0;
				BOOL bSubtitleBegun = FALSE;
				CString szLine;
				while(file.ReadString(szLine))
				{
					if(szLine != _T(""))
					{
						CString szLineLower = szLine;
						szLineLower.MakeLower();
						int nPos = szLineLower.Find(_T("<sync start="));
						if(nPos != -1)
						{
							int nPos = szLineLower.Find(_T("="));
							if(nPos != -1)
							{
								int nPos2 = szLineLower.Find(_T(">"));
								if(nPos2 != -1)
								{
									CString szFrame = szLine.Mid(nPos + 1, nPos2 - (nPos + 1));
									bSubtitleBegun = TRUE;

									int nTime = atoi(szFrame);
									nCount = m_aSubtitle.GetSize();
									m_aSubtitle.SetSize(nCount + 1);
									m_aSubtitle[nCount].m_szLine = szLine.Mid(nPos2 + 1);
									m_aSubtitle[nCount].m_timeShow = ((__int64)nTime) * 10000;
									if(nCount > 0)
									{
										m_aSubtitle[nCount - 1].m_timeHide = ((__int64)(nTime - 1)) * 10000;;
										CString & szOldLine = m_aSubtitle[nCount - 1].m_szLine;
										szOldLine.Replace(_T("<br>"), _T("\n"));
										szOldLine.Replace(_T("<BR>"), _T("\n"));
										szOldLine.Replace(_T("&nbsp;"), _T(" "));
										szOldLine.Replace(_T("&NBSP;"), _T(" "));

										// Remove HTML tags
										int nPos1 = -1;
										while((nPos1 = szOldLine.Find(_T('<'))) != -1)
										{
											if(nPos1 != -1)
											{
												nPos2 = szOldLine.Find(_T('>'), nPos1);
												if(nPos1 != -1)
													szOldLine = szOldLine.Left(nPos1) + szOldLine.Mid(nPos2 + 1);
											}
										}
									}
								}
							}
						}
						else if(bSubtitleBegun)
						{
							szLine.TrimLeft();
							szLine.TrimRight();
							m_aSubtitle[nCount].m_szLine += " " + szLine;
						}
					}
				}
				if(nCount)
					m_aSubtitle[nCount].m_timeHide = m_aSubtitle[nCount].m_timeShow + 1;

				file.Close();
				m_bTimeBased = TRUE;
				return TRUE;
			}
		}
		break;
	case SubtitleTypeSubStationAlpha:
		{
			CStdioFile file;
			if(file.Open(pszFilename, CFile::modeRead | CFile::typeText | CFile::shareDenyNone))
			{
				int nCount = 0;
				CString szLine;
				while(file.ReadString(szLine))
				{
					if(szLine != _T(""))
					{
						szLine.TrimLeft();
						szLine.TrimRight();
						
						int nPos = szLine.Find(_T("Dialogue:"));
						if(nPos != -1)
						{
							CString szTime, szText;
							nPos = szLine.Find(_T(","));
							if(nPos != -1)
							{
								szLine = szLine.Mid(nPos + 1);
								nPos = szLine.Find(_T(","));
								if(nPos != -1)
								{
									nPos = szLine.Find(_T(","), nPos + 1);
									if(nPos != -1)
									{
										szTime = szLine.Left(nPos);
										szLine = szLine.Mid(nPos + 1);

										nPos = -1;
										for(int i = 0; i < 6;i++)
										{
											nPos = szLine.Find(_T(","), nPos + 1);
											if(nPos == -1)
												break;
										}
										if(nPos != -1)
										{
											szText = szLine.Mid(nPos + 1);
										}
									}
								}
							}
							int nHour1, nMinutes1, nSecondes1, nMilliemes1;
							int nHour2, nMinutes2, nSecondes2, nMilliemes2;
							if(_stscanf(szTime, _T("%d:%d:%d.%d,%d:%d:%d.%d"), 
								&nHour1, &nMinutes1, &nSecondes1, &nMilliemes1,
								&nHour2, &nMinutes2, &nSecondes2, &nMilliemes2) == 8)
							{
								nCount = m_aSubtitle.GetSize();
								m_aSubtitle.SetSize(nCount + 1);
								
								REFERENCE_TIME t1, t2;
								
								t1 = ((((__int64)(nHour1 * 60 + nMinutes1) * 60 + nSecondes1) * 1000 + nMilliemes1) * 10000);
								t2 = ((((__int64)(nHour2 * 60 + nMinutes2) * 60 + nSecondes2) * 1000 + nMilliemes2) * 10000);
								if(t1 >= t2)
									t2 = t1 + 1;
								m_aSubtitle[nCount].m_timeShow = t1;
								m_aSubtitle[nCount].m_timeHide = t2;
								m_aSubtitle[nCount].m_szLine = szText;
								m_aSubtitle[nCount].m_szLine.Replace(_T("\\n"), _T("\n"));
								m_aSubtitle[nCount].m_szLine.Replace(_T("\\N"), _T("\n"));
							}
						}
					}
				}
				file.Close();
				m_bTimeBased = TRUE;
				return TRUE;
			}
		}
		break;
	}

	return FALSE;
}

BOOL CSubtitle::Start(int nCurrentSubtitleIndex)
{
	TRACE(_T("CSubtitle::Start\n"));

	int nSubtitleCount = m_aSubtitle.GetSize();
	if(nSubtitleCount)
	{
		if(m_nTimerCookie)
		{
			KillTimer(NULL, m_nTimerCookie);
			m_nTimerCookie = 0;
		}

		m_nCurrentSubtitleIndex = nCurrentSubtitleIndex;
		REFERENCE_TIME time2 = 0;

		if(!m_MediaTime.GetCurrentPosition(m_timeCurrent))
			return FALSE;

		m_timeCurrent += (__int64)(m_dDelay * 10000000.0);

		for(int i = 0; i < nSubtitleCount; i++)
		{
			time2 = TIME_CONVERTER(m_aSubtitle[i].m_timeHide);
			if(time2 > m_timeCurrent && i >= m_nCurrentSubtitleIndex)
			{
				m_nCurrentSubtitleIndex = i;
				break;
			}
		}

		TRACE(_T("CSubtitle::Start m_nCurrentSubtitleIndex = %d\n"), m_nCurrentSubtitleIndex);

		if(m_nCurrentSubtitleIndex != -1 && m_nCurrentSubtitleIndex < nSubtitleCount)
		{
			m_bIsPlaying = TRUE;

			// Show right now
			if((m_timeCurrent > TIME_CONVERTER(m_aSubtitle[m_nCurrentSubtitleIndex].m_timeShow)))
			{
				m_enSubtitleState = SubtitleStateWaitShow;
				Event();
			}
			else // Show soon
			{
				UINT nTimeOffset = (UINT)((TIME_CONVERTER(m_aSubtitle[m_nCurrentSubtitleIndex].m_timeShow) - m_timeCurrent) / 10000);
				m_enSubtitleState = SubtitleStateWaitShow;
				TRACE(_T("Show Subtitle in %d seconds\n"), nTimeOffset / 1000);
				m_timeCurrent = 0;
				if(m_nTimerCookie = SetTimer(NULL, NULL, nTimeOffset, CSubtitleManager::TimerProc))
					return TRUE;
				m_enSubtitleState = SubtitleStateUnkown;
			}
		}
		else
		{
			TRACE(_T("CSubtitle::Start Error\n"));
		}
	}
	m_bIsPlaying = FALSE;
	return FALSE;
}

void CSubtitle::Event()
{
	TRACE(_T("CSubtitle::Event m_enSubtitleState = %d, m_nCurrentSubtitleIndex = %d\n"), m_enSubtitleState, m_nCurrentSubtitleIndex);

	if(m_nTimerCookie)
	{
		KillTimer(NULL, m_nTimerCookie);
		m_nTimerCookie = 0;
	}
	if(m_nCurrentSubtitleIndex != -1)
	{
		switch(m_enSubtitleState)
		{
		case SubtitleStateUnkown:
			break;
		case SubtitleStateWaitShow:
			{
				TRACE(_T("CSubtitle Show %s\n"), m_aSubtitle[m_nCurrentSubtitleIndex].m_szLine);

				m_bSubtitleDisplayed = TRUE;
				if(m_hWnd)
					SendMessage(m_hWnd, WM_PAINT_SUBTITLE, 0, (LPARAM)this);

				if(m_timeCurrent == 0)
					m_timeCurrent = TIME_CONVERTER(m_aSubtitle[m_nCurrentSubtitleIndex].m_timeShow);
				UINT nTimeOffset = (UINT)((TIME_CONVERTER(m_aSubtitle[m_nCurrentSubtitleIndex].m_timeHide) - m_timeCurrent) / 10000);
				m_enSubtitleState = SubtitleStateWaitHide;

				TRACE(_T("Hide Subtitle in %d seconds\n"), nTimeOffset / 1000);
				if(m_nTimerCookie = SetTimer(NULL, NULL, nTimeOffset, CSubtitleManager::TimerProc))
					break;

				m_enSubtitleState = SubtitleStateUnkown;
			}
			break;
		case SubtitleStateWaitHide:
			{
				TRACE(_T("CSubtitle Hide %s\n"), m_aSubtitle[m_nCurrentSubtitleIndex].m_szLine);

				m_bSubtitleDisplayed = FALSE;
				if(m_hWnd)
					SendMessage(m_hWnd, WM_PAINT_SUBTITLE, 0, (LPARAM)this);

				if(++m_nCurrentSubtitleIndex < m_aSubtitle.GetSize())
					Start(m_nCurrentSubtitleIndex);
				else
					m_enSubtitleState = SubtitleStateUnkown;
			}
			break;
		}
	}
}

BOOL CSubtitle::Pause()
{
	TRACE(_T("CSubtitle::Pause\n"));

	KillTimer(NULL, m_nTimerCookie);
	m_bIsPlaying = FALSE;

	return TRUE;
}

BOOL CSubtitle::Stop()
{
	TRACE(_T("CSubtitle::Stop\n"));

	KillTimer(NULL, m_nTimerCookie);
	if(m_hWnd && m_bSubtitleDisplayed)
	{
		m_bSubtitleDisplayed = FALSE;
		SendMessage(m_hWnd, WM_PAINT_SUBTITLE, 0, (LPARAM)this);
	}
	m_bIsPlaying = FALSE;
	return TRUE;
}

BOOL CSubtitle::GetCurrentSutitle(CString & szSutitle)
{
	if(m_bSubtitleDisplayed && m_nCurrentSubtitleIndex != -1 && m_nCurrentSubtitleIndex < m_aSubtitle.GetSize())
	{
		szSutitle = m_aSubtitle[m_nCurrentSubtitleIndex].m_szLine;
		return TRUE;
	}
	return FALSE;
}

double CSubtitle::GetFramePerSecond()
{
	return m_dFramePerSecond;
}

double CSubtitle::GetDelay()
{
	return m_dDelay;
}

void CSubtitle::SetFramePerSecond(double dFramePerSecond)
{
	if(m_dFramePerSecond != dFramePerSecond)
	{
		m_dFramePerSecond = dFramePerSecond;
		if(m_bIsPlaying)
			Start();
	}
}

void CSubtitle::SetDelay(double dDelay)
{
	if(m_dDelay != dDelay)
	{
		m_dDelay = dDelay;
		if(m_bIsPlaying)
			Start();
	}
}


VOID CALLBACK CSubtitleManager::TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
	int nCount = g_aSubtitle.GetSize();
	for(int i = 0; i < nCount; i++)
	{
		CSubtitle * pSubtitle = g_aSubtitle[i];
		if(pSubtitle && pSubtitle->m_nTimerCookie == idEvent)
			pSubtitle->Event();
	}
}




CSubtitleManager::CSubtitleManager(CMediaTime & MediaTime) :
	m_MediaTime(MediaTime), m_aSubtitle(g_aSubtitle)
{
	m_hWnd = NULL;
	m_bIsPlaying = FALSE;
	m_nCurrentSubtitleIndex = -1;
}

CSubtitleManager::~CSubtitleManager()
{

}

void CSubtitleManager::Clear()
{
	Stop();
	m_nCurrentSubtitleIndex = -1;

	for(int i = 0; i < m_aSubtitle.GetSize(); i++)
	{
		CSubtitle * pSubtitle = m_aSubtitle[i];
		if(pSubtitle)
			delete pSubtitle;
	}
	m_aSubtitle.RemoveAll();
}

BOOL CSubtitleManager::AddSubtitle(LPCTSTR pszFilename, UINT * pnSubtitleIndex)
{
	CSubtitle * pSubtitle = new CSubtitle(m_MediaTime);
	if(pSubtitle)
	{
		pSubtitle->SetHWNDCallback(m_hWnd);
		if(pSubtitle->Load(pszFilename))
		{
			UINT nSubtitleIndex = m_aSubtitle.Add(pSubtitle);
			if(pnSubtitleIndex)
				*pnSubtitleIndex = nSubtitleIndex;
			return TRUE;
		}
		delete pSubtitle;
	}
	return FALSE;
}

BOOL CSubtitleManager::Start()
{
	m_bIsPlaying = TRUE;
	if(m_nCurrentSubtitleIndex >= 0 && m_nCurrentSubtitleIndex < m_aSubtitle.GetSize() && m_aSubtitle[m_nCurrentSubtitleIndex])
		return m_aSubtitle[m_nCurrentSubtitleIndex]->Start();
	return FALSE;
}

BOOL CSubtitleManager::Pause()
{
	m_bIsPlaying = FALSE;
	if(m_nCurrentSubtitleIndex >= 0 && m_nCurrentSubtitleIndex < m_aSubtitle.GetSize() && m_aSubtitle[m_nCurrentSubtitleIndex])
		return m_aSubtitle[m_nCurrentSubtitleIndex]->Pause();
	return FALSE;
}

BOOL CSubtitleManager::Stop()
{
	m_bIsPlaying = FALSE;
	if(m_nCurrentSubtitleIndex >= 0 && m_nCurrentSubtitleIndex < m_aSubtitle.GetSize() && m_aSubtitle[m_nCurrentSubtitleIndex])
		return m_aSubtitle[m_nCurrentSubtitleIndex]->Stop();
	return FALSE;
}

void CSubtitleManager::SetHWNDCallback(HWND hWnd)
{
	m_hWnd = hWnd;
	int nCount = m_aSubtitle.GetSize();
	for(int i = 0; i < nCount; i++)
		m_aSubtitle[i]->SetHWNDCallback(hWnd);
}

CSubtitle * CSubtitleManager::Select(int nSubtitleIndex)
{
	if(m_nCurrentSubtitleIndex != nSubtitleIndex)
	{
		if(m_nCurrentSubtitleIndex >= 0 && m_nCurrentSubtitleIndex < m_aSubtitle.GetSize() && m_aSubtitle[m_nCurrentSubtitleIndex])
			m_aSubtitle[m_nCurrentSubtitleIndex]->Stop();
		m_nCurrentSubtitleIndex = nSubtitleIndex;
		if(nSubtitleIndex >= 0 && nSubtitleIndex < m_aSubtitle.GetSize())
		{
			if(m_bIsPlaying)
				Start();

			return m_aSubtitle[m_nCurrentSubtitleIndex];
		}
		else
		{
			m_bIsPlaying = FALSE;
		}
	}
	return NULL;
}
