//    MPlayerX - Multimedia file player application
//    Copyright (C) 2002  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// SubtitleManager.h: interface for the CSubtitleManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SUBTITLEMANAGER_H__INCLUDED_)
#define AFX_SUBTITLEMANAGER_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MediaTime.h"

#define WM_PAINT_SUBTITLE WM_USER + 17

typedef struct SubtitleElement
{
	REFERENCE_TIME m_timeShow;
	REFERENCE_TIME m_timeHide;

	CString m_szLine;
} SubtitleElement;

typedef enum EnumSubtitleType
{
	SubtitleTypeUnkown,
	SubtitleTypeMicroDVD,
	SubtitleTypeSubRip,
	SubtitleTypeSubViewer,
	SubtitleTypeSubViewer2,
	SubtitleTypeSAMI,
	SubtitleTypeSubStationAlpha
} EnumSubtitleType;

typedef enum EnumSubtitleState
{
	SubtitleStateUnkown,
	SubtitleStateWaitShow,
	SubtitleStateWaitHide
} EnumSubtitleState;

class CSubtitle
{
public:
	CSubtitle(CMediaTime & MediaTime);
	virtual ~CSubtitle();

	static EnumSubtitleType GetTypeFromFilename(LPCTSTR pszFilename);
	BOOL Load(LPCTSTR pszFilename);
	BOOL Start(int nCurrentSubtitleIndex = -1);
	BOOL Pause();
	BOOL Stop();
	void Event();
	void SetRect(const CRect & rect) { m_rect = rect; }
	void GetRect(CRect & rect) { rect = m_rect; }
	void SetHWNDCallback(HWND hWnd) { m_hWnd = hWnd; }
	double GetFramePerSecond();
	void SetFramePerSecond(double dFramePerSecond);
	double GetDelay();
	void SetDelay(double dDelay);
	BOOL GetTimeBased() { return m_bTimeBased; }

	BOOL GetCurrentSutitle(CString & szSutitle);

	UINT m_nTimerCookie;
protected:
	BOOL m_bTimeBased;
	BOOL m_bIsPlaying;
	double m_dFramePerSecond;
	double m_dDelay;
	BOOL m_bSubtitleDisplayed;
	HWND m_hWnd;
	CRect m_rect;
	EnumSubtitleState m_enSubtitleState;
	CMediaTime & m_MediaTime;
	int m_nCurrentSubtitleIndex;
	CArray<SubtitleElement, SubtitleElement&> m_aSubtitle;
	REFERENCE_TIME m_timeCurrent;
};

class CSubtitleManager  
{
public:
	CSubtitleManager(CMediaTime & MediaTime);
	virtual ~CSubtitleManager();

	void SetHWNDCallback(HWND hWnd);

	BOOL AddSubtitle(LPCTSTR pszFilename, UINT * pnSubtitleIndex = NULL);
	BOOL Start();
	BOOL Pause();
	BOOL Stop();
	CSubtitle * Select(int nSubtitleIndex);
	void Clear();


	int GetSubtitleCount() { return m_aSubtitle.GetSize(); }
	CSubtitle * GetSubtitle(int nIndex) { return (nIndex >= 0 && nIndex < GetSubtitleCount() ? m_aSubtitle[nIndex] : NULL);}

	static VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);

protected:
	CArray<CSubtitle *, CSubtitle *> & m_aSubtitle;
	HWND m_hWnd;
	CMediaTime & m_MediaTime;
	BOOL m_bIsPlaying;
	int m_nCurrentSubtitleIndex;
};

#endif // !defined(AFX_SUBTITLEMANAGER_H__INCLUDED_)
